/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.api.graph.datasource;

import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.BladeFilter;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.supervisor.api.ClifException;

/**
 * The Interface RawDataRead.
 * @author Jordan BRUNIER
 * @author Gregory CALONNIER
 * @author Olivier LIU
 */
public interface RawDataRead {

    //  Fractal interface name
    /** The Constant DATA_SUPPLIER. */
    static public final String RAW_DATA_READ = "Raw data read";
    
    
    /**
     * Get the tests' names.
     * 
     * @return an array that contains the tests' names
     */
    public String[] getTestsNames();
    
    
    /**
     * Get the baldes' ids.
     * 
     * @param testName the test run name
     * 
     * @return an array that contains the blades ids of the given test
     */
    public String[] getBladesIds(String testName);
    
    
    /**
     * Get the events labels.
     * 
     * @param testName the test run name
     * @param bladeId the blade identifier
     * 
     * @return an array that contains the events label of the given blade
     */
    public String[] getEventsTypeLabels(String testName, String bladeId);
    
    
    /**
     * Get the events fields labels.
     * 
     * @param testName the test run name
     * @param bladeId the blade identifier from the given test run
     * @param eventTypeLabel the label of the event type
     * 
     * @return an array that contains the labels of the given event type
     */
    public String[] getEventFieldLabels(String testName, String bladeId, String eventTypeLabel);
    
    
    /**
     * Get the values for the X and Y fields.
     * 
     * @param testName the test run name
     * @param bladeId the blade identifier from the given test run
     * @param eventTypeLabel the label of the event type
     * @param XFieldLabel the label of the X field
     * @param YFieldLabel the label of the Y field
     * @param start the start
     * @param end the end
     * @param count the count
     * 
     * @return a two dimension array that contains the values of the given
     *         fields
     */
    public Object[][] getFieldsValues(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start,int end, int count);
    
    
    
    /**
     * Get the values for the X and Y fields.
     * 
     * @param testName
     * @param bladeId
     * @param eventTypeLabel
     * @param XFieldLabel
     * @param YFieldLabel
     * @param start
     * @param end
     * @param count
     * @return a two dimension array that contains the values of the given
     *         fields
     */
    public Object[][] getAllFieldsValues(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count);
   
    
    /**
     * Count values for the giving field label.
     * 
     * @param testName the test run name
     * @param bladeId the blade id
     * @param eventTypeLabel the event type label
     * @param YFieldLabel the Y field label
     * 
     * @return the number of values
     */
    public long countValues(String testName, String bladeId, String eventTypeLabel, String YFieldLabel);

    
    /**
     * Gets the max time for.
     * 
     * @param testName the test name
     * @param bladeId the blade id
     * 
     * @return the max time for
     */
    public long getMaxTimeFor(String testName, String bladeId);
    
    
    /**
     * Get the tests' names.
     * 
     * @return an array that contains the tests' names
     * @throws ClifException 
     */
    public BladeDescriptor[] getTestPlan(String testName, BladeFilter filter) throws ClifException;

    
    /**
     * Get the blade events 
     * 
     * @return an array that contains the blade event
     */
    public BladeEvent[] getBladeEvents(String testName,	String bladeId, String eventTypeLabel,EventFilter filter, long fromIndex,int count)throws ClifException;
    	   
    
    /**
     * Life-cycle Running 
     * 
     * @param testName a test name
     * @param bladeId a blade ID
     * @return the life-cycle running time 
     */
    public long getLifecycleRunning(String testName, String bladeId);
    
    
    /**
     * Life-cycle Completed
     * 
     * @param testName a test name
     * @param bladeId a blade ID
     * @return the life-cycle running time 
     */
    public long getLifecycleCompleted(String testName, String bladeId); 


    /**
     * Get the values for the X and Y fields.
     * 
     * @param bladeEventList
     * @param XFieldLabel
     * @param YFieldLabel
     * @return a two dimension array that contains the values of the given
     *         fields
     */
    public Object[][] getBladeEventsValues(BladeEvent[] bladeEventList, String XFieldLabel, String YFieldLabel);
    
    
    /**
     * Get the blade descriptor from a test and a blade filter
     * 
     * @param testName
     * @param filter
     * @return an one dimension array that contains the blade descriptor of the given
     *         fields
     */
    public BladeDescriptor[] getBladeDescriptor(String testName, BladeFilter filter);
}
