/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2008 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */

package org.ow2.clif.analyze.lib.graph;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;
import org.jfree.chart.JFreeChart;
import org.ow2.clif.storage.api.BladeEvent;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.ow2.clif.analyze.lib.graph.StatisticDataImpl;
import org.ow2.clif.analyze.statistics.util.data.ListOfLong;

/**
 * ChartData class : contains all the method to create a chart
 * @author Olivier LIU
 */
public class ChartData {
	// id that represents this dataset analysed
	private int datasetId;
	
	// index page
	private int pageIndex;
	
	// JFreeChart represented
	private JFreeChart jfChart;

	// XYPlot represented
	private XYPlot xyPlot;
	
	// xyDataset represented
	private XYDataset xyDataset;
	
	// number of series
	private int numberOfSeries;
	
	// number of range axis
	private int numberOfRangeAxis;
	
	//list of Yfield
	private List<String> rangeAxisLabelsList = new ArrayList<String>();
	
	// for a datasetId, all the indexes use in the chart
	private Map<Integer, List<Integer>> datasetIdAndIndexes = new HashMap<Integer, List<Integer>>();
	
	//associate a datasetId and a rangeAxisIndex
	private Map<Integer, Integer> datasetIdAndRangeAxis = new HashMap<Integer, Integer>();
	
	// rangeAxisIndexAndLabel = [ key=indexInRangeAxis, value=label used for this range axis ]
	private Map<Integer, String> rangeAxisIndexAndLabel = new HashMap<Integer, String>();
	
	// rangeAxisLabelAndIndex = [ key=label, value=indexInRangeAxis used for this label ] 
	private Map<String, Integer> rangeAxisLabelAndIndex = new HashMap<String, Integer>();
	
	public ChartData(int chartDataId, int pageIndex) {
		this.datasetId = chartDataId;
		this.pageIndex = pageIndex;
	}

	//variable
	private String RAW = "RAW";
	private String MOVINGMIN = "MOVINGMIN";
	private String MOVINGMAX = "MOVINGMAX";
	private String MOVINGMEAN = "MOVINGMEAN";
	private String MOVINGSTDDVT = "MOVINGSTDDVT";
	private String MOVINGTHROUGHPUT = "MOVINGTHROUGHPUT";
	
	private String ITEM_SEPARATOR = " - ";		//	ITEM_SEPARATOR = " - ";

	
	/**
	 * Simple JFreeChart
	 */
	public JFreeChart createSimpleChart(
			String XField, 
			String YField, 
			StatisticDataImpl sdi, 
			int datasetId, 
			String chartType, 
			int timeWindow, 
			int step,
			int maxPointsToDisplay,
			int timeStart,
			int timeEnd)
	{
		ListOfLong XFieldValue;
		ListOfLong YFieldValue;
		double[] xValues = null;
		double[] yValues = null;
		int indexTimeStart;
		int indexTimeEnd;
		if(chartType.equalsIgnoreCase(RAW)){
			XFieldValue = sdi.getStolDate().getData();
			YFieldValue = sdi.getStol().getData();
			xValues = toDoubleArray(XFieldValue);
			yValues = toDoubleArray(YFieldValue);
			// interval time start and time end
			indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
			indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
			// number maximum of points to display
			xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
			yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
		}
		else if(chartType.equalsIgnoreCase(MOVINGMIN)){
			xValues = toDouble(sdi.getMovingMinimum(timeWindow, step)[0]);
			yValues = toDouble(sdi.getMovingMinimum(timeWindow, step)[1]);
			// interval time start and time end
			indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
			indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
			// number maximum of points to display
			xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
			yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
		}
		else if(chartType.equalsIgnoreCase(MOVINGMAX)){
			xValues = toDouble(sdi.getMovingMaximum(timeWindow, step)[0]);
			yValues = toDouble(sdi.getMovingMaximum(timeWindow, step)[1]);
			// interval time start and time end
			indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
			indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
			// number maximum of points to display
			xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
			yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
		}
		else if(chartType.equalsIgnoreCase(MOVINGMEAN)){
			xValues = toDouble(sdi.getMovingMeans(timeWindow, step)[0]);
			yValues = toDouble(sdi.getMovingMeans(timeWindow, step)[1]);
			// interval time start and time end
			indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
			indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
			// number maximum of points to display
			xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
			yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
		}
		else if(chartType.equalsIgnoreCase(MOVINGSTDDVT)){
			xValues = toDouble(sdi.getMovingStandardDeviations(timeWindow, step)[0]);
			yValues = toDouble(sdi.getMovingStandardDeviations(timeWindow, step)[1]);
			// interval time start and time end
			indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
			indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
			// number maximum of points to display
			xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
			yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
		}
		else if(chartType.equals(MOVINGTHROUGHPUT)){
			xValues = toDouble(sdi.getMovingThroughput(timeWindow, step)[0]);
			yValues = toDouble(sdi.getMovingThroughput(timeWindow, step)[1]);
			// interval time start and time end
			indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
			indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
			// number maximum of points to display
			xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
			yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPointsToDisplay);
		}
		//create a XYDataset for these values
		String plotName = sdi.getTestName() + ITEM_SEPARATOR +  sdi.getBladeId()+ ITEM_SEPARATOR + sdi.getEventType() + ITEM_SEPARATOR + sdi.getField();
		XYDataset xyDataset = this.createDataset(xValues, yValues, plotName);
		
		this.datasetId = datasetId;
		
		//get the plot that represents this chart
		XYPlot plot =  this.xyPlot;

		//save index in the map
		List<Integer> indexesList = datasetIdAndIndexes.get(this.datasetId);

		int numberOfSeries;
		//plot doesn't exist yet, create new one
		if(plot == null){
			plot = new XYPlot();
			numberOfSeries = 0;
			numberOfRangeAxis = 0;
			plot.setRenderer(0, new StandardXYItemRenderer());
			//create the chart
			JFreeChart jfChart = ChartFactory.createXYLineChart("title", XField, YField, xyDataset, PlotOrientation.VERTICAL, true, true, true);
			plot = jfChart.getXYPlot();
			//save indexes for this dataset
			indexesList = new ArrayList<Integer>();
			indexesList.add(numberOfSeries);
			datasetIdAndIndexes.put(this.datasetId, indexesList);
			plot.setRenderer(numberOfSeries, new StandardXYItemRenderer());     
			plot.setDataset(numberOfSeries, xyDataset);
			
			
//			ValueAxis axis = new NumberAxis(fieldLabel);
//			plot.setRangeAxis(datasetIndex, axis);
			//save in map 
			rangeAxisIndexAndLabel.put(numberOfRangeAxis, YField);
			rangeAxisLabelAndIndex.put(YField, numberOfRangeAxis);
			datasetIdAndRangeAxis.put(this.datasetId, numberOfRangeAxis);
			rangeAxisLabelsList.add(YField);
			
			//link this dataset with the range axis
			plot.mapDatasetToRangeAxis(numberOfSeries, numberOfRangeAxis);
			
			numberOfSeries ++;
			numberOfRangeAxis ++;
		}
		//plot exists
		else{
			numberOfSeries = this.numberOfSeries;
			plot = this.jfChart.getXYPlot();
			//datasetId is not in the map
			if(existInChartData(datasetId) == false){
				//save indexes
				indexesList = new ArrayList<Integer>();
				indexesList.add(numberOfSeries);
				datasetIdAndIndexes.put(this.datasetId, indexesList);
				
				//check if the range axis is already in the chart data
				if(rangeAxisLabelsList.contains(YField) == true){
					int rangeAxisTmp = rangeAxisLabelAndIndex.get(YField.toString());
					//get the range axis for this label
					datasetIdAndRangeAxis.put(this.datasetId, rangeAxisTmp);
					plot.setRenderer(numberOfSeries, new StandardXYItemRenderer());     
					plot.setDataset(numberOfSeries, xyDataset);
					plot.mapDatasetToRangeAxis(numberOfSeries, rangeAxisTmp);
				}
				else{
					//create a new range axis
					ValueAxis axis = new NumberAxis(YField);
					//save it in the map
					rangeAxisIndexAndLabel.put(numberOfRangeAxis, YField);
					rangeAxisLabelAndIndex.put(YField, numberOfRangeAxis);
					datasetIdAndRangeAxis.put(this.datasetId, numberOfRangeAxis);
					rangeAxisLabelsList.add(YField);
					
					plot.setRangeAxis(numberOfRangeAxis, axis);
					plot.setRangeAxisLocation(numberOfRangeAxis, AxisLocation.BOTTOM_OR_RIGHT);
					
					plot.setRenderer(numberOfSeries, new StandardXYItemRenderer());     
					plot.setDataset(numberOfSeries, xyDataset);
					plot.mapDatasetToRangeAxis(numberOfSeries, numberOfRangeAxis);
					numberOfRangeAxis ++;
				}
//				System.out.println("range axis count" + plot.getRangeAxisCount() + ", " + plot.getRangeAxis(0));
				numberOfSeries ++;
			}
			// datasetId already in the map
			else{
				// get the correct index 
				int currentSimpleIndex = datasetIdAndIndexes.get(datasetId).get(0);
				// option to change the renderer 
//				plot.setRenderer(currentSimpleIndex, new StandardXYItemRenderer());
				
				// get the YField for this datasetId
				int indexTmp = datasetIdAndRangeAxis.get(datasetId);
				String fieldTmp = rangeAxisIndexAndLabel.get(indexTmp);
				if(fieldTmp == YField){
					plot.setDataset(currentSimpleIndex, xyDataset);
				}
				// modify the YField
				else{
					// the range axis label already exists
					if(rangeAxisLabelsList.contains(YField) == true){
						int rangeAxisTmp = rangeAxisLabelAndIndex.get(YField.toString());
						//get the range axis for this label
						datasetIdAndRangeAxis.put(this.datasetId, rangeAxisTmp);
						plot.setDataset(currentSimpleIndex, xyDataset);
						plot.mapDatasetToRangeAxis(currentSimpleIndex, rangeAxisTmp);
					}
					// create a new range axis
					else{			
						//delete old range axis if not used anymore
						int oldRangeAxisIndex = datasetIdAndRangeAxis.get(datasetId);
						String oldRangeAxisLabel = rangeAxisIndexAndLabel.get(oldRangeAxisIndex);
						
						ValueAxis axis = new NumberAxis(YField);
						//save it in the map
						rangeAxisIndexAndLabel.put(numberOfRangeAxis, YField);
						rangeAxisLabelAndIndex.put(YField, numberOfRangeAxis);
						datasetIdAndRangeAxis.put(this.datasetId, numberOfRangeAxis);
						rangeAxisLabelsList.add(YField);
						
						plot.setRangeAxis(numberOfRangeAxis, axis);
						plot.setRangeAxisLocation(numberOfRangeAxis, AxisLocation.BOTTOM_OR_RIGHT);
						
						plot.setDataset(currentSimpleIndex, xyDataset);
						plot.mapDatasetToRangeAxis(currentSimpleIndex, numberOfRangeAxis);
						numberOfRangeAxis ++;
						
						
						//list of the current label used
						List<String> currentLabelsUsed = getCurrentLabelsUsed();
						//remove the axis not used
						if(currentLabelsUsed.contains(oldRangeAxisLabel.toString()) == false){
							plot.getRangeAxis(oldRangeAxisIndex).setVisible(false);
							rangeAxisIndexAndLabel.remove(oldRangeAxisIndex);
							rangeAxisLabelAndIndex.remove(oldRangeAxisLabel);
							rangeAxisLabelsList.remove(oldRangeAxisLabel.toString());
						}
					}
				}
			}
		}	
		JFreeChart jfChartFinal = new JFreeChart(plot);
		//save data for this current chart data
		this.jfChart = jfChartFinal;
		this.xyPlot = plot;
		this.numberOfSeries = numberOfSeries;
		return jfChartFinal;
	}
	
	
	/**
	 * YRangeLabel
	 * 
	 * @return YAxis label
	 */
	private List<String> getCurrentLabelsUsed() {
		Set<Integer> currentRangeIndexesUsed = new HashSet<Integer>();
		List<String> currentLabelsUsed = new ArrayList<String>();
		Set<Entry<Integer, Integer>> setDP = datasetIdAndRangeAxis.entrySet();
		Iterator<Entry<Integer, Integer>> iteratorDP = setDP.iterator();		
		while(iteratorDP.hasNext()){
			Map.Entry<Integer,Integer> me = (Map.Entry<Integer,Integer>)iteratorDP.next();
			currentRangeIndexesUsed.add(me.getValue());
		} 
		Iterator<Integer> currIterator = currentRangeIndexesUsed.iterator();
		while (currIterator.hasNext()) {
			Integer index = (Integer) currIterator.next();
			String yFieldCurrent = rangeAxisIndexAndLabel.get(index);
			currentLabelsUsed.add(yFieldCurrent);
		}
		return currentLabelsUsed;
	}

	
	/**
	 * Existence of this chartData
	 * 
	 * @return true if this chart exists
	 */
	private boolean existInChartData(int key){
		if(datasetIdAndIndexes.containsKey(key)){
			return true;
		}
		else return false;
	}
	
	
	/**
	 * Create chart with multiple line charts
	 */
	public JFreeChart createMultipleChart(
			String XField, 
			String YField, 
			List<StatisticDataImpl> sdiList, 
			int datasetId, String chartType, 
			int timeWindow, 
			int step,
			int maxPoints,
			int timeStart,
			int timeEnd)
	{
		this.datasetId = datasetId;
		boolean boo = false;
		Iterator<StatisticDataImpl> iterator = sdiList.iterator();

		//get the plot that represents this chart
		XYPlot plot =  this.xyPlot;
		
		//get index in the map
		List<Integer> indexesList = datasetIdAndIndexes.get(this.datasetId);
		
		int indexTimeStart;
		int indexTimeEnd;
		int numberOfSeries;
		// plot doesn't exists : create a new one
		if(plot == null){
			plot = new XYPlot();
			numberOfSeries = 0;
			plot.setRenderer(0, new StandardXYItemRenderer());  
			JFreeChart jfChart = ChartFactory.createXYLineChart("title", XField, YField, xyDataset, PlotOrientation.VERTICAL, true, true, true);
			plot = jfChart.getXYPlot();
			indexesList = new ArrayList<Integer>();
			while (iterator.hasNext()) {
				StatisticDataImpl sdi = (StatisticDataImpl) iterator.next();
				ListOfLong XFieldValue;
				ListOfLong YFieldValue;
				double[] xValues = null;
				double[] yValues = null;
				if(chartType.equalsIgnoreCase(RAW)){
					XFieldValue = sdi.getStolDate().getData();
					YFieldValue = sdi.getStol().getData();
					xValues = toDoubleArray(XFieldValue, maxPoints);
					yValues = toDoubleArray(YFieldValue, maxPoints);	
					// interval time start and time end
					indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
					indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
					// number maximum of points to display
					xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
					yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
				}
				else if(chartType.equalsIgnoreCase(MOVINGMIN)){
					xValues = toDouble(sdi.getMovingMinimum(timeWindow, step)[0], maxPoints);
					yValues = toDouble(sdi.getMovingMinimum(timeWindow, step)[1], maxPoints);
					// interval time start and time end
					indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
					indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
					// number maximum of points to display
					xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
					yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
				}
				else if(chartType.equalsIgnoreCase(MOVINGMAX)){
					xValues = toDouble(sdi.getMovingMaximum(timeWindow, step)[0], maxPoints);
					yValues = toDouble(sdi.getMovingMaximum(timeWindow, step)[1], maxPoints);
					// interval time start and time end
					indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
					indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
					// number maximum of points to display
					xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
					yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
				}
				else if(chartType.equalsIgnoreCase(MOVINGMEAN)){
					xValues = toDouble(sdi.getMovingMeans(timeWindow, step)[0], maxPoints);
					yValues = toDouble(sdi.getMovingMeans(timeWindow, step)[1], maxPoints);
					// interval time start and time end
					indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
					indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
					// number maximum of points to display
					xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
					yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
				}
				else if(chartType.equalsIgnoreCase(MOVINGSTDDVT)){
					xValues = toDouble(sdi.getMovingStandardDeviations(timeWindow, step)[0], maxPoints);
					yValues = toDouble(sdi.getMovingStandardDeviations(timeWindow, step)[1], maxPoints);
					// interval time start and time end
					indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
					indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
					// number maximum of points to display
					xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
					yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
				}
				else if(chartType.equals(MOVINGTHROUGHPUT)){
					xValues = toDouble(sdi.getMovingThroughput(timeWindow, step)[0], maxPoints);
					yValues = toDouble(sdi.getMovingThroughput(timeWindow, step)[1], maxPoints);
					// interval time start and time end
					indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
					indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
					// number maximum of points to display
					xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
					yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
				}
				//create a XYDataset for these values
				String plotName = sdi.getBladeIdAlias()+ ITEM_SEPARATOR + sdi.getEventType() + ITEM_SEPARATOR + sdi.getField();
				XYDataset xyDataset = this.createDataset(xValues, yValues, plotName);

				plot.setRenderer(numberOfSeries, new StandardXYItemRenderer());     
				plot.setDataset(numberOfSeries, xyDataset);
				indexesList.add(numberOfSeries);
				
				//save in map 
				rangeAxisIndexAndLabel.put(numberOfRangeAxis, YField);
				rangeAxisLabelAndIndex.put(YField, numberOfRangeAxis);
				datasetIdAndRangeAxis.put(this.datasetId, numberOfRangeAxis);
				rangeAxisLabelsList.add(YField);
				
				//link this dataset with the range axis
//				plot.mapDatasetToRangeAxis(numberOfSeries, numberOfRangeAxis);
				
				numberOfSeries ++;
			}
			numberOfRangeAxis ++;
			datasetIdAndIndexes.put(this.datasetId, indexesList);
		}
		//plot already exists
		else{
			numberOfSeries = this.numberOfSeries;
			plot = this.jfChart.getXYPlot();
			//if this dataset doesn't exists
			if(existInChartData(datasetId) == false){
				indexesList = new ArrayList<Integer>();
				while (iterator.hasNext()) {
					StatisticDataImpl sdi = (StatisticDataImpl) iterator.next();
					
					ListOfLong XFieldValue;
					ListOfLong YFieldValue;
					double[] xValues = null;
					double[] yValues = null;
					if(chartType.equalsIgnoreCase(RAW)){
						XFieldValue = sdi.getStolDate().getData();
						YFieldValue = sdi.getStol().getData();
						xValues = toDoubleArray(XFieldValue, maxPoints);
						yValues = toDoubleArray(YFieldValue, maxPoints);	
					}
					else if(chartType.equalsIgnoreCase(MOVINGMIN)){
						xValues = toDouble(sdi.getMovingMinimum(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingMinimum(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					else if(chartType.equalsIgnoreCase(MOVINGMAX)){
						xValues = toDouble(sdi.getMovingMaximum(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingMaximum(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					else if(chartType.equalsIgnoreCase(MOVINGMEAN)){
						xValues = toDouble(sdi.getMovingMeans(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingMeans(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					else if(chartType.equalsIgnoreCase(MOVINGSTDDVT)){
						xValues = toDouble(sdi.getMovingStandardDeviations(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingStandardDeviations(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					else if(chartType.equals(MOVINGTHROUGHPUT)){
						xValues = toDouble(sdi.getMovingThroughput(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingThroughput(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					//create a XYDataset for these values
					String plotName = sdi.getBladeId()+ ITEM_SEPARATOR + sdi.getEventType() + ITEM_SEPARATOR + sdi.getField();
					XYDataset xyDataset = this.createDataset(xValues, yValues, plotName);
					
					
					//check if the range axis is already in the chart data
					if(rangeAxisLabelsList.contains(YField) == true){
						int rangeAxisTmp = rangeAxisLabelAndIndex.get(YField.toString());
						//get the range axis for this label
						datasetIdAndRangeAxis.put(this.datasetId, rangeAxisTmp);
						plot.setRenderer(numberOfSeries, new StandardXYItemRenderer());     
						plot.setDataset(numberOfSeries, xyDataset);
						plot.mapDatasetToRangeAxis(numberOfSeries, rangeAxisTmp);
					}
					else{
						//create a new range axis
						ValueAxis axis = new NumberAxis(YField);
						//save it in the map
						rangeAxisIndexAndLabel.put(numberOfRangeAxis, YField);
						rangeAxisLabelAndIndex.put(YField, numberOfRangeAxis);
						datasetIdAndRangeAxis.put(this.datasetId, numberOfRangeAxis);
						rangeAxisLabelsList.add(YField);
						
						plot.setRangeAxis(numberOfRangeAxis, axis);
						plot.setRangeAxisLocation(numberOfRangeAxis, AxisLocation.BOTTOM_OR_RIGHT);
						
						plot.setRenderer(numberOfSeries, new StandardXYItemRenderer());     
						plot.setDataset(numberOfSeries, xyDataset);
						plot.mapDatasetToRangeAxis(numberOfSeries, numberOfRangeAxis);
					}
					
					indexesList.add(numberOfSeries);
					numberOfSeries ++;
				}
				numberOfRangeAxis ++;
				datasetIdAndIndexes.put(this.datasetId, indexesList);
			}
			//if this dataset already exists
			else{				
				List<Integer> intList = datasetIdAndIndexes.get(datasetId); 
				int cpt = 0;
				while (iterator.hasNext()) {
					StatisticDataImpl sdi = (StatisticDataImpl) iterator.next();
					
					ListOfLong XFieldValue;
					ListOfLong YFieldValue;
					double[] xValues = null;
					double[] yValues = null;
					if(chartType.equalsIgnoreCase(RAW)){
						XFieldValue = sdi.getStolDate().getData();
						YFieldValue = sdi.getStol().getData();
						xValues = toDoubleArray(XFieldValue, maxPoints);
						yValues = toDoubleArray(YFieldValue, maxPoints);	
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					else if(chartType.equalsIgnoreCase(MOVINGMIN)){
						xValues = toDouble(sdi.getMovingMinimum(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingMinimum(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					else if(chartType.equalsIgnoreCase(MOVINGMAX)){
						xValues = toDouble(sdi.getMovingMaximum(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingMaximum(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					else if(chartType.equalsIgnoreCase(MOVINGMEAN)){
						xValues = toDouble(sdi.getMovingMeans(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingMeans(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					else if(chartType.equalsIgnoreCase(MOVINGSTDDVT)){
						xValues = toDouble(sdi.getMovingStandardDeviations(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingStandardDeviations(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					else if(chartType.equals(MOVINGTHROUGHPUT)){
						xValues = toDouble(sdi.getMovingThroughput(timeWindow, step)[0], maxPoints);
						yValues = toDouble(sdi.getMovingThroughput(timeWindow, step)[1], maxPoints);
						// interval time start and time end
						indexTimeStart = toDoubleArrayFromIndexStart(xValues, timeStart, timeEnd);
						indexTimeEnd = toDoubleArrayFromIndexEnd(xValues, timeStart, timeEnd);
						// number maximum of points to display
						xValues = toDoubleFromIndexToIndex(xValues, indexTimeStart, indexTimeEnd, maxPoints);
						yValues = toDoubleFromIndexToIndex(yValues, indexTimeStart, indexTimeEnd, maxPoints);
					}
					//create a XYDataset for these values
					String plotName = sdi.getBladeId()+ ITEM_SEPARATOR + sdi.getEventType() + ITEM_SEPARATOR + sdi.getField();
					XYDataset xyDataset = this.createDataset(xValues, yValues, plotName);
					
//					plot.setRenderer(numberOfSeries, new StandardXYItemRenderer());     
					
					// get the YField for this datasetId
					int indexTmp = datasetIdAndRangeAxis.get(datasetId);
					String fieldTmp = rangeAxisIndexAndLabel.get(indexTmp);
					// if the field hasn't changed
					if(fieldTmp == YField){
						plot.setDataset(intList.get(cpt), xyDataset);
					}
					// modify the YField
					else{
						// the range axis label already exists
						if(rangeAxisLabelsList.contains(YField) == true){
							try {
								//DEFAULT
//								int rangeAxisTmp = rangeAxisLabelAndIndex.get(YField.toString());
								//get the range axis for this label
//								datasetIdAndRangeAxis.put(this.datasetId, rangeAxisTmp);
								plot.setDataset(intList.get(cpt), xyDataset);
//								plot.mapDatasetToRangeAxis(intList.get(cpt), rangeAxisTmp);	
							} catch (Exception e) {
								e.printStackTrace();
							}
						}
						// create a new range axis
						else{			
							//delete old range axis if not used anymore
							int oldRangeAxisIndex = datasetIdAndRangeAxis.get(datasetId);
							String oldRangeAxisLabel = rangeAxisIndexAndLabel.get(oldRangeAxisIndex);
							
							ValueAxis axis = new NumberAxis(YField);
							//save it in the map
							rangeAxisIndexAndLabel.put(numberOfRangeAxis, YField);
							rangeAxisLabelAndIndex.put(YField, numberOfRangeAxis);
							datasetIdAndRangeAxis.put(this.datasetId, numberOfRangeAxis);
							rangeAxisLabelsList.add(YField);
							
							plot.setRangeAxis(numberOfRangeAxis, axis);
							plot.setRangeAxisLocation(numberOfRangeAxis, AxisLocation.BOTTOM_OR_RIGHT);
							
							plot.setDataset(intList.get(cpt), xyDataset);
							plot.mapDatasetToRangeAxis(intList.get(cpt), numberOfRangeAxis);
							
							boo = true;
							
							//list of the current label used
							List<String> currentLabelsUsed = getCurrentLabelsUsed();
							//remove the axis not used
							if(currentLabelsUsed.contains(oldRangeAxisLabel.toString()) == false){
								plot.getRangeAxis(oldRangeAxisIndex).setVisible(false);
								rangeAxisIndexAndLabel.remove(oldRangeAxisIndex);
								rangeAxisLabelAndIndex.remove(oldRangeAxisLabel);
								rangeAxisLabelsList.remove(oldRangeAxisLabel.toString());
							}
						}
					}
					cpt ++;
				}
				if(boo == true){
					numberOfRangeAxis ++;
				}
			}
		}	
		
		JFreeChart jfChartFinal = new JFreeChart(plot);
		//save data for this current chart data
		this.jfChart = jfChartFinal;
		this.xyPlot = plot;
		this.numberOfSeries = numberOfSeries;
		return jfChartFinal;		
	}
	
	
	
	/**
	 * Create a dataset from the dataset
	 * 
	 * @param data
	 * @return JFreeChart
	 */
	private JFreeChart createChart(XYDataset data, String XField, String YField) {
		JFreeChart chart = ChartFactory.createXYLineChart("title", XField, YField, data, PlotOrientation.VERTICAL, true, true, true);
		return chart;
		}
	
	
	/**
	 * Get dataset matching the values
	 */
	public XYDataset createDataset(double[] xValues, double[] yValues, String key) {
		XYSeries serie = new XYSeries(key); 	
		if( xValues.length == yValues.length){
			for (int i = 0; i < xValues.length; i++){
				serie.add(xValues[i], yValues[i]);
			}
		}
		else return null;
		XYSeriesCollection collection = new XYSeriesCollection();
		collection.addSeries(serie);
		return collection;
	}

	
	// ///////////////////////////////
	// private methods //
	// ///////////////////////////////
	
    /**
     * Get the bladeEventArray from a bladeEventList
     * @param bladeEventList
     * @return bladeEventArray an array that contains the blade events
     */
    private BladeEvent[] toBladeEventArray(List<BladeEvent> bladeEventList){
    	int size = bladeEventList.size();
    	BladeEvent[] bladeEventArray = new BladeEvent[size];
    	Iterator<BladeEvent> iterator = bladeEventList.iterator();
    	int count = 0;
    	while (iterator.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator.next();
			bladeEventArray[count] = bladeEvent;
			count++;
		}
    	return bladeEventArray;
    }
    
    
    /**
     * Get the value for one field label in a bladeEventArray
     * @param bladeEventArray
     * @param fieldLabel
     * @return an array that contains the values from a field in a bladeEvent
     */
    private double[] toDoubleArray(BladeEvent[] bladeEventArray, String fieldLabel){
    	Object[] values = new Object[bladeEventArray.length];
    	for (int i = 0; i < bladeEventArray.length; i++) {
			values[i] = bladeEventArray[i].getFieldValue(fieldLabel);
		}
    	return toDouble(values);
    }
    
    
    /**
     * To double.
     * 
     * @param values the values 
     * @return the double[]
     */
    private double[] toDouble(Object[] values){
    	double[] vals = new double[values.length];
    	if (values.length>0) {
    		if(values[0] instanceof Number) {
    			for (int i = 0; i < values.length && values[i]!=null; i++) {
    				vals[i] = ((Number)values[i]).doubleValue();
    			}
    		}
    		else if (values[0] instanceof Boolean) {
    			for (int i = 0; i < values.length && values[i]!=null; i++) {
    				vals[i] = ((Boolean)values[i]).booleanValue()?1.0:0.0;
    			}
    		}
    		else {
    			for (int i = 0; i < values.length && values[i]!=null; i++) {
    				vals[i] = 1.0;
    			}
    		}
    	}
    	return vals;
    }
    
    
    /**
     * To double.
     * 
     * @param values the values 
     * @return the double[]
     */
    private double[] toDouble(Object[] values, int maxPoints){

    	double[] vals = new double[values.length];
    	if(maxPoints == -1){
    		if (values.length>0) {
    			if(values[0] instanceof Number) {
    				for (int i = 0; i < values.length && values[i]!=null; i++) {
    					vals[i] = ((Number)values[i]).doubleValue();
    				}
    			}
    			else if (values[0] instanceof Boolean) {
    				for (int i = 0; i < values.length && values[i]!=null; i++) {
    					vals[i] = ((Boolean)values[i]).booleanValue()?1.0:0.0;
    				}
    			}
    			else {
    				for (int i = 0; i < values.length && values[i]!=null; i++) {
    					vals[i] = 1.0;
    				}
    			}
    		}
    	}
    	else{
    		if (values.length>0) {
    			if(values[0] instanceof Number) {
    				for (int i = 0; i < values.length && values[i]!=null && i<=maxPoints; i++) {
    					vals[i] = ((Number)values[i]).doubleValue();
    				}
    			}
    			else if (values[0] instanceof Boolean) {
    				for (int i = 0; i < values.length && values[i]!=null && i<=maxPoints; i++) {
    					vals[i] = ((Boolean)values[i]).booleanValue()?1.0:0.0;
    				}
    			}
    			else {
    				for (int i = 0; i < values.length && values[i]!=null && i<=maxPoints; i++) {
    					vals[i] = 1.0;
    				}
    			}
    		}
    	}
    	return vals;
    }
    
    
    /**
     * To double.
     * 
     * @param values the values 
     * @return the double[]
     */
    private double[] toDouble(Object[] values, int maxPoints, int timeStart, int timeEnd){

    	double[] vals = new double[values.length];
    	if(timeStart < timeEnd){
    		if(maxPoints == -1){
        		if (values.length>0) {
        			if(values[0] instanceof Number) {
        				for (int i = 0; i < values.length && values[i]!=null; i++) {
        					if(((Number)values[i]).doubleValue() > timeStart && ((Number)values[i]).doubleValue() < timeEnd){
        						vals[i] = ((Number)values[i]).doubleValue();
        					}
        				}
        			}
        			else if (values[0] instanceof Boolean) {
        				for (int i = 0; i < values.length && values[i]!=null; i++) {
        					if(((Number)values[i]).doubleValue() > timeStart && ((Number)values[i]).doubleValue() < timeEnd){
        					vals[i] = ((Boolean)values[i]).booleanValue()?1.0:0.0;
        					}
        				}
        			}
        			else {
        				for (int i = 0; i < values.length && values[i]!=null; i++) {
        					if(((Number)values[i]).doubleValue() > timeStart && ((Number)values[i]).doubleValue() < timeEnd){
        					vals[i] = 1.0;
        					}
        				}
        			}
        		}
        	}
        	else{
        		if (values.length>0) {
        			if(values[0] instanceof Number) {
        				for (int i = 0; i < values.length && values[i]!=null && i<=maxPoints; i++) {
        					if(((Number)values[i]).doubleValue() > timeStart && ((Number)values[i]).doubleValue() < timeEnd){
        					vals[i] = ((Number)values[i]).doubleValue();
        					}
        				}
        			}
        			else if (values[0] instanceof Boolean) {
        				for (int i = 0; i < values.length && values[i]!=null && i<=maxPoints; i++) {
        					if(((Number)values[i]).doubleValue() > timeStart && ((Number)values[i]).doubleValue() < timeEnd){
        					vals[i] = ((Boolean)values[i]).booleanValue()?1.0:0.0;
        					}
        				}
        			}
        			else {
        				for (int i = 0; i < values.length && values[i]!=null && i<=maxPoints; i++) {
        					if(((Number)values[i]).doubleValue() > timeStart && ((Number)values[i]).doubleValue() < timeEnd){
        					vals[i] = 1.0;
        					}
        				}
        			}
        		}
        	}
    	}
    	return vals;
    }
    
    
    /**
     * Convert long list to double array
     * 
     * @param longList
     * @return a double array
     */
    private double[] toDoubleArray(List<Long> longList){
    	int size = longList.size();
    	double[] longArray = new double[size];
    	Iterator<Long> iterator = longList.iterator();
    	int count = 0;
    	while (iterator.hasNext()) {
    		double currentValue = (double) iterator.next();
			longArray[count] = currentValue;
			count++;
		}
    	return longArray;
    }

    
    /**
     * Convert long list to double array with maxpoints
     * 
     * @param longList
     * @param maxPoints
     * @return a double array
     */
    private double[] toDoubleArray(List<Long> longList, int maxPoints){
    	int size = longList.size();
    	double[] longArray = new double[size];
    	Iterator<Long> iterator = longList.iterator();
    	int count = 0;
    	// no restriction on maxpoints
    	if(maxPoints == -1){
    		while (iterator.hasNext()) {
        		double currentValue = (double) iterator.next();
    			longArray[count] = currentValue;
    			count++;
    		}
    	}
    	// restriction on maxpoints
    	else{
    		while (iterator.hasNext() && count<maxPoints) {
        		double currentValue = (double) iterator.next();
    			longArray[count] = currentValue;
    			count++;
    		}
    	}
    	return longArray;
    }
    
    
    /**
     * Get the long list filtered from time start to time end with max points
     * 
     * @param longList
     * @param maxPoints
     * @param timeStart
     * @param timeEnd
     * @return a double array 
     */
    private double[] toDoubleArray(List<Long> longList, int maxPoints, int timeStart, int timeEnd){
    	int size = longList.size();
    	double[] longArray = new double[size];
    	Iterator<Long> iterator = longList.iterator();
    	int count = 0;
    	double currentValue = -1;
    	if( timeEnd > timeStart){
    		if(maxPoints == -1){
    			while (iterator.hasNext() && currentValue < timeEnd) {
    				currentValue = (double) iterator.next();
    				if(currentValue > timeStart && currentValue < timeEnd){
    					longArray[count] = currentValue;
    					count++;
    				}
    			}
    		}
    		else{
    			while (iterator.hasNext() && count<maxPoints && currentValue < timeEnd) {
    				currentValue = (double) iterator.next();
    				if(currentValue > timeStart && currentValue < timeEnd){
    					longArray[count] = currentValue;
    					count++;
    				}
    			}
    		}
    	}
    	return longArray;
    }
    
    
    /**
     * Filter array from time start to time end
     * 
     * @param data
     * @param timeStart
     * @param timeEnd
     * @return index start
     */
    private int toDoubleArrayFromIndexStart(double[] data, int timeStart, int timeEnd){
    	int indexStart = -1;
    	boolean indexStartBoo = true;
    	for (int i = 0; i < data.length && indexStartBoo; i++) {
			if(data[i] >= timeStart && indexStartBoo==true){
				indexStartBoo = false;
				indexStart = i;
			}
		}
    	return indexStart;
    }
    
    
    /**
     * Filter array from time start to time end
     * 
     * @param data
     * @param timeStart
     * @param timeEnd
     * @return index end
     */
    private int toDoubleArrayFromIndexEnd(double[] data, int timeStart, int timeEnd){
    	int indexEnd = -1;
    	for (int i = 0; i < data.length; i++) {
			if(data[i]<timeEnd){
				indexEnd = i; 
			}
		}
    	return indexEnd;
    }
    
    
    /**
     * Get this double array filtered from start index and end index with max points
     * 
     * @param data
     * @param indexStart
     * @param indexEnd
     * @param maxPoints
     * @return a double array
     */
    private double[] toDoubleFromIndexToIndex(double[] data, int indexStart, int indexEnd, int maxPoints){
    	double[] result;
    	int cpt = 0;
    	if(indexStart != -1 && indexEnd != -1){
        	if(maxPoints == -1){
        		result = new double[indexEnd - indexStart + 1];
        		for (int i = indexStart; i <= indexEnd; i++) {
        			result[cpt] = data[i];
        			cpt ++;
        		}
        	}
        	else {
        		if(indexEnd - indexStart > maxPoints){
        			result = new double[maxPoints + 1];
        		}
        		else {
        			result = new double[indexEnd - indexStart + 1];
        		}
        		for (int i = indexStart; i <= indexEnd && cpt <= maxPoints; i++) {
        			result[cpt] = data[i];
        			cpt ++;
        		}
        	}
        	return result;
    	}
    	else return data;
    }
    

	/**
	 * Get XYdataset
	 * 
	 * @return XYDataset
	 */
	public XYDataset getXyDataset() {
		return xyDataset;
	}


	/**
	 * Get the jfreechart
	 * 
	 * @return Jfreechart
	 */
	public JFreeChart getJfChart() {
		return jfChart;
	}


	/**
	 * Get all the indexes use for the chart
	 * @return list of indexes use for the chart 
	 */
	public List<Integer> getSeriesIndex(){
		return this.datasetIdAndIndexes.get(this.datasetId);
	}

	
	public JFreeChart removeDatasetFromTab(int keyDataset) {
		this.datasetId = keyDataset;
		
		//get the plot that represents this chart
		XYPlot plot =  this.xyPlot;
		
		//get index in the map
		List<Integer> indexesList = datasetIdAndIndexes.get(this.datasetId);

		
		//remove range axis if not used
		//delete old range axis if not used anymore
		int oldRangeAxisIndex = datasetIdAndRangeAxis.get(datasetId);
		String oldRangeAxisLabel = rangeAxisIndexAndLabel.get(oldRangeAxisIndex);
		
		datasetIdAndRangeAxis.remove(this.datasetId);
		
		//list of the current label used
		List<String> currentLabelsUsed = getCurrentLabelsUsed();
		//remove the axis not used
		if(currentLabelsUsed.contains(oldRangeAxisLabel.toString()) == false){
			plot.getRangeAxis(oldRangeAxisIndex).setVisible(false);
			rangeAxisIndexAndLabel.remove(oldRangeAxisIndex);
			rangeAxisLabelAndIndex.remove(oldRangeAxisLabel);
			rangeAxisLabelsList.remove(oldRangeAxisLabel.toString());
		}
		
		Iterator<Integer> intlist = indexesList.iterator();
		//remove from the chart
		while (intlist.hasNext()) {
		plot.setDataset((Integer) intlist.next(), null);
		}
		
		//remove key from map
		datasetIdAndIndexes.remove(keyDataset);
		
		//return correct JFreeChart
		JFreeChart jfChartFinal = new JFreeChart(plot);
		//save data for this current chart data
		this.jfChart = jfChartFinal;
		this.xyPlot = plot;
		return jfChartFinal;	
	}
	
	
	/**
	 * Get all the dataset ids used in this chart
	 * 
	 * @return all dataset ids 
	 */
	public List<Integer> getAllDatasetId(){
		List<Integer> inList = new ArrayList<Integer>();
		Set<Entry<Integer, List<Integer>>> set = datasetIdAndIndexes.entrySet();
		Iterator<Entry<Integer, List<Integer>>> iterator = set.iterator();
		while(iterator.hasNext()){
			Map.Entry<Integer,List<Integer>> me = (Map.Entry<Integer,List<Integer>>)iterator.next();
			int datasetId = me.getKey();
			inList.add(datasetId);
		}
		return inList;
	}

}
