/*
* CLIF is a Load Injection Framework
* Copyright (C) 2003-2005, 2008, 2010-2011 France Telecom R&D
* Copyright (C) 2003 INRIA
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.storage.lib.filestorage;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.math.BigInteger;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import org.objectweb.fractal.api.control.BindingController;
import org.omg.CORBA.LongHolder;
import org.ow2.clif.console.lib.ClifDeployDefinition;
import org.ow2.clif.console.lib.TestPlanReader;
import org.ow2.clif.console.lib.TestPlanWriter;
import org.ow2.clif.storage.api.AbstractEvent;
import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.BladeFilter;
import org.ow2.clif.storage.api.CollectListener;
import org.ow2.clif.storage.api.EventFactory;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.storage.api.NoMoreEvent;
import org.ow2.clif.storage.api.StorageAdmin;
import org.ow2.clif.storage.api.StorageProxyAdmin;
import org.ow2.clif.storage.api.StorageRead;
import org.ow2.clif.storage.api.TestDescriptor;
import org.ow2.clif.storage.api.TestFilter;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.util.ClifClassLoader;
import org.ow2.clif.util.UniqueKey;

/**
 * Part of the storage.
 * This part is deployed in the console
 *
 * @author Julien Buret
 * @author Nicolas Droze
 * @author Bruno Dillenseger
 */
public class ConsoleFileStorageImpl
	implements StorageAdmin, BindingController, StorageRead
{
	protected Map<String,StorageProxyAdmin> distributedStorage = new Hashtable<String,StorageProxyAdmin>();
	protected BufferedWriter test = null;
	protected String testDirname = null;
	protected Serializable testId = null;
	protected Map<BigInteger,EventIterator> eventIterators = new HashMap<BigInteger,EventIterator>();
	protected BigInteger eventIteratorKey = BigInteger.ZERO;


	/////////////////////////////////
	// interface BindingController //
	/////////////////////////////////


	public Object lookupFc(String clientItfName)
	{
		if (clientItfName.startsWith(StorageProxyAdmin.STORAGEPROXY_ADMIN))
		{
			return distributedStorage.get(clientItfName);
		}
		return null;
	}


	public void bindFc(String clientItfName, Object serverItf)
	{
		if (clientItfName.startsWith(StorageProxyAdmin.STORAGEPROXY_ADMIN))
		{
			distributedStorage.put(clientItfName, (StorageProxyAdmin)serverItf);
		}
	}


	public void unbindFc(String clientItfName)
	{
		if (clientItfName.startsWith(StorageProxyAdmin.STORAGEPROXY_ADMIN))
		{
			distributedStorage.remove(clientItfName);
		}
	}


	public String[] listFc()
	{
		return distributedStorage.keySet().toArray(new String[distributedStorage.size()]);
	}


	////////////////////////////
	// interface StorageAdmin //
	////////////////////////////


	/**
	 * Inform the storage system the beginning of a new test
	 * @param testPlan Map of scenario deployment definitions, indexed by a unique identifier
	 * @see org.ow2.clif.deploy.DeployDefinition
	 */
	public void newTest(
		Serializable testId,
		Map<String,ClifDeployDefinition> testPlan)
	throws ClifException
	{
		this.testId = testId;
		testDirname = FileStorageCommons.getTestDirbase() + File.separator + testId;
		try
		{
			File baseDir = new File(testDirname);
			int i = 0;
			while (! baseDir.mkdirs())
			{
				if (baseDir.exists())
				{
					testDirname = FileStorageCommons.getTestDirbase() + File.separator + testId + "-" + ++i;
					baseDir = new File(testDirname);
				}
				else
				{
					throw new IOException("Could not create " + testDirname + " directory.");
				}
			}
			if (testPlan != null)
			{
				TestPlanWriter.write2prop(
					new FileOutputStream(testDirname + TestPlanReader.FILE_EXT),
					testPlan);
			}
		}
		catch (Exception e)
		{
			throw new ClifException("Can't create new test run " + testId, e);
		}
	}


	public void collect(Serializable selBladesId, CollectListener listener)
	{
	    Iterator<StorageProxyAdmin> list = distributedStorage.values().iterator();
	    List<String> selBladesIdList = null;
	    if (selBladesId != null)
	    {
	    	selBladesIdList = Arrays.asList((String[])selBladesId);
	    }
	    Map<String,StorageProxyAdmin> storageProxiesByBladeId = new HashMap<String,StorageProxyAdmin>();
	    Map<String,UniqueKey> collectKeysByBladeId = new HashMap<String,UniqueKey>();
	    long fullCollectSize = 0;
	    StorageProxyAdmin store;
	    String bladeId;
	    UniqueKey key;
		while (list.hasNext())
		{
			store = list.next();
			bladeId = store.getBladeId();
			if (selBladesId == null || selBladesIdList.contains(bladeId))
			{
				key = store.initCollect(testId);
				storageProxiesByBladeId.put(bladeId, store);
				collectKeysByBladeId.put(bladeId, key);
		    	fullCollectSize += store.getCollectSize(key);
		    }
		}
	    if (listener != null)
	    {
	    	listener.collectStart(testId.toString(), fullCollectSize);
	    }
	    Iterator<Map.Entry<String,StorageProxyAdmin>> entryList = storageProxiesByBladeId.entrySet().iterator();
	    byte[] buffer = new byte[FileStorageCollect.BLOCK_SIZE];
	    try
	    {
	        while (entryList.hasNext() && (listener == null || !listener.isCanceled()))
	        {
	        	Map.Entry<String,StorageProxyAdmin> entry = entryList.next();
	            store = entry.getValue();
	            bladeId = entry.getKey();
	            key = collectKeysByBladeId.get(bladeId);
	            long size = store.getCollectSize(key);
	            if (listener != null)
	            {
	            	listener.bladeCollectStart(bladeId, size);
	            }
	            long progress = 0;
	            File directory = new File(testDirname + File.separator + store.getBladeId());
				if (directory.mkdirs())
				{
					FileStorageCollectStep step;
					Socket sock;
					while (
						(listener == null || ! listener.isCanceled() || ! listener.isCanceled(bladeId))
						&& (step = (FileStorageCollectStep) store.collect(key)) != null)
					{
					    File outputFile = new File(directory, step.getFilename());
					    if (!outputFile.exists())
					    {
							sock = new Socket();
							sock.connect(step.getSocketAddress(), 0);
							InputStream in = sock.getInputStream();
							OutputStream out = new FileOutputStream(outputFile);
							int n = in.read(buffer);
							while (
								(listener == null || ! listener.isCanceled() || ! listener.isCanceled(bladeId))
								&& n != -1)
							{
								out.write(buffer, 0, n);
								progress += n;
								if (listener != null)
								{
									listener.progress(bladeId, progress);
								}
								n = in.read(buffer);
							}
							in.close();
							out.close();
							sock.close();
					    }
	                }
	            }
				else if (listener != null)
				{
					listener.progress(bladeId, size);
				}
	        }
	        if (listener != null)
	        {
	        	listener.done();
	        }
	    }
	    catch (Exception ex)
	    {
	        throw new RuntimeException("Could not collect data", ex);
	    }
	}


	/**
	 * Terminate the storage system
	 */
	public void terminate()
	{
	}


	///////////////////////////
	// interface StorageRead //
	///////////////////////////


	/**
	 * @param filter only tests matching this filter will be retained
	 * @return a browser object to browse among available test execution results
	 */
	public TestDescriptor[] getTests(final TestFilter filter)
		throws ClifException
	{
		final List<TestDescriptor> descriptors = new ArrayList<TestDescriptor>();
		try
		{
			new File(FileStorageCommons.getTestDirbase()).listFiles(
				new FileFilter() {
					public boolean accept(File path)
					{
						if (path.isDirectory())
						{
							try
							{
								TestDescriptor desc = new TestDescriptorImpl(path);
								if (filter == null || filter.accept(desc))
								{
									descriptors.add(desc);
									return true;
								}
								else
								{
									return false;
								}
							}
							catch (Exception ex)
							{
								throw new RuntimeException("Cannot get test descriptor for " + path);
							}
						}
						else
						{
							return false;
						}
					}
				});
		}
		catch (Exception ex)
		{
			throw new ClifException("Could not get tests", ex);
		}
		return descriptors.toArray(new TestDescriptor[descriptors.size()]);
	}


	public BladeDescriptor[] getTestPlan(String testName, BladeFilter filter)
		throws ClifException
	{
		List<BladeDescriptor> blades = null;
		try
		{
			Map<String,ClifDeployDefinition> testplan = TestPlanReader.readFromProp(
				new FileInputStream(new File(FileStorageCommons.getTestDirbase(), testName + TestPlanReader.FILE_EXT)));
			blades = new ArrayList<BladeDescriptor>(testplan.size());
			Iterator<Map.Entry<String,ClifDeployDefinition>> iter = testplan.entrySet().iterator();
			File testDir = new File(FileStorageCommons.getTestDirbase(), testName);
			while (iter.hasNext())
			{
				Map.Entry<String,ClifDeployDefinition> entry = iter.next();
				BladeDescriptor desc = new BladeDescriptorImpl(
					new File(testDir, entry.getKey()),
					entry.getKey(),
					entry.getValue());
				if (filter == null || filter.accept(desc))
				{
					blades.add(desc);
				}
			}
		}
		catch (Exception ex)
		{
			throw new ClifException("Could not get test plan for " + testName, ex);
		}
		return blades.toArray(new BladeDescriptor[blades.size()]);
	}


	public Properties getBladeProperties(String testName, String bladeId)
		throws ClifException
	{
		Properties props = new Properties();
		try
		{
			props.load(
				new FileInputStream(
					new File(
						FileStorageCommons.getBladeDir(testName, bladeId),
						FileStorageCommons.JVMPROPS_FILENAME)));
		}
		catch (Exception ex)
		{
			throw new ClifException(
				"Could not get properties for blade " + bladeId + " from test " + testName);
		}
		return props;
	}


	private Class<?> loadEventClass(String testName, String bladeId, String eventTypeLabel)
		throws ClifException
	{
		BufferedReader reader = null;
		try
		{
			reader = new BufferedReader(
				new FileReader(
					FileStorageCommons.getEventClassFile(testName, bladeId, eventTypeLabel)));
			return Class.forName(
				reader.readLine(),
				true,
				ClifClassLoader.getClassLoader());
		}
		catch (Exception ex)
		{
			throw new ClifException(
				"Could not get event class for " + eventTypeLabel + " from " + testName + "/" + bladeId + "/",
				ex);
		}
		finally
		{
			if (reader != null)
			{
				try
				{
					reader.close();
				}
				catch (IOException ex)
				{
					ex.printStackTrace(System.err);
				}
			}
		}
	}


	public String[] getEventFieldLabels(String testName, String bladeId, String eventTypeLabel)
	{
		String[] result = AbstractEvent.getEventFieldLabels(eventTypeLabel);
		if (result == null)
		{
			try
			{
				Class<?> clazz = loadEventClass(testName, bladeId, eventTypeLabel);
				result = AbstractEvent.getEventFieldLabels(eventTypeLabel);
			}
			catch (Exception ex)
			{
				ex.printStackTrace(System.err);
			}
		}
		return result;
	}


	public Serializable getEventIterator(
			String testName,
			String bladeId,
			String eventTypeLabel,
			EventFilter filter)
	throws ClifException
	{
		eventIteratorKey = eventIteratorKey.add(BigInteger.ONE);
		eventIterators.put(
			eventIteratorKey,
			new EventIterator(testName, bladeId, eventTypeLabel, filter));
		return eventIteratorKey;
	}


	public BladeEvent[] getNextEvents(Serializable iteratorKey, int count)
		throws ClifException
	{
		EventIterator iter = eventIterators.get(iteratorKey);
		if (iter != null)
		{
			return iter.next(0, count, null);
		}
		else
		{
			throw new ClifException("Unknown event iterator key " + iteratorKey);
		}
	}


	public void closeEventIterator(Serializable iteratorKey)
	{
		EventIterator iter = eventIterators.remove(iteratorKey);
		if (iter != null)
		{
			iter.close();
		}
	}


	public BladeEvent[] getEvents(
		String testName,
		String bladeId,
		String eventTypeLabel,
		EventFilter filter,
		long fromIndex,
		int count)
	throws ClifException
	{
		EventIterator iter = new EventIterator(testName, bladeId, eventTypeLabel, filter);
		BladeEvent[] result = iter.next(fromIndex, count, null);
		iter.close();
		return result;
	}


	public long countEvents(
		String testName,
		String bladeId,
		String eventTypeLabel,
		EventFilter filter)
	throws ClifException
	{
		EventIterator iter = new EventIterator(testName, bladeId, eventTypeLabel, filter);
		LongHolder count = new LongHolder();
		iter.next(0, -1, count);
		iter.close();
		return count.value;
	}


	class EventIterator
	{
		BufferedReader reader;
		EventFilter filter;
		Class<?> clazz;
		EventFactory factory;


		EventIterator(String testName, String bladeId, String eventTypeLabel, EventFilter filter)
			throws ClifException
		{
			this.filter = filter;
			clazz = loadEventClass(testName, bladeId, eventTypeLabel);
			factory = AbstractEvent.getEventFactory(eventTypeLabel);
			try
			{
				reader = new BufferedReader(
					new FileReader(
						FileStorageCommons.getEventFile(testName, bladeId, eventTypeLabel)));	
			}
			catch (IOException ex)
			{
				throw new ClifException(
					"Unable to find " + eventTypeLabel + " events from blade " + bladeId + " in test " + testName,
					ex);
			}
		}


		BladeEvent[] next(long fromIndex, int n, LongHolder counting)
			throws ClifException
		{
			List<BladeEvent> eventList = null;
			long eventCount = 0;
			if (counting == null)
			{
				eventList = new LinkedList<BladeEvent>();
			}
			try
			{
				String line = reader.readLine();
				boolean isComment = FileStorageCommons.isCommentLine(line);
				while (line != null && (isComment || fromIndex > 0))
				{
					if (! isComment)
					{
						--fromIndex;
					}
					line = reader.readLine();
					isComment = FileStorageCommons.isCommentLine(line);
				}
				while (line != null && (counting != null || isComment || n != 0 || fromIndex < 0))
				{
					if (! isComment)
					{
						if (counting == null)
						{
							BladeEvent event = factory.makeEvent(
								AbstractEvent.DEFAULT_SEPARATOR,
								line);
							if (filter == null || filter.accept(event))
							{
								eventList.add(event);
								if (fromIndex < 0 && n == 0)
								{
									eventList.remove(0);
								}
								else
								{
									--n;
								}
							}
						}
						else
						{
							if (filter == null
								|| filter.accept(factory.makeEvent(
									AbstractEvent.DEFAULT_SEPARATOR,
									line)))
							{
								++eventCount;
							}
						}
					}
					line = reader.readLine();
					isComment = FileStorageCommons.isCommentLine(line);
				}
			}
			catch (IOException ex)
			{
				throw new ClifException("Could not get events", ex);
			}
			catch (NoMoreEvent ex)
			{
			}
			if (counting == null)
			{
				return eventList.toArray(new BladeEvent[eventList.size()]);
			}
			else
			{
				counting.value = eventCount;
				return null;
			}
		}


		void close()
		{
			try
			{
				reader.close();
			}
			catch (IOException ex)
			{
				ex.printStackTrace(System.err);
			}
		}
	}
}
