/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2007 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */
package org.ow2.clif.analyze.lib.graph;

import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.channels.FileChannel;
import java.util.Calendar;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Map.Entry;
import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.Legend;
import org.jfree.chart.title.TextTitle;
import org.jfree.chart.title.Title;
import org.ow2.clif.analyze.api.graph.ChartControl;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.ow2.clif.analyze.api.graph.ExportControl;
import org.ow2.clif.analyze.lib.graph.ChartData;
import org.ow2.clif.analyze.lib.graph.DatasetOptions;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;

/**
 * The Class ExportEngineImpl.
 * 
 * @author Jordan BRUNIER
 * @author Gregory CALONNIER
 * @author Olivier LIU
 */
public class ExportEngineImpl implements BindingController, ExportControl
{

	/** The instance. */
	private static ExportEngineImpl instance = new ExportEngineImpl();

	/** The interface list. */
	static private String[] itfList = new String[] {
			ChartControl.CHART_CONTROL, DataSupplier.DATA_SUPPLIER };

	// field for client binding (access to stored data)
	/** The data provider interface. */
	private DataSupplier dpItf;

	/** The chart control interface. */
	private ChartControl ccItf;

	/**
	 * Instantiates a new export engine impl.
	 */
	public ExportEngineImpl()
	{

	}

	/**
	 * Gets the single instance of ExportEngineImpl.
	 * 
	 * @return single instance of ExportEngineImpl
	 */
	public ExportEngineImpl getInstance()
	{
		return instance;
	}

	private String REPORT_ANALYZE = "ReportAnalyze";
	private String REPORT_COMMENT = "ReportComment";
	private String COMMENT = "Comment";
	private String PAGE = "Page";
	private String PAGE_ID = "id";
	private String PAGE_COMMENT = "PageComment";
	private String DATASET = "Dataset";
	private String DATASET_TEST_NAME = "TestName";
	private String DATASET_BLADE_ID = "BladeId";
	private String DATASET_FIELD = "Field";
	private String DATASET_TYPE_ATT = "Type";
	private String DATASET_COMMENT = "Comment";
	private String DATASET_TYPE = "DatasetType";
	private String STATISTIC = "Statistic";
	private String VALUE = "value";
	private String URL = "url";

	private String FILTER = "Filter";
	private String FIELD_FILTER = "Field";
	private String OPERATOR_FILTER = "Operator";
	private String KEYWORD_FILTER = "Keyword";

	private String TIME_INTERVAL = "TimeInterval";
	private String TIME_INTERVAL_START = "TimeIntervalStart";
	private String TIME_INTERVAL_END = "TimeIntervalEnd";

	public enum ReportObj
	{
		COMMENT, CHART, STATISTICS, PLOT, OTHER
	};

	// ///////////////////////////////
	// interface BindingController //
	// ///////////////////////////////

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.
	 * String, java.lang.Object)
	 */
	public void bindFc(String clientItfName, Object serverItf)
			throws NoSuchInterfaceException, IllegalBindingException,
			IllegalLifeCycleException
	{
		if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
		{
			dpItf = (DataSupplier) serverItf;
		} else if (clientItfName.equals(ChartControl.CHART_CONTROL))
		{
			ccItf = (ChartControl) serverItf;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.fractal.api.control.BindingController#listFc()
	 */
	public String[] listFc()
	{
		return itfList;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang
	 * .String)
	 */
	public Object lookupFc(String clientItfName)
			throws NoSuchInterfaceException
	{
		if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
		{
			return dpItf;
		} else if (clientItfName.equals(ChartControl.CHART_CONTROL))
		{
			return ccItf;
		} else
		{
			return null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang
	 * .String)
	 */
	public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException
	{
		if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
		{
			dpItf = null;
		} else if (clientItfName.equals(ChartControl.CHART_CONTROL))
		{
			ccItf = null;
		}
	}

	// ///////////////////////////
	// interface ExportControl //
	// ///////////////////////////

	public String getDate()
	{
		String date = " ";
		Calendar calendar = Calendar.getInstance();
		date += calendar.get(Calendar.YEAR) + "-";
		date += calendar.get(Calendar.MONTH) + 1 + "-";
		date += calendar.get(Calendar.DAY_OF_MONTH) + " ";
		date += calendar.get(Calendar.HOUR) + ":";
		date += calendar.get(Calendar.MINUTE) + " ";
		date += calendar.get(Calendar.HOUR) + ":";
		date += calendar.get(Calendar.SECOND) + " ";
		return date;
	}

	public String tabs(int nb)
	{
		String retString = " ";
		String tab = "  ";
		for (int i = 1; i <= nb; i++)
		{
			retString += tab;
		}
		return retString;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.ow2.clif.analyze.api.graph.ExportControl#ExportToHTML(java.lang.String
	 * , int, java.lang.String, java.lang.String, java.lang.String)
	 */
	public boolean exportToHTML(String fileName, int chartId,
			String imageFormat, String comment,
			Map<String, Map<String, Double>> statistics, String title)
	{
		FileWriter HTMLWriter = null;
		String tab = "\t";

		File folder = new File(fileName);

		File HTMLFile = new File(fileName + ".html");
		try
		{
			folder.mkdir();
			copyLogoAndCSS(folder);
			HTMLFile.createNewFile();

			writeChartImage(fileName + "/" + folder.getName(), imageFormat,
					chartId);

			HTMLWriter = new FileWriter(HTMLFile);
			String date = " ";
			Calendar calendar = Calendar.getInstance();
			date += calendar.get(Calendar.YEAR) + "-";
			date += calendar.get(Calendar.MONTH) + 1 + "-";
			date += calendar.get(Calendar.DAY_OF_MONTH) + " ";
			date += calendar.get(Calendar.HOUR) + ":";
			date += calendar.get(Calendar.MINUTE) + " ";
			date += (calendar.get(Calendar.AM_PM) == 0) ? "AM" : "PM" + " ";
			writeHTMLHeader(HTMLWriter, folder.getName(), title);

			HTMLWriter.write(tabs(2) + "exportToHTML-old(<br>\n");
			HTMLWriter.write(tabs(3) + "fileName:" + fileName + "<br>\n");
			HTMLWriter.write(tabs(3) + "chartId:" + chartId + "<br>\n");
			HTMLWriter.write(tabs(3) + "imageFormat:" + imageFormat + "<br>\n");
			HTMLWriter.write(tabs(3) + "comment:" + comment + "<br>\n");
			HTMLWriter.write(tabs(3) + "title:" + title + "<br>\n");
			HTMLWriter.write(tabs(2) + ");<br>\n");
			HTMLWriter.write(tabs(2) + "<hr/>\n");

			if (!title.trim().equals(""))
			{
				HTMLWriter.write(tab + tab + tab + tab + "<h3>" + title
						+ "</h3>\n");
			} else
			{
				HTMLWriter.write(tab + tab + tab + tab + "<h3>Chart</h3>\n");
			}
			if (!imageFormat.toUpperCase().equals("SVG"))
			{
				HTMLWriter
						.write(tab
								+ tab
								+ tab
								+ tab
								+ "<img src=\"./"
								+ folder.getName()
								+ "/"
								+ folder.getName()
								+ "."
								+ imageFormat.toLowerCase()
								+ "\" border=\"1\" alt=\"a chart\" align=\"middle\"><br>\n\n");
			} else
			{
				HTMLWriter.write(tab + tab + tab + tab
						+ "<object type='image/svg+xml' data='./"
						+ folder.getName() + "/" + folder.getName()
						+ ".svg' width='450' height='350'>\n" + "<img src=\"./"
						+ folder.getName() + "/" + folder.getName() + ".png\""
						+ " alt='a chart' width='450' height='350'>\n"
						+ "</object>\n\n");
			}
			HTMLWriter.write(tab + tab + tab + tab + "<h3>Comments</h3>\n\n");
			HTMLWriter.write(tab + tab + tab + tab + "<br>"
					+ comment.replaceAll("\n", "<br>") + "<br>\n\n");

			HTMLWriter.write(tab + tab + tab + tab + "<h3>Statistics</h3>\n");

			// Create Statistics table
			String[] allYAxis;
			if (null == statistics)
			{
				allYAxis = new String[0];
			} else
			{
				allYAxis = statistics.keySet().toArray(new String[0]);
			}
			String[] allStat;

			if (allYAxis.length > 0)
			{
				HTMLWriter.write("<br><table><tr>");
				// Table Header
				String[] allHeader = (String[]) statistics.get(allYAxis[0])
						.keySet().toArray(new String[0]);
				HTMLWriter.write("<th></th>");
				for (int i = 0; i < allHeader.length; i++)
				{
					HTMLWriter.write("<th>" + allHeader[i] + "</th>");
				}
				HTMLWriter.write("</tr>\n");

				// Table
				for (int i = 0; i < allYAxis.length; i++)
				{
					if (i % 2 == 0)
					{
						HTMLWriter.write("<tr class='pair'>");
					} else
					{
						HTMLWriter.write("<tr class='impair'>");
					}
					HTMLWriter.write("<td><b>" + allYAxis[i] + "<b></td>");

					allStat = (String[]) statistics.get(allYAxis[i]).keySet()
							.toArray(new String[0]);
					for (int j = 0; j < allStat.length; j++)
					{
						HTMLWriter.write("<td>"
								+ statistics.get(allYAxis[i]).get(allStat[j])
								+ "</td>");
					}
					HTMLWriter.write("</tr>\n");
				}
				HTMLWriter.write("</table>");
			}

			HTMLWriter.write(tab + tab + tab + tab + "</td>\n");
			HTMLWriter.write(tab + tab + tab + "</tr>\n");
			HTMLWriter.write(tab + tab + tab + "<tr>\n");

			HTMLWriter.write(tab + tab + tab + tab
					+ "<td valign=\"top\" width=\"15\">\n");
			HTMLWriter
					.write(tab
							+ tab
							+ tab
							+ tab
							+ tab
							+ "<img alt=\" \" height=\"1\" width=\"15\" src=\"./images/pix.gif\">\n");
			HTMLWriter.write(tab + tab + tab + tab + "</td>\n");
			HTMLWriter.write(tab + tab + tab + tab + "<td valign=\"top\">\n");
			HTMLWriter
					.write(tab
							+ tab
							+ tab
							+ tab
							+ tab
							+ "<img alt=\" \" height=\"1\" width=\"465\" src=\"./images/pix.gif\">\n");
			HTMLWriter.write(tab + tab + tab + tab + tab + "<address>\n");
			HTMLWriter.write(tab + tab + tab + tab + tab + tab + "Generated at"
					+ date + "\n");
			HTMLWriter.write(tab + tab + tab + tab + tab + "</address>\n");
			HTMLWriter.write(tab + tab + tab + tab + "</td>\n");
			HTMLWriter.write(tab + tab + tab + "</tr>\n");

			writeHTMLFooter(HTMLWriter);
			HTMLWriter.close();
			return true;
		} catch (IOException ioe)
		{
			ioe.printStackTrace(System.err);
		}
		return false;
	}

	FileWriter htmlWriter = null;
	boolean debugTrace = false;

	void tracelog(String mess) throws IOException
	{
		if (debugTrace)
		{
			htmlWriter.write(mess);
		}
	}

	private String convertToSymbol(String operator)
	{
		String OPERATOR_EQUALS = "equals";
		String OPERATOR_NOT_EQUALS = "not equals";
		String OPERATOR_CONTAINS = "contains";
		String OPERATOR_NOT_CONTAINS = "not contains";
		String OPERATOR_BEGINS_WITH = "begins with";
		String OPERATOR_ENDS_WITH = "ends with";
		String OPERATOR_GREATER_THAN = "greater than";
		String OPERATOR_LESS_THAN = "less than";
		String OPERATOR_GREATER_THAN_OR_EQUALS = "greater than or equals";
		String OPERATOR_LESS_THAN_OR_EQUALS = "less than or equals";

		if (operator.equals(OPERATOR_EQUALS))
		{
			return "=";
		} else if (operator.equals(OPERATOR_NOT_EQUALS))
		{
			return "&#8800;"; // not equal to
		} else if (operator.equals(OPERATOR_CONTAINS))
		{
			return "&#8835; (contains)"; // &sup; &#8835;
		} else if (operator.equals(OPERATOR_NOT_CONTAINS))
		{
			return "!&#8835; (not contains)";
		} else if (operator.equals(OPERATOR_BEGINS_WITH))
		{
			return "&#254; (begins with)"; // &thorn;
		} else if (operator.equals(OPERATOR_ENDS_WITH))
		{
			return "&#208; (ends with)"; // &#272; &Dstrok;
		} else if (operator.equals(OPERATOR_GREATER_THAN))
		{
			return ">";
		} else if (operator.equals(OPERATOR_LESS_THAN))
		{
			return "<";
		} else if (operator.equals(OPERATOR_GREATER_THAN_OR_EQUALS))
		{
			return ">=";
		} else if (operator.equals(OPERATOR_LESS_THAN_OR_EQUALS))
		{
			return "<=";
		}
		return null;
	}

	SortedSet<ReportSection> sections = new TreeSet<ReportSection>(
			new ReportSectionComparator());
	Map<Integer, List<StatisticDataImpl>> statisticMap;
	Map<String, String> commentMap;
	Map<Integer, JFreeChart> jFreeChartMap;
	Map<String, Integer> mapPlotNameStatIndex;

	/**
	 * export comments of one section To HTML
	 * 
	 * @param _sectionName
	 * @throws IOException
	 */
	private void exportToHTMLComment(String _sectionName) throws IOException
	{
		htmlWriter.write(tabs(3) + "<h3> " + _sectionName + "</h3>\n");
		htmlWriter.write(tabs(4)
				+ "<span class=\"left_h2\">"
				+ commentMap.get(_sectionName).replaceAll("\n",
						"<br>\n" + tabs(5)) + "</span></br>");
		htmlWriter.write("\n\n");
		htmlWriter.flush();
	}

	/**
	 * xport all comments To HTML
	 * 
	 * @throws IOException
	 */
	private void exportToHTMLComments() throws IOException
	{
		htmlWriter.write(tabs(2) + "<hr/>\n");
		htmlWriter.write(tabs(2) + "<h2>Comments</h2>\n");
		for (Iterator<ReportSection> it = sections.iterator(); it.hasNext();)
		{
			ReportSection key = it.next();
			switch (key.reportObjectType)
			{
			case COMMENT:
				exportToHTMLComment(key.sectionName);
				break;
			default:
				break;
			}
			htmlWriter.flush();
		}
		htmlWriter.flush();

	}

	/**
	 * export statistics from one dataset to HTML
	 * 
	 * @param sdiList
	 * @throws IOException
	 */
	private void exportToHTMLStatistic(List<StatisticDataImpl> sdiList)
			throws IOException
	{
		String testName = "<test>";
		String bladeName = "<blade>";
		String eventName = "<event>";
		String fieldName = "<field>";
		String dataSetType = "<dataSetType>";
		String filterDescription = "<filterDescription>";

		if (null != sdiList)
		{
			for (Iterator<StatisticDataImpl> itd = sdiList.iterator(); itd
					.hasNext();)
			{
				testName = "<test>";
				bladeName = "<blade>";
				eventName = "<event>";
				fieldName = "<field>";
				dataSetType = "<dataSetType>";
				filterDescription = "<filterDescription>";
				StatisticDataImpl sdi = itd.next();

				if (sdi.getDatasetType() == 0 || sdi.getDatasetType() == 1)
				{ // simple dataset and multiple dataset
					testName = sdi.getTestName();
					bladeName = sdi.getBladeId();
					eventName = sdi.getEventType();
					fieldName = sdi.getField();
					dataSetType = sdi.getDatasetType() == 0 ? "Simple"
							: "Multiple";
				} else if (sdi.getDatasetType() == 2)
				{ // Aggregate dataset
					testName = sdi.getTestName();
					bladeName = sdi.getBladeIdAlias();
					eventName = sdi.getEventType();
					fieldName = sdi.getField();
					dataSetType = sdi.getTestName() + ","
							+ sdi.getBladeIdAlias() + "," + sdi.getEventType();
				}

				// filters
				if (sdi.getFfo().size() > 0)
				{
					filterDescription = "";
					List<FieldFilterAndOp> ffo = sdi.getFfo();
					Iterator<FieldFilterAndOp> ffoIt = ffo.iterator();
					while (ffoIt.hasNext())
					{
						FieldFilterAndOp tmp = ffoIt.next();
						filterDescription += tmp.getField() + " "
								+ convertToSymbol(tmp.getOperator()) + " "
								+ tmp.getFilter();
					}

					if (ffo.size() > 1)
					{
						filterDescription += "Logical Operator used :"
								+ sdi.getLogicalOp();
					}
				} else
				{
					filterDescription += "No filter used";
				}

				// one html table with 2 embedded tables containing statistical
				// data
				htmlWriter
						.write(tabs(3)
								+ "<table class=\"table1_class\" cellpadding=\"10\" cellspacing=\"20\" border=\"5\"> \n");
				htmlWriter.write(tabs(4) + "<caption>Statistics for "
						+ testName + " - " + bladeName + " - " + eventName
						+ " - " + fieldName + " </caption>\n");
				htmlWriter.write(tabs(4) + "<tr>\n");
				htmlWriter.write(tabs(5) + "<td>\n");
				htmlWriter
						.write(tabs(6)
								+ "<table class=\"table1_class\" cellpadding=\"1\" cellspacing=\"2\" border=\"5\"> \n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter
						.write(tabs(8)
								+ "<th class=\"tab_header\" colspan=\"2\"> Dataset definition </th>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Dataset Type </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ dataSetType + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Test name </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ testName + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Blade </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ bladeName + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Event </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ eventName + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Field </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ fieldName + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Time Interval </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_filter\"> "
						+ sdi.gettimeIntervalStart() + " ms to "
						+ sdi.gettimeIntervalEnd() + " ms </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Filter </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_filter\"> "
						+ filterDescription + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(6) + "</table>\n");
				htmlWriter.write(tabs(5) + " </td>\n");

				htmlWriter.write(tabs(5) + "<td>\n");

				// headers
				htmlWriter
						.write(tabs(6)
								+ "<table class=\"table2_class\" cellpadding=\"1\" cellspacing=\"2\" border=\"5\"> \n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter
						.write(tabs(8)
								+ "<th class=\"tab_header\" colspan=\"2\"> Global Statistics</th>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");

				// events per second
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter
						.write(tabs(8)
								+ "<td class=\"tab_measure\"> Throughput (events per second)</td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ sdi.getThroughput() + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Minimum </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ sdi.getMinimum() + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Maximum </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ sdi.getMaximum() + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Mean </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ sdi.getMean() + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Median  </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ sdi.getMedian() + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Std deviation </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ sdi.getStdDvt() + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(7) + "<tr>\n");
				htmlWriter.write(tabs(8)
						+ "<td class=\"tab_measure\"> Nb. values </td>\n");
				htmlWriter.write(tabs(8) + "<td class=\"tab_value\"> "
						+ sdi.getNumberOfValues() + " </td>\n");
				htmlWriter.write(tabs(7) + "</tr>\n");
				htmlWriter.write(tabs(6) + "</table>\n");
				htmlWriter.write(tabs(5) + " </td>\n");
				htmlWriter.write(tabs(4) + "</tr>\n");
				htmlWriter.write(tabs(3) + "</table><br><br>\n");

				htmlWriter.flush();
			} // for(... sdiList.iterator ...)
		} // if (null != sdiList)
		else
		{
			htmlWriter.write(tabs(4) + "No statistics.<br>\n");
		}
		htmlWriter.flush();
	}

	/**
	 * exports statistics for all datasets to HTML
	 * 
	 * @throws IOException
	 */
	private void exportToHTMLStatistics() throws IOException
	{
		htmlWriter.write(tabs(2) + "<h2>Statistics</h2>\n");

		// must sweep in order
		for (Iterator<ReportSection> it = sections.iterator(); it.hasNext();)
		{
			ReportSection key = it.next();
			switch (key.reportObjectType)
			{
			case STATISTICS:
				Integer key1 = key.statisticMapIndex;
				exportToHTMLStatistic(statisticMap.get(key1));
				break;
			default:
				break;
			} // switch (key.reportObjectType){
		} // for (Iterator<ReportSection> ...
		tracelog(tabs(2) + "------------  end Statistics 2<br><hr/><br>\n\n");
		htmlWriter.flush();
	}

	void exportToHTMLChart(ReportSection key) throws IOException
	{
		htmlWriter
				.write(tabs(3)
						+ "<div style=\"page-break-before:always; margin-top:2cm;\">\n");
		htmlWriter.write(tabs(3) + "<h3> " + key.sectionName + "</h3>\n");
		htmlWriter.write(tabs(3) + "<img \n");
		htmlWriter.write(tabs(4) + "src=\"" + key.fileNameRelative + "\"\n");
		htmlWriter.write(tabs(4) + "alt=\"" + key.imageName + "\"\n");
		htmlWriter.write(tabs(4) + "title=\"" + key.imageName + "\"\n");
		htmlWriter.write(tabs(4) + "style=\"border:5px solid black\"\n");
		htmlWriter.write(tabs(4) + "align=\"middle\"\n");
		htmlWriter.write(tabs(3) + "/></div><br><br><br><br>\n\n\n");
		htmlWriter.flush();
	}

	void exportToHTMLCharts() throws IOException
	{
		htmlWriter.write(tabs(2) + "<hr/>\n");
		htmlWriter.write(tabs(2) + "<h2>Charts</h2>\n");

		for (Iterator<ReportSection> it = sections.iterator(); it.hasNext();)
		{
			ReportSection key = it.next();
			switch (key.reportObjectType)
			{
			case CHART:
				exportToHTMLChart(key);
				break;

			default:
				break;
			}
			htmlWriter.flush();
		}
	}

	void exportHTMLInOrder() throws IOException
	{
		for (Iterator<ReportSection> it = sections.iterator(); it.hasNext();)
		{
			ReportSection index = it.next();
			switch (index.reportObjectType)
			{
			case COMMENT:
				exportToHTMLComment(index.sectionName);
				break;

			case CHART:
				exportToHTMLChart(index);
				break;

			case STATISTICS:
//				Integer statsIndex = index.statisticMapIndex;
//				exportToHTMLStatistic(statisticMap.get(statsIndex));
				break;

			case PLOT:
				String plotName = index.plotName;
				Integer statiscticsIndex = mapPlotNameStatIndex.get(plotName);
				if (null == statisticMap.get(statiscticsIndex))
				{
					htmlWriter.write(tabs(3) + "Statistics for " + plotName
							+ ".<br>\n");
					Integer size = mapPlotNameStatIndex.size();
					for (int ix = 0; ix < size; ix++)
					{
						exportToHTMLStatistic(statisticMap.get(ix));
					}
				} else
				{
					exportToHTMLStatistic(statisticMap.get(statiscticsIndex));
				}
				break;

			default:
				break;
			}
			htmlWriter.flush();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.ow2.clif.analyze.api.graph.ExportControl#exportToXML(java.lang.String
	 * , java.util.Map, java.util.Map, java.util.Map, java.lang.String,
	 * java.util.Map, java.util.Map, java.util.Map)
	 */
	public boolean exportToHTML(String fileName,
			Map<Integer, List<StatisticDataImpl>> _statisticMap,
			Map<String, String> _commentMap,
			Map<Integer, JFreeChart> _jFreeChartMap, String imageType,
			Map<Integer, List<Integer>> pageIdAndDatasetIds,
			Map<Integer, ChartData> chartDataList,
			Map<Integer, DatasetOptions> datasetIdAndGraphOptions)
	{
		String title = "Clif Report";
		statisticMap = _statisticMap;
		commentMap = _commentMap;
		jFreeChartMap = _jFreeChartMap;
		mapPlotNameStatIndex = new HashMap();

		sections = new TreeSet<ReportSection>(new ReportSectionComparator());
		File htmlFile = new File(fileName + ".html");

		try
		{
			try
			{
				htmlFile.createNewFile();
				htmlWriter = new FileWriter(htmlFile);
			} catch (Exception e)
			{
				System.out.println("ERROR Creating HTML file \"" + htmlFile
						+ "\"");
				return false;
			}

			// html header
			htmlWriter
					.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 "
							+ "Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n");
			htmlWriter.write("<html>\n");
			htmlWriter.write(tabs(1) + "<head>\n");
			htmlWriter.write(tabs(2)
					+ "<META http-equiv=\"Content-Type\" content=\"text/html;"
					+ "charset=iso-8859-1\">\n");
			htmlWriter.write(tabs(2)
					+ "<meta content=\"text/html; charset=iso-8859-1\" "
					+ "http-equiv=\"Content-Type\">\n");
			htmlWriter.write(tabs(2)
					+ "<meta content=\"en\" http-equiv=\"Content-Language\">"
					+ "\n\n");
			htmlWriter.write(tabs(2) + "<title>" + title + "</title>\n\n");
			htmlWriter.write(tabs(2) + "<meta name=\"Robots\" "
					+ "content=\"noindex, follow\">\n");
			htmlWriter.write(tabs(2)
					+ "<meta content=\"CLIF team\" name=\"author\">\n");
			htmlWriter.write(tabs(2)
					+ "<meta content=\"clif@ow2.org\" name=\"email\">\n");
			htmlWriter.write(tabs(2) + "<style type='text/css'>\n");
			htmlWriter.write(tabs(3)
					+ "tr.impair {background-color:#ffffff;}\n");
			htmlWriter.write(tabs(3)
					+ "tr.pair   {background-color:#ffedcf;}\n");
			htmlWriter.write(tabs(3) + "caption {caption-side:bottom;}\n");

			htmlWriter.write(tabs(3) + "caption { caption-side: top }\n");
			htmlWriter.write(tabs(3)
					+ "th { text-align: center; font-weight: bold }\n");
			htmlWriter.write(tabs(3) + "td { vertical-align: middle }\n");
			htmlWriter.write(tabs(3) + "table  { border-collapse: collapse "
					+ "}\n");
			htmlWriter.write(tabs(3)
					+ ".filter_class {font-weight: bold;  color: red;}\n");
			htmlWriter.write(tabs(3) + ".tab_header {color: blue;}\n");
			htmlWriter.write(tabs(3) + ".tab_filter     {color: red;}\n");

			htmlWriter.write(tabs(2) + "</style>\n");
			htmlWriter.write(tabs(1) + "</head>\n");
			htmlWriter.write(tabs(1) + "\n");
			htmlWriter.write(tabs(1) + "<body bgcolor=\"white\">\n");

			// sorting data
			tracelog(tabs(2) + "<h2>sorting data</h2>\n");
			tracelog(tabs(2) + "<h2>Comments sweeping</h2>\n");

			tracelog(tabs(3) + "adding comments:<ul>\n");
			for (Iterator<String> it = commentMap.keySet().iterator(); it
					.hasNext();)
			{
				String sectionName = it.next();
				String commentText = (String) commentMap.get(sectionName);
				tracelog(tabs(4) + "<li>add comment: " + sectionName
						+ "</li>\n");
				sections.add(new ReportSection(ReportObj.COMMENT, sectionName,
						commentText));
				htmlWriter.flush();
			}
			tracelog(tabs(3) + "</ul>End comments<br><br>\n\n");

			tracelog(tabs(3)
					+ "Adding charts and generate associated files:<br><ul>\n");
			for (Iterator<Integer> it = jFreeChartMap.keySet().iterator(); it
					.hasNext();)
			{
				Integer sectionNumber = it.next();
				String sectionName = "[Report, Page " + sectionNumber + ", Chart]";

				File folder = new File(fileName + "_dir");
				folder.mkdir();
				String absolutePath = folder.getParent();
				String relativePath = ".";
				String baseName = folder.getName();
				String chartsFolderName = folder.getName();
				String ext = imageType.toLowerCase();
				String fs = File.separator;
				String imageName = baseName + "-Page" + sectionNumber;
				String fileNameAbsolute = absolutePath + fs + chartsFolderName
						+ fs + imageName; // for writeChartImage(..) method
				String fileNameAbsComplete = fileNameAbsolute + "." + ext;
				String fileNameRelative = relativePath + fs + chartsFolderName
						+ fs + imageName + "." + ext; // this is the link to put
				// inside HTML file
				tracelog(tabs(4) + "<li>adding chart: " + sectionName
						+ "  (section: " + sectionNumber + ",  imageName: \""
						+ imageName + "\") <br></li>\n");

				sections.add(new ReportSection(ReportObj.CHART, sectionName,
						sectionNumber, imageName, fileNameAbsolute,
						fileNameAbsComplete, fileNameRelative));

				JFreeChart chart = jFreeChartMap.get(sectionNumber);
				tracelog(tabs(4) + "<ul>\n");
				sectionName = "[Report, Page " + sectionNumber + ", Chart, PLOT]";
				for (int i = 0; i < chart.getXYPlot().getDatasetCount(); i++)
				{
					if (null != chart.getXYPlot().getDataset(i))
					{
						String plotName = chart.getXYPlot().getDataset(i)
								.getSeriesName(0);
						tracelog(tabs(5) + "<li>adding plot: " + sectionName
								+ "  (section: " + sectionNumber + ") \"" + plotName
								+ "\" <br></li>\n");
						sections.add(new ReportSection(ReportObj.PLOT,
								sectionName, plotName));
					}
				}
				tracelog(tabs(4) + "</ul>\n");

				if (writeChartImage(fileNameAbsolute, ext, chart))
				{
					tracelog(tabs(3) + "\n");
				} else
				{
					htmlWriter.write(tabs(3) + "*** ERROR *** "
							+ ".- generating file: " + fileNameAbsComplete
							+ "<br>\n");
				}
				htmlWriter.flush();
			}
			tracelog(tabs(3) + "</ul>End graphs<br><br>\n\n");
			htmlWriter.flush();

			tracelog(tabs(3) + "Add statistics:<br><ul>\n");
			for (Iterator<Integer> itStat = statisticMap.keySet().iterator(); itStat
					.hasNext();)
			{
				Integer statisticsIndex = itStat.next();
				tracelog(tabs(4) + "<ul>add:statisticMap( " + statisticsIndex
						+ ")\n");
				List<StatisticDataImpl> val1 = statisticMap
						.get(statisticsIndex);
				for (Iterator<StatisticDataImpl> itSDI = val1.iterator(); itSDI
						.hasNext();)
				{
					StatisticDataImpl sdi = itSDI.next();
					String plotName = sdi.getTestName() + " - "
							+ sdi.getBladeId() + " - " + sdi.getEventType()
							+ " - " + sdi.getField();
					tracelog(tabs(5) + "<li>add: " + plotName + "</li>\n");
//					sections.add(new ReportSection(plotName, ReportObj.STATISTICS, key1));
//					ReportSection rs = new ReportSection(ReportObj.STATISTICS, statisticsIndex,
//					plotName);
//					rs = new ReportSection(plotName, ReportObj.STATISTICS, statisticsIndex); 	//	old
//					sections.add(rs);
//					String sectionName = "unkown";
//					sections.add(new ReportSection(ReportObj.STATISTICS, sectionName, plotName,
					// statisticsIndex));
					mapPlotNameStatIndex.put(plotName, statisticsIndex);

					htmlWriter.flush();
				}
				tracelog(tabs(4) + "</ul>\n");
			}
			tracelog(tabs(3)
					+ "</ul style=\"page-break-before:always\">End statistics:<br><br>\n\n");
			htmlWriter.flush();

			tracelog(tabs(3) + "Table of Contents:<br><ul>\n");
			for (Iterator<ReportSection> it = sections.iterator(); it.hasNext();)
			{
				ReportSection key = it.next();
				switch (key.reportObjectType)
				{
				case COMMENT:
					tracelog(tabs(4) + "<li> " + key.sectionName
							+ " (comment)</li>\n");
					htmlWriter.flush();
					break;

				case CHART:
					tracelog(tabs(4) + "<li> " + key.sectionName + " \""
							+ key.imageName + "\" (chart)</li>\n");
					htmlWriter.flush();
					break;

				case PLOT:
					tracelog(tabs(4) + "<li> " + key.sectionName + " (plot) "
							+ key.plotName + " </li>\n");
					htmlWriter.flush();
					break;

				case STATISTICS:
					tracelog(tabs(4) + "<li> " + key.sectionName
							+ " (statistics)</li>\n");
					htmlWriter.flush();
					break;

				default:
					break;
				}
			}
			tracelog(tabs(3) + "</ul>\n");
			tracelog(tabs(2) + "<hr/>\n\n");
			htmlWriter.flush();
			// sorting data

			// start filling the Html file with the report
			boolean inOrder = true;
			if (inOrder)
			{
				exportHTMLInOrder();
			} else
			{
				exportToHTMLStatistics();
				exportToHTMLComments();
				exportToHTMLCharts();
			}

			htmlWriter.write(tabs(2) + "<hr/>\n");
			htmlWriter.write(tabs(2) + "End of report\n");
			htmlWriter.write(tabs(1) + "</body>\n");
			htmlWriter.write("</html>\n");

			htmlWriter.close();
			return true;
		} catch (Exception e)
		{
			try
			{
				e.printStackTrace();
				tracelog("Exception: " + e.getCause() + "\n");
				tracelog(e.getStackTrace()[0] + "<br>\n");
				tracelog(e.getStackTrace()[1] + "<br>\n");
				tracelog(e.getStackTrace()[2] + "<br>\n");
				tracelog("...<br>\n");
				htmlWriter.write(tabs(2)
						+ "<h2>*** ERROR *** generating  report</h2>\n\n");
				htmlWriter.write(tabs(1) + "</body>\n");
				htmlWriter.write("</html>\n");
				htmlWriter.close();
			} catch (IOException e1)
			{
				e1.printStackTrace();
			}
			return false;
		}
	}

	// fonction used (OLIVIER)
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.ow2.clif.analyze.api.graph.ExportControl#exportToText(java.lang.String
	 * , java.util.Map, java.util.Map)
	 */
	public boolean exportToText(String fileName,
			Map<Integer, List<StatisticDataImpl>> statisticMap,
			Map<String, String> commentMap,
			Map<Integer, JFreeChart> jFreeChartMap, String imageType)
	{
		// From exportToText(... <same parameters> ...)
		// export graphs
		Set<Entry<Integer, JFreeChart>> setChartMap = jFreeChartMap.entrySet();
		Iterator<Entry<Integer, JFreeChart>> iteratorCM = setChartMap
				.iterator();
		while (iteratorCM.hasNext())
		{
			Map.Entry<Integer, JFreeChart> meCM = iteratorCM.next();
			String imageName = fileName + "-Page" + meCM.getKey();
			writeChartImage(imageName, imageType, meCM.getValue());
		}

		File commentsFile = new File(fileName + "-comments.txt");
		File statsFile = new File(fileName + "-stats.txt");
		FileWriter commentsWriter = null;
		FileWriter statsWriter = null;
		try
		{
			commentsFile.createNewFile();
			statsFile.createNewFile();

			commentsWriter = new FileWriter(commentsFile);
			statsWriter = new FileWriter(statsFile);

			// Comments
			Set<Entry<String, String>> setComments = commentMap.entrySet();
			Iterator<Entry<String, String>> iteratorComments = setComments
					.iterator();
			while (iteratorComments.hasNext())
			{
				Map.Entry<String, String> me = iteratorComments.next();
				commentsWriter.write(me.getKey());
				commentsWriter.write("\n");
				commentsWriter.write(me.getValue());
				commentsWriter.write("\n \n \n");
			}

			// Statistics
			Set<Entry<Integer, List<StatisticDataImpl>>> setDP = statisticMap
					.entrySet();
			Iterator<Entry<Integer, List<StatisticDataImpl>>> iteratorDP = setDP
					.iterator();
			while (iteratorDP.hasNext())
			{
				Map.Entry<Integer, List<StatisticDataImpl>> me = iteratorDP
						.next();

				List<StatisticDataImpl> sdiList = me.getValue();

				Iterator<StatisticDataImpl> it = sdiList.iterator();
				while (it.hasNext())
				{
					StatisticDataImpl sdi = it.next();

					statsWriter
							.write("---------------------------------------------\n");

					// simple dataset and multiple dataset
					if (sdi.getDatasetType() == 0 || sdi.getDatasetType() == 1)
					{
						statsWriter.write(sdi.getTestName() + ","
								+ sdi.getBladeId() + "," + sdi.getEventType());
					}
					// Aggregate dataset
					else if (sdi.getDatasetType() == 2)
					{
						statsWriter.write(sdi.getTestName() + ","
								+ sdi.getBladeIdAlias() + ","
								+ sdi.getEventType());
					}
					if (sdi.getField() != null && sdi.getField() != "")
					{
						statsWriter.write("," + sdi.getField());
						if (sdi.getStolSize() > 0)
						{
							statsWriter.write("\n");

							// Time Interval
							statsWriter.write("\nTime Interval: ");
							statsWriter.write(sdi.gettimeIntervalStart()
									+ " ns to ");
							statsWriter.write(sdi.gettimeIntervalEnd()
									+ " ns\n");

							// filter
							if (sdi.getFfo().size() > 0)
							{
								statsWriter.write("\nFilters used :\n");
								List<FieldFilterAndOp> ffo = sdi.getFfo();
								Iterator<FieldFilterAndOp> ffoIt = ffo
										.iterator();
								while (ffoIt.hasNext())
								{
									FieldFilterAndOp tmp = ffoIt.next();
									statsWriter.write("Operator : "
											+ tmp.getField() + " "
											+ tmp.getOperator() + " "
											+ tmp.getFilter() + " \n");
								}

								if (ffo.size() > 1)
								{
									statsWriter.write("Logical Operator used :"
											+ sdi.getLogicalOp());
									statsWriter.write("\n");
								}
							} else
							{
								statsWriter.write("\nNo filter used\n");
							}

							statsWriter.write("\n");

							statsWriter.write(
							// request per second
									"Throughput : " + sdi.getThroughput()
											+ "\n" + "Minimum :"
											+ sdi.getMinimum() + "\n"
											+ "Maximum :" + sdi.getMaximum()
											+ "\n" + "Mean :" + sdi.getMean()
											+ "\n" + "Median :"
											+ sdi.getMedian() + "\n"
											+ "Standard deviation :"
											+ sdi.getStdDvt() + "\n"
											+ "Number of values :"
											+ (int) sdi.getNumberOfValues()
											+ "\n");
						} else
						{
							statsWriter
									.write("\n"
											+ "Non numeric field chosen for the dataset or no result found\n");
						}
					} else
					{
						statsWriter.write("\n" + "Throughput : "
								+ sdi.getThroughput() + "\n"
								+ "Number of values :"
								+ (int) sdi.getNumberOfValues() + "\n");
					}
					statsWriter.write("\n"
							+ "=======================================\n\n");
				}
			}
			commentsWriter.close();
			statsWriter.close();
			return true;
		} catch (Exception e)
		{
			return false;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.ow2.clif.analyze.api.graph.ExportControl#ExportToXML(java.lang.String
	 * , int, java.lang.String, java.lang.String, java.lang.String)
	 */
	public boolean exportToXML(String fileName, int chartId,
			String imageFormat, String comment,
			Map<String, Map<String, Double>> statistics, String title)
	{
// // File folder = new File(fileName);
//
// writeChartImage(fileName, imageFormat, chartId);
//
// Element root = new Element("chart");
// root.setAttribute("title", title);
//
// // Creation of a new JDOM Document based on root that we just have
// // created
// org.jdom.Document document = new org.jdom.Document(root);
//
// Element image = new Element("image");
// root.addContent(image);
//
// Attribute fichier = new Attribute("file", "./"
// + fileName.substring(fileName.lastIndexOf("/") + 1, fileName
// .length()) + "." + imageFormat.toLowerCase());
// image.setAttribute(fichier);
//
// Element comments = new Element("comments");
// comments.setText(comment);
// root.addContent(comments);
//
// // Create Statistics
// Element stats = new Element("statistics");
// String[] allYAxis = statistics.keySet().toArray(new String[0]);
// String[] allStat;
//
// if (allYAxis.length > 0)
// {
// Element serie;
// Attribute serieName;
// Element measure;
// Attribute measureName;
// Attribute measureValue;
// for (int i = 0; i < allYAxis.length; i++)
// {
// serie = new Element("serie");
// serieName = new Attribute("name", allYAxis[i]);
// serie.setAttribute(serieName);
//
// allStat = (String[]) statistics.get(allYAxis[i]).keySet()
// .toArray(new String[0]);
// for (int j = 0; j < allStat.length; j++)
// {
// measure = new Element("measure");
// measureName = new Attribute("name", allStat[j]);
// measureValue = new Attribute("value", statistics.get(
// allYAxis[i]).get(allStat[j]).toString());
// measure.setAttribute(measureName);
// measure.setAttribute(measureValue);
// serie.addContent(measure);
// }
// stats.addContent(serie);
// }
// }
//
// root.addContent(stats);
//
// Element generationDate = new Element("generation");
// String date = "";
// Calendar calendar = Calendar.getInstance();
// date += calendar.get(Calendar.YEAR) + "-";
// date += calendar.get(Calendar.MONTH) + 1 + "-";
// date += calendar.get(Calendar.DAY_OF_MONTH) + " ";
// date += calendar.get(Calendar.HOUR) + ":";
// date += calendar.get(Calendar.MINUTE) + " ";
// date += (calendar.get(Calendar.AM_PM) == 0) ? "AM" : "PM";
// generationDate.setText(date);
// root.addContent(generationDate);
//
// try
// {
// // classic display with getPrettyFormat()
// XMLOutputter sortie = new XMLOutputter(Format.getPrettyFormat());
//
// // Note that it only requires to create a FileOutputStream instance
// // with argument the filename to serialize
// sortie.output(document, new FileOutputStream(fileName + ".xml"));
// return true;
// } catch (java.io.IOException e)
// {
		return false;
// }
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.ow2.clif.analyze.api.graph.ExportControl#exportToXML(java.lang.String
	 * , java.util.Map, java.util.Map, java.util.Map, java.lang.String,
	 * java.util.Map, java.util.Map, java.util.Map)
	 */
	public boolean exportToXML(String fileName,
			Map<Integer, List<StatisticDataImpl>> statisticMap,
			Map<String, String> commentMap,
			Map<Integer, JFreeChart> jFreeChartMap, String imageType,
			Map<Integer, List<Integer>> pageIdAndDatasetIds,
			Map<Integer, ChartData> chartDataList,
			Map<Integer, DatasetOptions> datasetIdAndGraphOptions)
	{
		// File folder = new File(fileName);
		// export graphs
		Set<Entry<Integer, JFreeChart>> setChartMap = jFreeChartMap.entrySet();
		Iterator<Entry<Integer, JFreeChart>> iteratorCM = setChartMap
				.iterator();
		while (iteratorCM.hasNext())
		{
			Map.Entry<Integer, JFreeChart> meCM = iteratorCM.next();
			String imageName = fileName + "-Page" + meCM.getKey();
			writeChartImage(imageName, imageType, meCM.getValue());
		}

		// Report
		Element racine = new Element(REPORT_ANALYZE);

		String reportComString = commentMap.get("[Report]");
		if (reportComString != null)
		{
			Attribute reportComment = new Attribute(COMMENT, reportComString);
			racine.setAttribute(reportComment);
		}

		// Creation of a new JDOM Document based on root that we just have
		// created
		org.jdom.Document document = new org.jdom.Document(racine);

		// Pages
		Element page;
		Attribute pageAtt;
		Attribute pageCommAtt;
		Set<Entry<Integer, List<Integer>>> setDataset = pageIdAndDatasetIds
				.entrySet();
		Iterator<Entry<Integer, List<Integer>>> iteratorDataset = setDataset
				.iterator();
		while (iteratorDataset.hasNext())
		{
			Map.Entry<Integer, List<Integer>> meDS = iteratorDataset.next();
			int currentPage = meDS.getKey();
			page = new Element(PAGE);
			pageAtt = new Attribute(PAGE_ID, String.valueOf(currentPage));
			page.setAttribute(pageAtt);
			List<Integer> datasetList = meDS.getValue();

			// Dataset
			Element dataset;

			Iterator<Integer> iterator = datasetList.iterator();
			while (iterator.hasNext())
			{
				Integer currentDataset = iterator.next();
				List<StatisticDataImpl> statList = statisticMap
						.get(currentDataset);
				Iterator<StatisticDataImpl> sdiIt = statList.iterator();

				dataset = new Element(DATASET);

				// Dataset Attributes
				Attribute datasetTestName;
				Attribute datasetAliasId;
				Attribute datasetField;
				Attribute datasetCommentAtt;
				boolean oneTimeFilter = true;

				while (sdiIt.hasNext())
				{
					StatisticDataImpl sdi = sdiIt.next();
					String datasetTestNameN = sdi.getTestName();
					String datasetAliasIdN;
					String datasetCom;
					if (sdi.getDatasetType() == 0)
					{
						datasetAliasIdN = sdi.getBladeId();
					} else
					{
						datasetAliasIdN = sdi.getBladeIdAlias();
					}
					String datasetFieldN = sdi.getField();
					if (datasetFieldN == null)
					{
						datasetFieldN = "";
					}
					datasetTestName = new Attribute(DATASET_TEST_NAME,
							datasetTestNameN);
					datasetAliasId = new Attribute(DATASET_BLADE_ID,
							datasetAliasIdN);
					datasetField = new Attribute(DATASET_FIELD, datasetFieldN);

					dataset.setAttribute(datasetTestName);
					dataset.setAttribute(datasetAliasId);
					dataset.setAttribute(datasetField);
					datasetCom = commentMap.get("[Report, Page "
							+ String.valueOf(currentPage) + ", "
							+ datasetAliasIdN + "]");
					if (datasetCom != null)
					{
						datasetCommentAtt = new Attribute(COMMENT, datasetCom);
						dataset.setAttribute(datasetCommentAtt);
					}

					Attribute datasetTypeAtt;
					if (sdi.getDatasetType() == 0)
					{
						datasetTypeAtt = new Attribute(DATASET_TYPE_ATT,
								"Simple");
					} else if (sdi.getDatasetType() == 1)
					{
						datasetTypeAtt = new Attribute(DATASET_TYPE_ATT,
								"Multiple");
					} else
					{
						datasetTypeAtt = new Attribute(DATASET_TYPE_ATT,
								"Aggregate");
					}
					dataset.setAttribute(datasetTypeAtt);

					// Time Interval
					Element timeInterval;
					Attribute timeIntervalStartAtt;
					Attribute timeIntervalEndAtt;

					timeInterval = new Element(TIME_INTERVAL);
					timeIntervalStartAtt = new Attribute(TIME_INTERVAL_START,
							String.valueOf(sdi.gettimeIntervalStart()));

					timeIntervalEndAtt = new Attribute(TIME_INTERVAL_END,
							String.valueOf(sdi.gettimeIntervalEnd()));
					timeInterval.setAttribute(timeIntervalStartAtt);
					timeInterval.setAttribute(timeIntervalEndAtt);

					dataset.addContent(timeInterval);

					// Filter
					while (oneTimeFilter == true)
					{
						// Filter
						Element filter;

						List<FieldFilterAndOp> ffo = sdi.getFfo();
						if (ffo != null)
						{
							Attribute fieldFilterAtt;
							Attribute operatorFilterAtt;
							Attribute keywordFilterAtt;

							if (ffo.size() > 0)
							{

								// Field, filter and keyword
								String fieldFilter;
								String operatorFilter;
								String keywordFilter;
								Iterator<FieldFilterAndOp> ffoIt = ffo
										.iterator();
								while (ffoIt.hasNext())
								{
									filter = new Element(FILTER);
									FieldFilterAndOp ffoTmp = ffoIt.next();
									fieldFilter = ffoTmp.getField();
									operatorFilter = ffoTmp.getOperator();
									keywordFilter = ffoTmp.getFilter();

									fieldFilterAtt = new Attribute(
											FIELD_FILTER, fieldFilter);
									operatorFilterAtt = new Attribute(
											OPERATOR_FILTER, operatorFilter);
									keywordFilterAtt = new Attribute(
											KEYWORD_FILTER, keywordFilter);

									filter.setAttribute(fieldFilterAtt);
									filter.setAttribute(operatorFilterAtt);
									filter.setAttribute(keywordFilterAtt);
									dataset.addContent(filter);
								}
								if (ffo.size() > 1)
								{
									Element logicalOp = new Element(
											"LogicalOperator");
									if (sdi.getLogicalOp() == "and")
									{
										logicalOp.setAttribute("Operator",
												"and");
									} else
									{
										logicalOp
												.setAttribute("Operator", "or");
									}
									dataset.addContent(logicalOp);
								}
							}
						}
						oneTimeFilter = false;
					}

				}
				// Statistics
				Element statistics;

				Element throughput;
				Attribute throughputAtt;
				Element minimum;
				Attribute minimumAtt;
				Element maximum;
				Attribute maximumAtt;
				Element mean;
				Attribute meanAtt;
				Element median;
				Attribute medianAtt;
				Element standardDeviation;
				Attribute standardDeviationAtt;
				Element nbValues;
				Attribute nbValuesAtt;
				Element bladeId;
				Attribute bladeIdAtt;

				Iterator<StatisticDataImpl> statIterator = statList.iterator();
				while (statIterator.hasNext())
				{
					StatisticDataImpl statisticDataImpl = statIterator.next();
					statistics = new Element("Statistics");
					if (statisticDataImpl.getDatasetType() == 0
							|| statisticDataImpl.getDatasetType() == 1)
					{
						bladeIdAtt = new Attribute(VALUE, statisticDataImpl
								.getBladeId());
					} else
					{
						bladeIdAtt = new Attribute(VALUE, statisticDataImpl
								.getBladeIdAlias());
					}
					bladeId = new Element("BladeName");
					throughput = new Element("Throughput");
					minimum = new Element("Minimum");
					maximum = new Element("Maximum");
					mean = new Element("Mean");
					median = new Element("Median");
					standardDeviation = new Element("StandardDeviation");
					nbValues = new Element("NumberOfValues");
					throughputAtt = new Attribute(VALUE, String
							.valueOf(statisticDataImpl.getThroughput()));
					minimumAtt = new Attribute(VALUE, String
							.valueOf(statisticDataImpl.getMinimum()));
					maximumAtt = new Attribute(VALUE, String
							.valueOf(statisticDataImpl.getMaximum()));
					meanAtt = new Attribute(VALUE, String
							.valueOf(statisticDataImpl.getMean()));
					medianAtt = new Attribute(VALUE, String
							.valueOf(statisticDataImpl.getMedian()));
					standardDeviationAtt = new Attribute(VALUE, String
							.valueOf(statisticDataImpl.getStdDvt()));
					nbValuesAtt = new Attribute(VALUE, String
							.valueOf(statisticDataImpl.getNumberOfValues()));

					bladeId.setAttribute(bladeIdAtt);
					throughput.setAttribute(throughputAtt);
					minimum.setAttribute(minimumAtt);
					maximum.setAttribute(maximumAtt);
					mean.setAttribute(meanAtt);
					median.setAttribute(medianAtt);
					standardDeviation.setAttribute(standardDeviationAtt);
					nbValues.setAttribute(nbValuesAtt);

					statistics.addContent(bladeId);
					statistics.addContent(throughput);
					statistics.addContent(minimum);
					statistics.addContent(maximum);
					statistics.addContent(median);
					statistics.addContent(mean);
					statistics.addContent(standardDeviation);
					statistics.addContent(nbValues);

					dataset.addContent(statistics);
				}
				page.addContent(dataset);
			}
			racine.addContent(page);
			String sectionComment = commentMap.get("[Report, Page "
					+ String.valueOf(currentPage) + "]");
			if (sectionComment != null)
			{
				pageCommAtt = new Attribute(COMMENT, sectionComment);
				page.setAttribute(pageCommAtt);
			}

			JFreeChart jfc = jFreeChartMap.get(currentPage);
			if (jfc != null)
			{
				Element chart = new Element("Chart");
				String chartAttString = fileName + "-Page"
						+ String.valueOf(currentPage) + "."
						+ imageType.toLowerCase();
				Attribute chartAtt = new Attribute(URL, chartAttString);
				chart.setAttribute(chartAtt);
				page.addContent(chart);
			}
		}
		Element generationDate = new Element("generation");
		String date = "";
		Calendar calendar = Calendar.getInstance();
		date += calendar.get(Calendar.YEAR) + "-";
		date += calendar.get(Calendar.MONTH) + 1 + "-";
		date += calendar.get(Calendar.DAY_OF_MONTH) + " ";
		date += calendar.get(Calendar.HOUR) + ":";
		date += calendar.get(Calendar.MINUTE) + " ";
		date += (calendar.get(Calendar.AM_PM) == 0) ? "AM" : "PM";
		generationDate.setText(date);
		racine.addContent(generationDate);

		try
		{
			// classic display with getPrettyFormat()
			XMLOutputter sortie = new XMLOutputter(Format.getPrettyFormat());

			// Note that it only requires to create a FileOutputStream instance
			// with argument the filename to serialize
			sortie.output(document, new FileOutputStream(fileName + ".xml"));
			return true;
		} catch (java.io.IOException e)
		{
			return false;
		}
	}

	// ////////////////////
	// private function //
	// ////////////////////

	/**
	 * Write chart image.
	 * 
	 * @param fileName
	 *            the file name
	 * @param imageFormat
	 *            the image format
	 * @param chartId
	 *            the chart id
	 * 
	 * @return true, if successful
	 */
	private boolean writeChartImage(String fileName, String imageFormat,
			int chartId)
	{
		BufferedImage img = ccItf.getChartImage(chartId, 700, 400);
		try
		{
			File file = new File(fileName + "." + imageFormat.toLowerCase());
			FileOutputStream output = new FileOutputStream(file);
			if (imageFormat.equalsIgnoreCase("PNG"))
			{
				ChartUtilities.writeBufferedImageAsPNG(output, img);
				return true;
			} else if (imageFormat.equalsIgnoreCase("JPG"))
			{
				ChartUtilities.writeBufferedImageAsJPEG(output, img);
				return true;
			} else if (imageFormat.equalsIgnoreCase("JPEG"))
			{
				ChartUtilities.writeBufferedImageAsJPEG(output, img);
				return true;
			} else if (imageFormat.equalsIgnoreCase("SVG"))
			{
				DOMImplementation domImpl = GenericDOMImplementation
						.getDOMImplementation();
				Document document = domImpl.createDocument(null, "svg", null);
				SVGGraphics2D svgGenerator = new SVGGraphics2D(document);
				Writer out = new OutputStreamWriter(output, "UTF-8");

				ChartPanel cp = (ChartPanel) ccItf.getChart(chartId);
				cp.getChart().draw(svgGenerator,
						new Rectangle2D.Double(0, 0, 400, 300), null);

				svgGenerator.stream(out, true);
				return true;
			}
			return false;
		} catch (Exception e)
		{
			return false;
		}
	}

	/**
	 * OLIVIER Write chart image.
	 * 
	 * @param fileName
	 *            the file name
	 * @param imageFormat
	 *            the image format
	 * @param chartId
	 *            the chart id
	 * 
	 * @return true, if successful
	 */
	private boolean writeChartImage(String fileName, String imageFormat,
			JFreeChart jFreeChart)
	{
		BufferedImage img = jFreeChart.createBufferedImage(700, 400);
		try
		{
			File file = new File(fileName + "." + imageFormat.toLowerCase());
			FileOutputStream output = new FileOutputStream(file);
			if (imageFormat.equalsIgnoreCase("PNG"))
			{
				ChartUtilities.writeBufferedImageAsPNG(output, img);
				output.close();
				return true;
			} else if (imageFormat.equalsIgnoreCase("JPG"))
			{
				ChartUtilities.writeBufferedImageAsJPEG(output, img);
				output.close();
				return true;
			} else if (imageFormat.equalsIgnoreCase("JPEG"))
			{
				ChartUtilities.writeBufferedImageAsJPEG(output, img);
				output.close();
				return true;
			} else if (imageFormat.equalsIgnoreCase("SVG"))
			{
				DOMImplementation domImpl = GenericDOMImplementation
						.getDOMImplementation();
				Document document = domImpl.createDocument(null, "svg", null);
				SVGGraphics2D svgGenerator = new SVGGraphics2D(document);
				Writer out = new OutputStreamWriter(output, "UTF-8");

				jFreeChart.draw(svgGenerator, new Rectangle2D.Double(0, 0, 400,
						300), null);

				svgGenerator.stream(out, true);
				output.close();
				return true;
			}
			output.close();
			return false;
		} catch (Exception e)
		{
			e.printStackTrace(); // n'importe quelle exception
			return false;
		}
	}

	/**
	 * Write HTML header.
	 * 
	 * @param HTMLFile
	 *            the HTML file
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private void writeHTMLHeader(FileWriter HTMLFile, String folder,
			String title) throws IOException
	{
		String tab = "\t";

		HTMLFile
				.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n");
		HTMLFile.write("<html>\n");
		HTMLFile.write(tab + "<head>\n");
		HTMLFile
				.write(tab
						+ tab
						+ "<META http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
		HTMLFile
				.write(tab
						+ tab
						+ "<meta content=\"text/html; charset=iso-8859-1\" http-equiv=\"Content-Type\">\n");
		HTMLFile.write(tab + tab
				+ "<meta content=\"en\" http-equiv=\"Content-Language\">\n\n");
		HTMLFile.write(tab + tab + "<title>" + title + "</title>\n\n");
		HTMLFile.write(tab + tab
				+ "<meta name=\"Robots\" content=\"noindex, follow\">\n");
		HTMLFile.write(tab + tab
				+ "<meta content=\"CLIF team\" name=\"author\">\n");
		HTMLFile.write(tab + tab
				+ "<meta content=\"clif@ow2.org\" name=\"email\">\n");
		HTMLFile.write(tab + tab + "<link type=\"text/css\" href=\"./" + folder
				+ "/common.css\" rel=\"stylesheet\" id=\"stylesheet\">\n");
		HTMLFile
				.write(tab
						+ tab
						+ "<style type='text/css'> tr.impair {background-color:#ffffff;} tr.pair {background-color:#ffedcf;} </style>\n");
		HTMLFile.write(tab + "</head>\n\n");
		HTMLFile.write(tab + "<body bgcolor=\"white\">\n");
		HTMLFile
				.write(tab
						+ tab
						+ "<table cellpadding=\"0\" cellspacing=\"0\" width=\"100%\" border=\"0\">\n");
		HTMLFile.write(tab + tab + tab + "<tr>\n");
		HTMLFile.write(tab + tab + tab + tab
				+ "<td valign=\"top\" width=\"15\">\n");
		HTMLFile
				.write(tab
						+ tab
						+ tab
						+ tab
						+ tab
						+ "<img alt=\" \" height=\"1\" width=\"15\" src=\"./images/pix.gif\">\n");
		HTMLFile.write(tab + tab + tab + tab + "</td>\n");
		HTMLFile.write(tab + tab + tab + tab
				+ "<td class=\"contenu\" valign=\"top\">\n");
		HTMLFile.write(tab + tab + tab + tab + tab
				+ "<!-- end of writeHTMLHeader(...) -->\n");
	}

	/**
	 * Write HTML footer.
	 * 
	 * @param HTMLFile
	 *            the HTML file
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private void writeHTMLFooter(FileWriter HTMLFile) throws IOException
	{
		String tab = "\t";
		HTMLFile.write(tab + tab + "</table>\n");
		HTMLFile.write(tab + "</body>\n");
		HTMLFile.write("</html>\n");
	}

	/**
	 * Copy logo and CSS.
	 * 
	 * @param dest
	 *            the dest folder
	 */
	private void copyLogoAndCSS(File dest)
	{
		File logo = new File("./doc/clifhtmldoc/images/logo_clif_100px.gif");
		File css = new File("./doc/rom/style/style.css");

		String destFile = dest.getAbsolutePath() + "/logo_clif_100px.gif";
		File newLogo = new File(destFile);

		destFile = dest.getAbsolutePath() + "/common.css";
		File newCSS = new File(destFile);

		FileChannel in = null; // input stream
		FileChannel out = null; // output stream

		try
		{
			// Init
			newLogo.createNewFile();
			in = new FileInputStream(logo).getChannel();
			out = new FileOutputStream(newLogo).getChannel();

			// Copy from "in" to "out"
			in.transferTo(0, in.size(), out);

			in.close();
			out.close();

			newCSS.createNewFile();
			in = new FileInputStream(css).getChannel();
			out = new FileOutputStream(newCSS).getChannel();

			// Copy from "in" to "out"
			in.transferTo(0, in.size(), out);

		} catch (Exception e)
		{
			e.printStackTrace(); // any exception
		} finally
		{ // close the input stream
			if (in != null)
			{
				try
				{
					in.close();
				} catch (IOException e)
				{
				}
			}
			if (out != null)
			{
				try
				{
					out.close();
				} catch (IOException e)
				{
				}
			}
		}
	}

	/**
	 * Save Routine As XML
	 */
	public boolean saveRoutineAsXML(String fileName,
			Map<Integer, List<StatisticDataImpl>> statisticMap,
			Map<String, String> commentMap,
			Map<Integer, JFreeChart> jFreeChartMap, String imageType,
			Map<Integer, List<Integer>> pageIdAndDatasetIds,
			Map<Integer, ChartData> chartDataList,
			Map<Integer, DatasetOptions> datasetIdAndGraphOptions)
	{
		// File folder = new File(fileName);
		// export graphs
		Set<Entry<Integer, JFreeChart>> setChartMap = jFreeChartMap.entrySet();
		Iterator<Entry<Integer, JFreeChart>> iteratorCM = setChartMap
				.iterator();
		while (iteratorCM.hasNext())
		{
			Map.Entry<Integer, JFreeChart> meCM = iteratorCM.next();
			String imageName = fileName + "-Page" + meCM.getKey();
			writeChartImage(imageName, imageType, meCM.getValue());
		}

		// Report
		Element racine = new Element(REPORT_ANALYZE);

		// Creation of a new JDOM Document based on root that we just have
		// created
		org.jdom.Document document = new org.jdom.Document(racine);

		// Pages
		Element page;
		Attribute pageAtt;
		Attribute pageCommAtt;
		Set<Entry<Integer, List<Integer>>> setDataset = pageIdAndDatasetIds
				.entrySet();
		Iterator<Entry<Integer, List<Integer>>> iteratorDataset = setDataset
				.iterator();
		while (iteratorDataset.hasNext())
		{
			Map.Entry<Integer, List<Integer>> meDS = iteratorDataset.next();
			int currentPage = meDS.getKey();
			page = new Element(PAGE);
			pageAtt = new Attribute(PAGE_ID, String.valueOf(currentPage));
			page.setAttribute(pageAtt);
			List<Integer> datasetList = meDS.getValue();

			// Dataset
			Element dataset;

			Iterator<Integer> iterator = datasetList.iterator();
			while (iterator.hasNext())
			{
				Integer currentDataset = iterator.next();
				List<StatisticDataImpl> statList = statisticMap
						.get(currentDataset);
				Iterator<StatisticDataImpl> sdiIt = statList.iterator();

				dataset = new Element(DATASET);

				// Dataset Attributes
				Attribute datasetTestName;
				Attribute datasetAliasId;
				Attribute datasetField;
				Attribute datasetCommentAtt;
				boolean oneTimeFilter = true;

				while (sdiIt.hasNext())
				{
					StatisticDataImpl sdi = sdiIt.next();
					String datasetTestNameN = sdi.getTestName();
					String datasetAliasIdN;
					String datasetCom;
					if (sdi.getDatasetType() == 0)
					{
						datasetAliasIdN = sdi.getBladeId();
					} else
					{
						datasetAliasIdN = sdi.getBladeIdAlias();
					}
					String datasetFieldN = sdi.getField();
					if (datasetFieldN == null)
					{
						datasetFieldN = "";
					}
					datasetTestName = new Attribute(DATASET_TEST_NAME,
							datasetTestNameN);
					datasetAliasId = new Attribute(DATASET_BLADE_ID,
							datasetAliasIdN);
					datasetField = new Attribute(DATASET_FIELD, datasetFieldN);

					dataset.setAttribute(datasetTestName);
					dataset.setAttribute(datasetAliasId);
					dataset.setAttribute(datasetField);
					datasetCom = commentMap.get("[Report, Page "
							+ String.valueOf(currentPage) + ", "
							+ datasetAliasIdN + "]");
					if (datasetCom != null)
					{
						datasetCommentAtt = new Attribute(COMMENT, datasetCom);
						dataset.setAttribute(datasetCommentAtt);
					}

					Attribute datasetTypeAtt;
					if (sdi.getDatasetType() == 0)
					{
						datasetTypeAtt = new Attribute(DATASET_TYPE_ATT,
								"Simple");
					} else if (sdi.getDatasetType() == 1)
					{
						datasetTypeAtt = new Attribute(DATASET_TYPE_ATT,
								"Multiple");
					} else
					{
						datasetTypeAtt = new Attribute(DATASET_TYPE_ATT,
								"Aggregate");
					}
					dataset.setAttribute(datasetTypeAtt);

					while (oneTimeFilter == true)
					{
						// Filter
						Element filter;

						List<FieldFilterAndOp> ffo = sdi.getFfo();
						if (ffo != null)
						{
							Attribute fieldFilterAtt;
							Attribute operatorFilterAtt;
							Attribute keywordFilterAtt;

							if (ffo.size() > 0)
							{

								// Field, filter and keyword
								String fieldFilter;
								String operatorFilter;
								String keywordFilter;
								Iterator<FieldFilterAndOp> ffoIt = ffo
										.iterator();
								while (ffoIt.hasNext())
								{
									filter = new Element(FILTER);
									FieldFilterAndOp ffoTmp = ffoIt.next();
									fieldFilter = ffoTmp.getField();
									operatorFilter = ffoTmp.getOperator();
									keywordFilter = ffoTmp.getFilter();

									fieldFilterAtt = new Attribute(
											FIELD_FILTER, fieldFilter);
									operatorFilterAtt = new Attribute(
											OPERATOR_FILTER, operatorFilter);
									keywordFilterAtt = new Attribute(
											KEYWORD_FILTER, keywordFilter);

									filter.setAttribute(fieldFilterAtt);
									filter.setAttribute(operatorFilterAtt);
									filter.setAttribute(keywordFilterAtt);
									dataset.addContent(filter);
								}
								if (ffo.size() > 1)
								{
									Element logicalOp = new Element(
											"LogicalOperator");
									if (sdi.getLogicalOp() == "and")
									{
										logicalOp.setAttribute("Operator",
												"and");
									} else
									{
										logicalOp
												.setAttribute("Operator", "or");
									}
									dataset.addContent(logicalOp);
								}
							}
						}
						oneTimeFilter = false;
					}
				}
				page.addContent(dataset);
			}
			racine.addContent(page);
			String sectionComment = commentMap.get("[Report, Page "
					+ String.valueOf(currentPage) + "]");
			if (sectionComment != null)
			{
				pageCommAtt = new Attribute(COMMENT, sectionComment);
				page.setAttribute(pageCommAtt);
			}
		}
		Element generationDate = new Element("generation");
		String date = "";
		Calendar calendar = Calendar.getInstance();
		date += calendar.get(Calendar.YEAR) + "-";
		date += calendar.get(Calendar.MONTH) + 1 + "-";
		date += calendar.get(Calendar.DAY_OF_MONTH) + " ";
		date += calendar.get(Calendar.HOUR) + ":";
		date += calendar.get(Calendar.MINUTE) + " ";
		date += (calendar.get(Calendar.AM_PM) == 0) ? "AM" : "PM";
		generationDate.setText(date);
		racine.addContent(generationDate);

		try
		{
			// classic display with getPrettyFormat()
			XMLOutputter sortie = new XMLOutputter(Format.getPrettyFormat());

			// Note that it only requires to create a FileOutputStream instance
			// with argument the filename to serialize
			sortie.output(document, new FileOutputStream(fileName + ".xml"));
			return true;
		} catch (java.io.IOException e)
		{
			return false;
		}
	}

	class ReportSectionComparator implements Comparator<ReportSection>
	{

		public int compare(ReportSection _first, ReportSection _second)
		{
			int res = _first.compareTo(_second);
			// System.out.println("      \""+_first.sectionName + "\" ? \"" +
			// _second.sectionName + "\" => " + res);
			return res;
		}
	}

	// This data structure holds all the report sections names with the indexes
	// to retrieve their
	// associated data (comments, charts, statistics, ...)
	// sections are sorted according to their name and thus retrieved in (that)
	// order

	class ReportSection implements Comparable<ReportSection>
	{
		String sectionName;
		ReportObj reportObjectType;

		// for comments
		String commentIndex;

		// for charts
		Integer pageNumber;
		String imageName;
		String fileNameAbsolute;
		String fileNameAbsComplete;
		String fileNameRelative;

		// for statistics
		Integer statisticMapIndex;
		int statisticDataImplIndex;

		// for plot
		String plotName;
		Integer page;

		ReportSection(String _sectionName, String _commentIndex)
		{
			sectionName = _sectionName;
			commentIndex = _commentIndex;
			reportObjectType = ReportObj.COMMENT;
		}

		public ReportSection(ReportObj _reportObjectType, String _sectionName,
				String val)
		{
			switch (_reportObjectType)
			{
			case COMMENT:
				sectionName = _sectionName;
				commentIndex = val;
				reportObjectType = _reportObjectType;
				break;

			case STATISTICS:
				sectionName = _sectionName;
				commentIndex = val;
				reportObjectType = _reportObjectType;
				break;

			case PLOT:
				sectionName = _sectionName;
				plotName = val;
				reportObjectType = _reportObjectType;
				break;

			default:
				break;
			}
		}

		public ReportSection(ReportObj _reportObjectType, String _sectionName,
				String _plotName, Integer _statisticsIndex)
		{
			switch (_reportObjectType)
			{
			case STATISTICS:
				sectionName = _sectionName;
				commentIndex = _plotName;
				statisticMapIndex = _statisticsIndex;
				reportObjectType = _reportObjectType;
				break;

			default:
				break;
			}
		}

		ReportSection(String _sectionName, Integer _chartIndex)
		{
			sectionName = _sectionName;
			pageNumber = _chartIndex;
			reportObjectType = ReportObj.CHART;
		}

		ReportSection(ReportObj _reportObjectType, String _sectionName,
				Integer _pageNumber, String _imageName,
				String _fileNameAbsolute, String _fileNameAbsComplete,
				String _fileNameRelative)
		{
			switch (_reportObjectType)
			{
			case CHART:
				sectionName = _sectionName;
				pageNumber = _pageNumber;
				imageName = _imageName;
				fileNameAbsolute = _fileNameAbsolute;
				fileNameAbsComplete = _fileNameAbsComplete;
				fileNameRelative = _fileNameRelative;
				reportObjectType = _reportObjectType;
				break;

			default:
				break;
			}
		}

		ReportSection(String _sectionName, ReportObj _reportObjectType,
				Integer _statisticMapIndex)
		{
			switch (_reportObjectType)
			{
			case STATISTICS:
				sectionName = _sectionName;
				statisticMapIndex = _statisticMapIndex;
				reportObjectType = _reportObjectType;
				plotName = null;
				break;

			default:
				break;
			}
		}

		public ReportSection(ReportObj _reportObjectType, Integer _page,
				String _plotName)
		{
			switch (_reportObjectType)
			{
			case PLOT:
				page = _page;
				plotName = _plotName;
				reportObjectType = _reportObjectType;
				break;

			default:
				break;
			}
		}

		ReportSection(String _sectionName)
		{
			sectionName = _sectionName;
			reportObjectType = ReportObj.OTHER;
		}

		public int compareTo(ReportSection _section)
		{
			int ret;

			System.out.print("     convert \"" + this.sectionName + "\" -> ");
			String a = this.sectionName.replace(",", " ");
			String a1 = a.replace('[', ' ');
			String a2 = a1.replace(']', ' ');
			System.out.println("\"" + a2 + "\"");

			System.out.print("     convert \"" + _section.sectionName
					+ "\" -> ");
			String b = _section.sectionName.replace(',', ' ');
			String b1 = b.replace('[', ' ');
			String b2 = b1.replace(']', ' ');
			System.out.println("\"" + b2 + "\"");

			ret = a2.compareTo(b2);
			System.out.print("   comp3 \"" + a2 + "\" to \"" + b2 + "\"  => "
					+ ret);
			if (ret == 0)
			{
				System.out.println(" equal");
				if (hashCode() > _section.hashCode())
					return 1;
				else if (this == _section)
					return 0;
				else if (hashCode() < _section.hashCode())
					return -1;
				else
					System.out
							.println("same hashcode for 2 different objects.");
			} else
			{
				System.out.println((ret > 0) ? " to right" : " to left");
			}
			return ret;
		}

	}

	public int compareTo(ReportSection o)
	{
		return 0;
	}

}
