/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2007, 2008 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */

package org.ow2.clif.analyze.lib.graph.ui;

import java.awt.Color;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import javax.swing.JInternalFrame;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.tree.DefaultMutableTreeNode;
import org.jfree.chart.JFreeChart;
import org.ow2.clif.analyze.api.graph.ChartControl;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.ow2.clif.analyze.lib.graph.ChartData;
import org.ow2.clif.analyze.lib.graph.DatasetOptions;
import org.ow2.clif.analyze.api.graph.ExportControl;
import org.ow2.clif.analyze.api.graph.VirtualDataSupplier;
import org.ow2.clif.analyze.lib.graph.AnalyzerImpl;
import org.ow2.clif.analyze.lib.graph.StatisticDataImpl;
import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.BladeFilter;
import org.ow2.clif.supervisor.api.ClifException;

/**
 * The Class SwingGUI.
 * 
 * @author Jordan BRUNIER
 * @author Gregory CALONNIER
 * @author Olivier Liu
 * @author Bruno Dillenseger
 */
public class SwingGUI
{

	private static final String DEFAULT_X_FIELD_LABEL = "date";

	/***** Properties *****/
	private AnalyzerImpl analyzer;

	private JInternalFrame quickViewFrame;
	private DataQuickViewContainer quickViewContainer;
	private JMenuBar quickViewMenu;

	
	private DataSupplier dataSupplier;
	private VirtualDataSupplier virtualDataSupplier;
	private ChartControl chart;
	private int chartID;
	private ExportControl export;
	private SimpleDataset simpleDataset;

	// Moving Statistic Preferences
	private int timeWindow = 100000;
	private int step = 50000;

		
/**
	 * Instantiates a new swing GUI.
	 * 
	 * @param frame
	 *            the frame
	 * @param analyzer
	 *            the analyzer
	 * 
	 * @throws ClifException
	 *             the clif exception
	 */
	public SwingGUI(JInternalFrame frame, AnalyzerImpl analyzer)
			throws ClifException
	{
		this.analyzer = analyzer;
		this.quickViewFrame = frame;
		this.quickViewContainer = new DataQuickViewContainer(this);
		this.quickViewMenu = new DataQuickViewMenu(this);

		// GUI setup
		this.quickViewFrame.add(this.quickViewContainer);
		this.quickViewFrame.setJMenuBar(this.quickViewMenu);
		this.quickViewFrame.pack();
		this.quickViewContainer.validate();

		// Create Export Control
		this.export = ((ExportControl) analyzer
				.lookupFc(ExportControl.EXPORT_CONTROL));

		// Create new chart
		this.chart = ((ChartControl) analyzer
				.lookupFc(ChartControl.CHART_CONTROL));
		chartID = chart.createNewChart(null);

		// Create VirtualDataSupplier
		this.virtualDataSupplier = (VirtualDataSupplier) analyzer
				.lookupFc(VirtualDataSupplier.VIRTUAL_DATA_SUPPLIER);

		// Setup new chart
		this.chart.setXAxis(chartID, "date");
		this.chart.setXAxisLabel(chartID, "Date (ms)");

		this.simpleDataset = getSimpleDataset();
	}

	// ------------------------ Public Methods ----------------------//
	/**
	 * Gets the internal frame.
	 * 
	 * @return the internal frame
	 */
	public JInternalFrame getInternalFrame()
	{
		return this.quickViewFrame;
	}

	/**
	 * Sets the moving statistic time window.
	 * 
	 * @param timeWindow
	 *            the new time window
	 */
	public void setTimeWindow(int timeWindow)
	{
		this.timeWindow = timeWindow;
	}

	/**
	 * Sets the moving statistic step.
	 * 
	 * @param step
	 *            the new step
	 */
	public void setStep(int step)
	{
		this.step = step;
	}

	/**
	 * Gets the moving statistic time window.
	 * 
	 * @return the time window
	 */
	public int getTimeWindow()
	{
		return this.timeWindow;
	}

	/**
	 * Gets the moving statistic step.
	 * 
	 * @return the step
	 */
	public int getStep()
	{
		return this.step;
	}

	public DataQuickViewContainer getQuickViewContainer()
	{
		return this.quickViewContainer;
	}

	// ------------------------ Package Methods ----------------------//
	/**
	 * Gets the initial execution tree.
	 * 
	 * @return the initial execution tree
	 */
	DefaultMutableTreeNode getInitialExecutionTree()
	{
		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("CLIF");
		DefaultMutableTreeNode childNode;
		DefaultMutableTreeNode childChildNode;
		DefaultMutableTreeNode childChildChildNode;
		DefaultMutableTreeNode childChildChildChildNode;
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		// Build Execution Tree
		String[] testsNames = dataSupplier.getTestsNames();
		for (int iTest = 0; iTest < testsNames.length; iTest++)
		{
			childNode = new DefaultMutableTreeNode(testsNames[iTest]);

			String[] baldesIds = dataSupplier.getBladesIds(testsNames[iTest]);
			if (baldesIds != null)
			{
				for (int iBlade = 0; iBlade < baldesIds.length; iBlade++)
				{

					childChildNode = new DefaultMutableTreeNode(
							baldesIds[iBlade]);

					String[] eventsTypeLabels = dataSupplier
							.getEventsTypeLabels(testsNames[iTest],
									baldesIds[iBlade]);
					if (eventsTypeLabels != null)
					{
						for (int iEvent = 0; iEvent < eventsTypeLabels.length; iEvent++)
						{
							childChildChildNode = new DefaultMutableTreeNode(
									eventsTypeLabels[iEvent]);

							String[] eventsFieldLabels = dataSupplier
									.getEventFieldLabels(testsNames[iTest],
											baldesIds[iBlade],
											eventsTypeLabels[iEvent]);

							if (eventsFieldLabels != null)
							{
								for (int iField = 0; iField < eventsFieldLabels.length; iField++)
								{
									if (eventsFieldLabels[iField] != null)
									{
										childChildChildChildNode = new DefaultMutableTreeNode(
												eventsFieldLabels[iField]);
										childChildChildNode
												.add(childChildChildChildNode);
									}
								}
							}
							childChildNode.add(childChildChildNode);
						}
					}

					childNode.add(childChildNode);
				}
			}

			rootNode.add(childNode);
		}

		return rootNode;
	}

	// ------------------------ Package Methods ----------------------//
	/**
	 * Gets the initial execution tree.
	 * 
	 * @return the initial execution tree
	 */
	DefaultMutableTreeNode getBladeExecutionTree()
	{
		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("CLIF");
		DefaultMutableTreeNode childNode;
		DefaultMutableTreeNode childChildNode;
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		// Build Execution Tree
		String[] testsNames = dataSupplier.getTestsNames();
		for (int iTest = 0; iTest < testsNames.length; iTest++)
		{
			childNode = new DefaultMutableTreeNode(testsNames[iTest]);

			String[] baldesIds = dataSupplier.getBladesIds(testsNames[iTest]);
			if (baldesIds != null)
			{
				for (int iBlade = 0; iBlade < baldesIds.length; iBlade++)
				{

					childChildNode = new DefaultMutableTreeNode(
							baldesIds[iBlade]);
					childNode.add(childChildNode);
				}
			}
			rootNode.add(childNode);
		}
		return rootNode;
	}

	/**
	 * Refresh execution tree.
	 * 
	 * @param rootNode
	 *            the root node
	 * 
	 * @return the default mutable tree node
	 */
	DefaultMutableTreeNode refreshExecutionTree(DefaultMutableTreeNode rootNode)
	{
		DefaultMutableTreeNode childNode;
		DefaultMutableTreeNode childChildNode;
		DefaultMutableTreeNode childChildChildNode;
		DefaultMutableTreeNode childChildChildChildNode;
		int iTest;
		int iNode;

		String[] testsNames = dataSupplier.getTestsNames();
		int[] deleteNode = new int[rootNode.getChildCount()];
		Arrays.fill(deleteNode, 0);

		for (iTest = 0; iTest < testsNames.length; iTest++)
		{
			// Find existing node
			for (iNode = 0; iNode < rootNode.getChildCount(); iNode++)
			{
				if (testsNames[iTest].equals(rootNode.getChildAt(iNode)
						.toString()))
				{
					// Node exists
					deleteNode[iNode] = 1;
					break;
				}
			}

			// Add a new node
			if (iNode == rootNode.getChildCount())
			{
				childNode = new DefaultMutableTreeNode(testsNames[iTest]);

				String[] baldesIds = dataSupplier
						.getBladesIds(testsNames[iTest]);
				if (baldesIds != null)
				{
					for (int iBlade = 0; iBlade < baldesIds.length; iBlade++)
					{
						childChildNode = new DefaultMutableTreeNode(
								baldesIds[iBlade]);

						String[] eventsTypeLabels = dataSupplier
								.getEventsTypeLabels(testsNames[iTest],
										baldesIds[iBlade]);
						if (eventsTypeLabels != null)
						{
							for (int iEvent = 0; iEvent < eventsTypeLabels.length; iEvent++)
							{
								childChildChildNode = new DefaultMutableTreeNode(
										eventsTypeLabels[iEvent]);
								String[] eventsFieldLabels = dataSupplier
										.getEventFieldLabels(testsNames[iTest],
												baldesIds[iBlade],
												eventsTypeLabels[iEvent]);
								if (eventsFieldLabels != null)
								{
									for (int iField = 0; iField < eventsFieldLabels.length; iField++)
									{
										childChildChildChildNode = new DefaultMutableTreeNode(
												eventsFieldLabels[iField]);
										childChildChildNode
												.add(childChildChildChildNode);
									}
								}
								childChildNode.add(childChildChildNode);
							}
						}

						childNode.add(childChildNode);
					}
				}

				rootNode.add(childNode);
			}
		}

		// Delete old node
		for (int j = 0; j < deleteNode.length; j++)
		{
			if (deleteNode[j] == 0)
			{
				rootNode.remove(j);
			}
		}

		return rootNode;
	}

	/**
	 * Add an Y axis in chart.
	 * 
	 * @param testName
	 *            the test name
	 * @param bladeId
	 *            the blade id
	 * @param eventTypeLabel
	 *            the event type label
	 * @param fieldLabel
	 *            the field label
	 */
	void addYAxisInChart(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel)
	{
		chart.addYAxis(chartID, testName, bladeId, eventTypeLabel, fieldLabel);
	}

	/**
	 * Gets the chart panel.
	 * 
	 * @return the chart panel
	 */
	JPanel getChartPanel()
	{
		return chart.getChart(chartID);
	}

	/**
	 * Get chart line color.
	 * 
	 * @param testName
	 *            the test name
	 * @param bladeId
	 *            the blade id
	 * @param eventTypeLabel
	 *            the event type label
	 * @param fieldLabel
	 *            the field label
	 * 
	 * @return the color
	 */
	Color getChartLineColor(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel)
	{
		Color color = chart.getColor(this.chartID, testName, bladeId,
				eventTypeLabel, fieldLabel);

		return color;
	}

	/**
	 * Sets the chart line color.
	 * 
	 * @param testName
	 *            the test name
	 * @param bladeId
	 *            the blade id
	 * @param eventTypeLabel
	 *            the event type label
	 * @param fieldLabel
	 *            the field label
	 * @param color
	 *            the color
	 */
	void setChartLineColor(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, Color color)
	{
		chart.setColor(this.chartID, testName, bladeId, eventTypeLabel,
				fieldLabel, color);
	}

	/**
	 * Sets the X axis label.
	 * 
	 * @param label
	 *            the new X axis label
	 */
	void setXAxisLabel(String label)
	{
		chart.setXAxisLabel(chartID, label);
	}

	/**
	 * Removes the Y axis.
	 * 
	 * @param testName
	 *            the test name
	 * @param bladeId
	 *            the blade id
	 * @param eventTypeLabel
	 *            the event type label
	 * @param fieldLabel
	 *            the field label
	 */
	void removeYAxis(String testName, String bladeId, String eventTypeLabel,
			String fieldLabel)
	{
		chart.removeYAxis(this.chartID, testName, bladeId, eventTypeLabel,
				fieldLabel);
	}

	/**
	 * Gets the charts types.
	 * 
	 * @return the charts types
	 */
	Map getChartsTypes()
	{
		return chart.getRendererList();
	}

	/**
	 * Sets the chart type.
	 * 
	 * @param testName
	 *            the test name
	 * @param bladeId
	 *            the blade id
	 * @param eventTypeLabel
	 *            the event type label
	 * @param fieldLabel
	 *            the field label
	 * @param rendererType
	 *            the renderer type
	 */
	void setChartType(String testName, String bladeId, String eventTypeLabel,
			String fieldLabel, String rendererType)
	{
		chart.setRenderer(this.chartID, testName, bladeId, eventTypeLabel,
				fieldLabel, rendererType);
	}

//	/**
//	 * Export to HTML.
//	 * 
//	 * @param fileName
//	 *            the file name
//	 * @param imageFormat
//	 *            the image format
//	 * @param comment
//	 *            the comment
//	 * @param statistics
//	 *            the statistics
//	 */
//	void exportToHTML(String fileName, String imageFormat, String comment,
//			Map<String, Map<String, Double>> statistics, String chartTitle)
//	{
//		this.export.exportToHTML(fileName, this.chartID, imageFormat, comment,
//				statistics, chartTitle);
//	}

	/**
	 * export To HTML
	 * 
	 * @param fileName
	 * @param statisticMap
	 * @param commentMap
	 * @param jFreeChartMap
	 * @param imageType
	 * @param pageIdAndDatasetIds
	 * @param chartDataList
	 * @param datasetIdAndGraphOptions
	 * @return true, if successful
	 */
	public boolean exportToHTML(String fileName,
			Map<Integer, List<StatisticDataImpl>> statisticMap,
			Map<String, String> commentMap,
			Map<Integer, JFreeChart> jFreeChartMap, String imageType,
			Map<Integer, List<Integer>> pageIdAndDatasetIds,
			Map<Integer, ChartData> chartDataList,
			Map<Integer, DatasetOptions> datasetIdAndGraphOptions)
	{
		return export.exportToHTML(fileName, statisticMap, commentMap,
				jFreeChartMap, imageType, pageIdAndDatasetIds, chartDataList,
				datasetIdAndGraphOptions);
	}


	/**
	 * export To XML
	 * 
	 * @param fileName
	 * @param statisticMap
	 * @param commentMap
	 * @param jFreeChartMap
	 * @param imageType
	 * @param pageIdAndDatasetIds
	 * @param chartDataList
	 * @param datasetIdAndGraphOptions
	 * @return true, if successful
	 */
	public boolean exportToXML(String fileName,
			Map<Integer, List<StatisticDataImpl>> statisticMap,
			Map<String, String> commentMap,
			Map<Integer, JFreeChart> jFreeChartMap, String imageType,
			Map<Integer, List<Integer>> pageIdAndDatasetIds,
			Map<Integer, ChartData> chartDataList,
			Map<Integer, DatasetOptions> datasetIdAndGraphOptions)
	{
		return export.exportToXML(fileName, statisticMap, commentMap,
				jFreeChartMap, imageType, pageIdAndDatasetIds, chartDataList,
				datasetIdAndGraphOptions);
	}

	/**
	 * Save Routine As XML
	 */
	public boolean saveRoutineAsXML(String fileName,
			Map<Integer, List<StatisticDataImpl>> statisticMap,
			Map<String, String> commentMap,
			Map<Integer, JFreeChart> jFreeChartMap, String imageType,
			Map<Integer, List<Integer>> pageIdAndDatasetIds,
			Map<Integer, ChartData> chartDataList,
			Map<Integer, DatasetOptions> datasetIdAndGraphOptions)
	{
		return export.saveRoutineAsXML(fileName, statisticMap, commentMap,
				jFreeChartMap, imageType, pageIdAndDatasetIds, chartDataList,
				datasetIdAndGraphOptions);
	}

	/**
	 * 
	 * Export to text.
	 * 
	 * @param fileName
	 * @param statisticMap
	 * @param commentMap
	 * @param jFreeChartMap
	 * @param imageType
	 */
	void exportToText(String fileName,
			Map<Integer, List<StatisticDataImpl>> statisticMap,
			Map<String, String> commentMap,
			Map<Integer, JFreeChart> jFreeChartMap, String imageType)
	{
		this.export.exportToText(fileName, statisticMap, commentMap,
				jFreeChartMap, imageType);
	}

	public Map<Integer, List<StatisticDataImpl>> getStatisticMap()
	{
		return this.quickViewContainer.getStatisticMap();
	}

	public Map<String, Map<String, Double>> getStatistics()
	{
		// TODO Auto-generated method stub
		// to be defined
		return null;
	}

	public Map<String, String> getMapTreeComment()
	{
		return this.quickViewContainer.getMapTreeComment();
	}

	public String getComment()
	{
		return this.quickViewContainer.getComment();
	}

	public String getChartTitle()
	{
		return new String("Chart Title (to be defined)");
	}

	public Map<Integer, JFreeChart> getJFreeChartMap()
	{
		return this.quickViewContainer.getJFreeChartMap();
	}

	/**
	 * Sets the moving statistic.
	 * 
	 * @param testName
	 *            the test name
	 * @param bladeId
	 *            the blade id
	 * @param eventTypeLabel
	 *            the event type label
	 * @param fieldLabel
	 *            the field label
	 * @param movingStatistic
	 *            the moving statistic
	 */
	void setYAxisMovingStatistics(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel, String movingStatistic)
	{
		chart.setYAxisMovingStatistics(this.chartID, testName, bladeId,
				eventTypeLabel, fieldLabel, movingStatistic, this.timeWindow,
				this.step);
	}

	/**
	 * Sets the Y axis label.
	 * 
	 * @param testName
	 *            the test name
	 * @param bladeId
	 *            the blade id
	 * @param eventTypeLabel
	 *            the event type label
	 * @param fieldLabel
	 *            the field label
	 * @param label
	 *            the label
	 */
	void setYAxisLabel(String testName, String bladeId, String eventTypeLabel,
			String fieldLabel, String label)
	{
		chart.setYAxisLabel(this.chartID, testName, bladeId, eventTypeLabel,
				fieldLabel, label);
	}

	/**
	 * Gets the statistics for.
	 * 
	 * @param testName
	 *            the test name
	 * @param bladeId
	 *            the blade id
	 * @param eventTypeLabel
	 *            the event type label
	 * @param yFieldLabel
	 *            the y field label
	 * 
	 * @return the statistics for
	 */
	Map getStatisticsFor(String testName, String bladeId,
			String eventTypeLabel, String yFieldLabel)
	{
		return dataSupplier.getStatistics(testName, bladeId, eventTypeLabel,
				DEFAULT_X_FIELD_LABEL, yFieldLabel, chart
						.getLowerLimit(this.chartID), chart
						.getUpperLimit(this.chartID), chart
						.getMaxNumberOfPoint(this.chartID));
	}

	/**
	 * Sets the domain range.
	 * 
	 * @param lower
	 *            the lower
	 * @param upper
	 *            the upper
	 */
	void setDomainRange(int lower, int upper)
	{
		chart.setDomainRange(this.chartID, lower, upper);
	}

	/**
	 * Sets the max number of point.
	 * 
	 * @param value
	 *            the new max number of point
	 */
	void setMaxNumberOfPoint(int value)
	{
		chart.setMaxNumberOfPoint(this.chartID, value);
	}

	/**
	 * Gets the max time.
	 * 
	 * @param testName
	 *            the test name
	 * @param bladeId
	 *            the blade id
	 * @param eventTypeLabel
	 *            the event type label
	 * 
	 * @return the max time
	 */
	long getMaxTimeFor(String testName, String bladeId)
	{
		return dataSupplier.getMaxTimeFor(testName, bladeId);
	}

	/**
	 * Gets the chart lower limit.
	 * 
	 * @return the chart lower limit
	 */
	int getChartLowerLimit()
	{
		return chart.getLowerLimit(this.chartID);
	}

	/**
	 * Gets the chart upper limit.
	 * 
	 * @return the chart upper limit
	 */
	int getChartUpperLimit()
	{
		return chart.getUpperLimit(this.chartID);
	}

	public void aggregateBlades()
	{

	}

	public long getCommonMinTime(String testName)
	{
		long result = virtualDataSupplier.getCommonMinTime(testName);
		return result;
	}

	public long getCommonMaxTime(String testName)
	{
		long result = virtualDataSupplier.getCommonMaxTime(testName);
		return result;

	}

	public long getMaxTime(String testName, String bladeId)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getMaxTimeFor(testName, bladeId);
	}

	public String[] getTestNames()
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getTestsNames();
	}

	public String[] getBladeIds(String testName)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getBladesIds(testName);
	}

	public String[] getEventTypes(String testName, String bladeId)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getEventsTypeLabels(testName, bladeId);
	}

	public String[] getFieldLabels(String testName, String bladeId,
			String eventTypeLabel)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getEventFieldLabels(testName, bladeId,
				eventTypeLabel);
	}

	public Object[][] getAllFieldsValues(String testName, String bladeId,
			String eventTypeLabel, String XFieldLabel, String YFieldLabel,
			int start, int end, int count)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getAllFieldsValues(testName, bladeId,
				eventTypeLabel, XFieldLabel, YFieldLabel, start, end, count);
	}

	public long getLifecycleRunning(String testName, String bladeId)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getLifecycleRunning(testName, bladeId);
	}

	public long getLifecycleCompleted(String testName, String bladeId)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getLifecycleCompleted(testName, bladeId);
	}

	public void getFieldValues(String testName, String bladeId,
			String eventTypeLabel, String XFieldLabel, String YFieldLabel)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		// not used in ISAC
		// int start = (int) virtualDataSupplier.getCommonMinTime(testName);
		// int end = (int) virtualDataSupplier.getCommonMaxTime(testName);
		BladeEvent[] bladeEventList = null;
		try
		{
			bladeEventList = dataSupplier.getBladeEvents(testName, bladeId,
					eventTypeLabel, null, 0, -1);
			if (bladeEventList != null)
			{
				for (int i = 0; i < bladeEventList.length; i++)
				{
					System.out.println(bladeEventList[i]
							.getFieldValue(YFieldLabel));
				}
			}
		} catch (ClifException ce)
		{
		}
	}

	public List<BladeEvent> fieldFilter(String testName, String bladeId,
			String eventTypeLabel, String fieldLabel,
			String fieldFilterKeyword, String filterOperator)
	{
		return virtualDataSupplier.fieldFilter(testName, bladeId,
				eventTypeLabel, fieldLabel, fieldFilterKeyword, filterOperator);

	}

	/**
	 *Get the values for a test, bladeId, eventType
	 * 
	 * @param testName
	 * @param bladeId
	 * @param eventTypeLabel
	 * @return List<BladeEvent>
	 * @throws ClifException
	 */
	public List<BladeEvent> getBladeOneEventValues(String testName,
			String bladeId, String eventTypeLabel) throws ClifException
	{
		return virtualDataSupplier.getBladeOneEventValues(testName, bladeId,
				eventTypeLabel);
	}

	/**
	 * Get the values for a test, bladeId, eventType
	 * 
	 * @param testName
	 * @param bladeId
	 * @param eventTypeLabel
	 * @return Array<BladeEvent>
	 */
	public BladeEvent[] getBladeOneEventValuesArray(String testName,
			String bladeId, String eventTypeLabel)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		try
		{
			return dataSupplier.getBladeEvents(testName, bladeId,
					eventTypeLabel, null, 0, -1);
		} catch (Exception e)
		{
			// TODO: handle exception
		}
		return null;
	}

	public Map<String, List<BladeEvent>> getBladeAllEventsValues(
			String testName, String bladeId, String eventTypeLabel)
			throws ClifException
	{
		return virtualDataSupplier.getBladeAllEventsValues(testName, bladeId,
				eventTypeLabel);
	}

	public List<BladeEvent> getBladeAllEventsValuesFiltered(String testName,
			String bladeId, String eventTypeLabel,
			Map<String, Map<String, String>> filter, String linkingOp)
	{
		return virtualDataSupplier.getBladeAllEventsValuesFiltered(testName,
				bladeId, eventTypeLabel, filter, linkingOp);
	}

	public Object[][] getBladeEventsValues(BladeEvent[] bladeEventList,
			String XFieldLabel, String YFieldLabel)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getBladeEventsValues(bladeEventList, XFieldLabel,
				YFieldLabel);
	}

	public BladeDescriptor[] getBladeDescriptor(String testName,
			BladeFilter filter)
	{
		dataSupplier = (DataSupplier) analyzer
				.lookupFc(DataSupplier.DATA_SUPPLIER);
		return dataSupplier.getBladeDescriptor(testName, filter);
	}

	public void setSimpleDataset(SimpleDataset simpleDataset)
	{
		this.simpleDataset = simpleDataset;
	}

	public SimpleDataset getSimpleDataset()
	{
		return this.simpleDataset;
	}

	public void addSimpleDataset(SimpleDataset simpleDataset)
	{

	}

	public String getEventSelectedIDfromSD()
	{
		return this.simpleDataset.getEventSelectedIdFromSD();
	}

	public List<BladeEvent> aggregateBladesOneSameEvent(
			List<List<BladeEvent>> bladeList)
	{
		return virtualDataSupplier.aggregateBladesOneSameEvent(bladeList);
	}

	public List<BladeEvent> fieldFilter(List<BladeEvent> bladeEventList,
			String fieldAlone, String keywordAlone, String operatorAlone)
	{
		return virtualDataSupplier.fieldFilter(bladeEventList, fieldAlone,
				keywordAlone, operatorAlone);
	}

	/**
	 * Filter
	 * 
	 * @param bladeEventList
	 * @param filter
	 * @param linkingOp
	 * @return bladeEventlist
	 */
	public List<BladeEvent> getBladeAllEventsValuesFiltered(
			List<BladeEvent> bladeEventList,
			Map<String, Map<String, String>> filter, String linkingOp)
	{
		return virtualDataSupplier.getBladeAllEventsValuesFiltered(
				bladeEventList, filter, linkingOp);
	}

	/**
	 * Get bladeEvent list filtered by time
	 * 
	 * @param bladeEventList
	 * @param start
	 * @param end
	 * @return bladeEvent list
	 */
	public List<BladeEvent> getBladeByTime(List<BladeEvent> bladeEventList,
			int start, int end)
	{
		return virtualDataSupplier.getBladeByTime(bladeEventList, start, end);
	}

	/**
	 * @return new Statistics
	 */
	public String getNewStatistics()
	{
		return this.quickViewContainer.getNewStatistics();
	}

	/**
	 * @return pageIdAndDatasetIds map
	 */
	public Map<Integer, List<Integer>> getPageIdAndDatasetIds()
	{
		return this.quickViewContainer.getPageIdAndDatasetIds();
	}

	/**
	 * @return chartDataList map
	 */
	public Map<Integer, ChartData> getChartDataList()
	{
		return this.quickViewContainer.getChartDataList();
	}

	/**
	 * @return datasetId associated with graphOptions
	 */
	public Map<Integer, DatasetOptions> getDatasetIdAndGraphOptions()
	{
		return this.quickViewContainer.getDatasetIdAndGraphOptions();
	}
}
