/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2005, 2006 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF
 *
 * Contact: clif@ow2.org
 */
package org.ow2.isac.plugin.random;

import java.util.Hashtable;
import java.util.Map;
import org.ow2.clif.scenario.isac.plugin.ControlAction;
import org.ow2.clif.scenario.isac.plugin.DataProvider;
import org.ow2.clif.scenario.isac.plugin.SampleAction;
import org.ow2.clif.scenario.isac.plugin.TimerAction;
import org.ow2.clif.scenario.isac.plugin.SessionObjectAction;
import org.ow2.clif.scenario.isac.util.ParameterParser;
import org.ow2.clif.storage.api.ActionEvent;
import org.ow2.clif.util.Random;

/**
 * This plug-in provides samples and timers with random durations.
 * 
 * Its sample actions do not do anything, but take a certain time as explained below.
 * The following distribution laws are provided:
 * <ul>
 * <li>Dirac distribution, centered on the parameter <code>"value"</code>, which turns to be a constant;
 * <li>uniform distribution, on the interval delimited by the parameters <code>"min"</code> and <code>"max"</code>;
 * <li>Poisson distribution, with the parameter given by the parameter <code>"parameter"</code>
 * setting both the duration mean and deviation.
 * <li>Gaussian distribution, with minimum duration given by parameter <code>min</code>,
 * maximum duration given by parameter <code>max</code>,
 * duration mean given by parameter <code>mean</code>, and
 * duration deviation given by parameter <code>deviation</code>
 * <li>negative exponential distribution,
 * with minimum duration given by parameter <code>min</code>
 * and duration mean given by parameter <code>mean</code>
 * </ul> 
 * 
 * @author Emmanuel Varoquaux
 * @author Bruno Dillenseger
 */
public class SessionObject extends Random implements
	SessionObjectAction,
	SampleAction,
	TimerAction,
	ControlAction,
	DataProvider
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -592953548394386633L;

	private static class UnknownIdentifierError extends Error {
		private static final long serialVersionUID	= -859973658854852340L;
		private final int number;
		private String action;

		private UnknownIdentifierError(int number) {
			this.number = number;
		}

		private void setAction(String action) {
			this.action = action;
		}

		public String getMessage() {
			return "Fatal error in plug-in " + SessionObject.class.getName()
					+ ": unknown " + (action != null ? action + " " : "")
					+ "identifier: " + number;
		}
	}

	// ids for samples, timers, tests, controls and gets
	public static final int DIRAC_LAW = 0;
	public static final int UNIFORM_LAW = 1;
	public static final int POISSON_LAW = 2;
	public static final int GAUSSIAN_LAW = 3;
	public static final int NEG_EXPO_LAW = 4;
	public static final String STRING_GET = "string";
	public static final String INT_GET = "int";

	// parameters names
	public static final String MIN_ARG = "min";
	public static final String MAX_ARG = "max";
	public static final String MEAN_ARG = "mean";
	public static final String DEVIATION_ARG = "deviation";
	public static final String UNIT_ARG = "unit";
	public static final String VALUE_ARG = "value";
	public static final String STRING_ARG = "string";
	public static final String SUCCESSFUL_ARG = "successful"; 
	public static final String ENABLE_CBX = "enable";
	public static final String COMMENT_ARG = "comment";

	private int min=0, max=0, unit=1;
	private double mean=0, deviation=0;
	private int mode = DIRAC_LAW;

	private long random()
	{
		switch (mode)
		{
		case DIRAC_LAW:
			return min;
		case UNIFORM_LAW:
			return nextUniform(min, max);
		case POISSON_LAW:
			return nextPoisson(mean, unit);
		case GAUSSIAN_LAW:
			return nextGaussian(min, max, (int)mean, (int)deviation);
		case NEG_EXPO_LAW:
			return nextNegativeExponential(min, (int)mean);
		default:
			throw new Error("unknown random mode " + mode);
		}
	}

	//////////////////
	// constructors //
	//////////////////

	public SessionObject()
	{
	}

	public SessionObject(Hashtable params)
	{
		this();
	}

	///////////////////////////////////
	// SessionObjectAction interface //
	///////////////////////////////////

	public Object createNewSessionObject()
	{
		return new SessionObject();
	}

	public void reset() {}

	public void close() {}

	////////////////////////////
	// SampleAction interface //
	////////////////////////////

	public ActionEvent doSample(int number, Map params, ActionEvent report)
	{
		report.setDate(System.currentTimeMillis());
		long value = random();
		try
		{
			Thread.sleep(value);
		}
		catch (UnknownIdentifierError e)
		{
			e.setAction("sample");
			throw e;
		}
		catch (InterruptedException e)
		{
			Thread.currentThread().interrupt();
		}
		report.successful = ParameterParser.getCheckBox((String)params.get(SUCCESSFUL_ARG)).contains(ENABLE_CBX);
		report.duration = (int)(System.currentTimeMillis() - report.getDate());
		report.type = value + "ms dummy action";
		report.comment = (String)params.get(COMMENT_ARG);
		if (report.comment == null)
		{
			report.comment = "";
		}
		return report;
	}

	///////////////////////////
	// TimerAction interface //
	///////////////////////////

	public long doTimer(int number, Map params)
	{
		try
		{
			return random();
		}
		catch (UnknownIdentifierError e) {
			e.setAction("timer");
			throw e;
		}
	}

	////////////////////////////
	// DataProvider interface //
	////////////////////////////

	public String doGet(String arg)
	{
		long value = random();
		if (arg.equals(INT_GET))
		{
			return String.valueOf(value);
		}
		else
		{
			return nextStringBuilder((int)value).toString();
		}
	}

	/////////////////////////////
	// ControlAction interface //
	/////////////////////////////

	public void doControl(int id, Map params)
	{
		switch(id)
		{
		case DIRAC_LAW:
			min = Integer.parseInt((String)params.get(VALUE_ARG));
			mode = DIRAC_LAW;
			break;
		case UNIFORM_LAW:
			min = Integer.parseInt((String)params.get(MIN_ARG));
			max = Integer.parseInt((String)params.get(MAX_ARG));
			mode = UNIFORM_LAW;
			break;
		case POISSON_LAW:
			mean = Double.parseDouble((String)params.get("parameter"));
			unit = (params.containsKey(UNIT_ARG)
				? Integer.parseInt((String)params.get(UNIT_ARG))
				: 1);
			mode = POISSON_LAW;
			break;
		case GAUSSIAN_LAW:
			min = Integer.parseInt((String)params.get(MIN_ARG));
			max = Integer.parseInt((String)params.get(MAX_ARG));
			mean = Integer.parseInt((String)params.get(MEAN_ARG));
			deviation = Integer.parseInt((String)params.get(DEVIATION_ARG));
			mode = GAUSSIAN_LAW;
			break;
		case NEG_EXPO_LAW:
			min = Integer.parseInt((String)params.get(MIN_ARG));
			mean = Integer.parseInt((String)params.get(MEAN_ARG));
			mode = NEG_EXPO_LAW;
			break;
		default:
			throw new UnknownIdentifierError(id);
		}
	}
}
