/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2008 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */
package org.ow2.clif.loadtest.parser.clif;

import org.apache.commons.io.FileUtils;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.clif.loadtest.model.Injector;
import org.ow2.clif.loadtest.model.LoadTestReport;
import org.ow2.clif.loadtest.model.Measure;
import org.ow2.clif.loadtest.model.TestPlan;
import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.TestDescriptor;

import java.io.File;
import java.io.StringReader;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import static org.easymock.EasyMock.*;
import static org.junit.Assert.*;

/**
 * @author Julien Coste
 */
public class ClifParserTest {

    @Rule
    public TemporaryFolder testFolder = new TemporaryFolder();

    @Test
    public void testExtractProbeSimpleName() {
        ClifParser parser = new ClifParser("", null);

        assertEquals("jvm", parser.extractProbeSimpleName("org.ow2.clif.probe.jvm.Insert"));
        assertNull(parser.extractProbeSimpleName("org.ow2.clif.injector"));
    }

    @Test
    public void testAddStat() {
        ClifParser parser = new ClifParser("", null);

        ParsingContext ctx = new ParsingContext();
        initContext(ctx);

        Map<String, ActionStatInfo> statsByAction = new HashMap<String, ActionStatInfo>();
        ActionStatInfo actionStat1 = new ActionStatInfo(ctx);
        actionStat1.addStat(1, 1);

        statsByAction.put("action1", actionStat1);

        Map<String, ActionStatInfo> aggregatedStatsByAction = new HashMap<String, ActionStatInfo>();
        ActionStatInfo aggregatedActionStat1 = new ActionStatInfo(ctx);
        aggregatedActionStat1.addStat(1, 1);
        aggregatedActionStat1.addStat(2, 2);
        aggregatedStatsByAction.put("action1", aggregatedActionStat1);

        parser.statsByAction = statsByAction;
        parser.aggregatedStatsByAction = aggregatedStatsByAction;

        BladeEvent bladeEvent1 = createMock(BladeEvent.class);
        expect(bladeEvent1.getFieldValue("duration")).andReturn(new Integer(6));
        expect(bladeEvent1.getFieldValue("date")).andReturn(new Long(500));
        replay(bladeEvent1);
        parser.context = ctx;
        parser.addEventToStat("action1", bladeEvent1);

        verify(bladeEvent1);

        actionStat1.compute();
        aggregatedActionStat1.compute();

        assertEquals(2L, statsByAction.get("action1").getN());
        assertEquals(3.5d, statsByAction.get("action1").getMean(), 0.001d);

        assertEquals(3L, aggregatedStatsByAction.get("action1").getN());
        assertEquals(3d, aggregatedStatsByAction.get("action1").getMean(), 0.001d);

        BladeEvent bladeEvent2 = createMock(BladeEvent.class);
        expect(bladeEvent2.getFieldValue("duration")).andReturn(new Integer(4));
        expect(bladeEvent2.getFieldValue("date")).andReturn(new Long(500));
        replay(bladeEvent2);
        parser.addEventToStat("action2", bladeEvent2);

        verify(bladeEvent1);

        assertNotNull(statsByAction.get("action2"));
        statsByAction.get("action2").compute();

        assertEquals(1L, statsByAction.get("action2").getN());
        assertEquals(4d, statsByAction.get("action2").getMean(), 0.0001);

    }



    @Test
    public void testProperties() throws Exception {
        String vals = "HTTP GET-http://www.google.fr=Accueil Google";
        vals = vals.replaceAll(" ", "\\\\ ");
        vals = vals.replaceAll(":", "\\\\:");
        Properties props = new Properties();
        props.load(new StringReader(vals));

        assertEquals("Accueil Google", props.getProperty("HTTP GET-http://www.google.fr"));


    }

    @Test
    public void testExtractTestPlanName() {
        assertEquals("TestHttpFromAnt", ClifParser.extractTestPlanName("TestHttpFromAnt_2010-12-09_09h40m41"));
        assertEquals("TestHttpFromAnt_2010-12-09", ClifParser.extractTestPlanName("TestHttpFromAnt_2010-12-09"));
    }

    @Test
    public void testIsSuccessfulResult() {
        ClifParser parser = new ClifParser("", null);
        parser.addSuccessfulResultPattern("HTTP GET", ".*OK");

        BladeEvent bladeEvent1 = createMock(BladeEvent.class);
        expect(bladeEvent1.getFieldValue("result")).andReturn("HTTP/1.1 200 OK");
        expect(bladeEvent1.getFieldValue("action type")).andReturn("HTTP GET");
        BladeEvent bladeEvent2 = createMock(BladeEvent.class);
        expect(bladeEvent2.getFieldValue("result")).andReturn("HTTP/1.1 404 KO");
        expect(bladeEvent2.getFieldValue("action type")).andReturn("HTTP GET");
        BladeEvent bladeEvent3 = createMock(BladeEvent.class);
        expect(bladeEvent3.getFieldValue("result")).andReturn("HTTP/1.1 200 OK");
        expect(bladeEvent3.getFieldValue("action type")).andReturn("HTTP POST");

        replay(bladeEvent1, bladeEvent2, bladeEvent3);

        assertTrue(parser.isSuccessfulResult(bladeEvent1));
        assertFalse(parser.isSuccessfulResult(bladeEvent2));
        assertTrue(parser.isSuccessfulResult(bladeEvent3));

        verify(bladeEvent1, bladeEvent2, bladeEvent3);

    }

    @Test
    public void testEnableDataCleanup() {
        ClifParser parser = new ClifParser("", null);

        assertFalse(parser.context.dataCleanup);
        parser.enableDataCleanup(10, 50);
        assertTrue(parser.context.dataCleanup);
        assertEquals(10, parser.context.keepFactor, 0.0001);
        assertEquals(50, parser.context.keepPercentage, 0.0001);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testEnableDataCleanupBadArgs() {
        ClifParser parser = new ClifParser("", null);

        assertFalse(parser.context.dataCleanup);
        parser.enableDataCleanup(-10, 50);
        fail();
    }

    @Test(expected = IllegalArgumentException.class)
    public void testEnableDataCleanupBadArgs2() {
        ClifParser parser = new ClifParser("", null);

        assertFalse(parser.context.dataCleanup);
        parser.enableDataCleanup(10, 150);
        fail();
    }

    @Test(expected = IllegalArgumentException.class)
    public void testEnableDataCleanupBadArgs3() {
        ClifParser parser = new ClifParser("", null);

        assertFalse(parser.context.dataCleanup);
        parser.enableDataCleanup(10, -150);
        fail();
    }


    @Test
    public void testGetAlias() {
        ClifParser parser = new ClifParser("", null);

        String alias = parser.getAlias("action1");
        assertNotNull(alias);
        assertEquals("action1", alias);
        parser.addActionAliasPattern("Action", "action\\d");
        parser.addActionAliasPattern("Appel Google", ".*google.*");

        alias = parser.getAlias("action1");
        assertNotNull(alias);
        assertEquals("Action", alias);

        alias = parser.getAlias("maps.google.fr");
        assertNotNull(alias);
        assertEquals("Appel Google", alias);
    }

    @Test
    public void testParse() throws Exception {
        File reportDir = getFileResource("report");
        assertNotNull("report dir not found", reportDir);

        File tempOutput = testFolder.getRoot();
        System.out.println("Temp output =" + tempOutput);
        ClifParser parser = new ClifParser(reportDir.getAbsolutePath(), tempOutput);
        parser.addActionAliasPattern("Google", ".*google.*");
        parser.addActionAliasPattern("Yahoo", ".*yahoo.*");
        LoadTestReport report = parser.parse(System.out);

        assertNotNull("Null generated report", report);

        assertNotNull("No testplans associated to the report", report.getTestplans());
        assertEquals("Wrong number of testplan", 1, report.getTestplans().size());
        TestPlan tp = report.getTestplans().get(0);
        assertEquals("Bad testplan name", "TestHttpFromAnt", tp.getName());
        assertNotNull("No probe associated to the testplan", tp.getProbes());
        assertEquals("Wrong number of probes", 2, tp.getProbes().size());

        assertNotNull("No injector associated to the testplan", tp.getInjectors());
        assertEquals("Wrong number of injectors", 1, tp.getInjectors().size());
        Injector i = tp.getInjectors().get(0);
        assertEquals("Bad injector id", "2", i.getId());
        assertEquals("Bad injector name", "IsacRunner", i.getName());
        assertNotNull("No measure associated to the injector", i.getMeasures());
        assertEquals("Wrong number of measures", 2, i.getMeasures().size());

        Measure mGoogle = i.getMeasure("Google");
        assertNotNull("Google measure not found for IsacRunner", mGoogle);
        assertEquals("Bad countError for Google measure", 0, mGoogle.countErrors());
        assertEquals("Bad size for Google measure", 99, mGoogle.getSize());
        assertEquals("Bad avg for Google measure", 242, mGoogle.getAverage());
        assertEquals("Bad median for Google measure", 235, mGoogle.getMedian());
        assertEquals("Bad min for Google measure", 46, mGoogle.getMin());
        assertEquals("Bad max for Google measure", 719, mGoogle.getMax());

        Measure mYahoo = i.getMeasure("Yahoo");
        assertNotNull("Yahoo measure not found for IsacRunner", mYahoo);

        assertNotNull("No aggregated measures attached to the testplan", tp.getAggregatedMeasures());
        assertEquals("Wrong number of aggregated measures", 2, tp.getAggregatedMeasures().size());

        Measure amGoogle = tp.getAggregatedMeasure("Google");
        assertNotNull("Aggregated measure Google not found", amGoogle);

        Measure amYahoo = tp.getAggregatedMeasure("Yahoo");
        assertNotNull("Aggregated measure Yahoo not found", amYahoo);
    }


    @Test
    public void testParseWithDataCleanup() throws Exception {
        File reportDir = getFileResource("report");
        assertNotNull("report dir not found", reportDir);

        File tempOutput = testFolder.getRoot();
        System.out.println("Temp output =" + tempOutput);
        ClifParser parser = new ClifParser(reportDir.getAbsolutePath(), tempOutput);
        parser.addActionAliasPattern("Google", ".*google.*");
        parser.addActionAliasPattern("Yahoo", ".*yahoo.*");
        parser.enableDataCleanup(2, 10);
        LoadTestReport report = parser.parse(System.out);

        assertNotNull("Null generated report", report);

        assertNotNull("No testplans associated to the report", report.getTestplans());
        assertEquals("Wrong number of testplan", 1, report.getTestplans().size());
        TestPlan tp = report.getTestplans().get(0);
        assertEquals("Bad testplan name", "TestHttpFromAnt", tp.getName());
        assertNotNull("No probe associated to the testplan", tp.getProbes());
        assertEquals("Wrong number of probes", 2, tp.getProbes().size());

        assertNotNull("No injector associated to the testplan", tp.getInjectors());
        assertEquals("Wrong number of injectors", 1, tp.getInjectors().size());
        Injector i = tp.getInjectors().get(0);
        assertEquals("Bad injector id", "2", i.getId());
        assertEquals("Bad injector name", "IsacRunner", i.getName());
        assertNotNull("No measure associated to the injector", i.getMeasures());
        assertEquals("Wrong number of measures", 2, i.getMeasures().size());

        Measure mGoogle = i.getMeasure("Google");
        assertNotNull("Google measure not found for IsacRunner", mGoogle);
        assertEquals("Bad countError for Google measure", 0, mGoogle.countErrors());
        assertEquals("Bad size for Google measure", 96, mGoogle.getSize());
        assertEquals("Bad avg for Google measure", 227, mGoogle.getAverage());
        assertEquals("Bad median for Google measure", 234, mGoogle.getMedian());
        assertEquals("Bad min for Google measure", 46, mGoogle.getMin());
        assertEquals("Bad max for Google measure", 468, mGoogle.getMax());

        Measure mYahoo = i.getMeasure("Yahoo");
        assertNotNull("Yahoo measure not found for IsacRunner", mYahoo);

        assertNotNull("No aggregated measures attached to the testplan", tp.getAggregatedMeasures());
        assertEquals("Wrong number of aggregated measures", 2, tp.getAggregatedMeasures().size());

        Measure amGoogle = tp.getAggregatedMeasure("Google");
        assertNotNull("Aggregated measure Google not found", amGoogle);

        Measure amYahoo = tp.getAggregatedMeasure("Yahoo");
        assertNotNull("Aggregated measure Yahoo not found", amYahoo);
    }

    @Test
    public void testParseWithEverythingInError() throws Exception {
        File reportDir = getFileResource("report");
        assertNotNull("report dir not found", reportDir);

        File tempOutput = testFolder.getRoot();
        System.out.println("Temp output =" + tempOutput);
        ClifParser parser = new ClifParser(reportDir.getAbsolutePath(), tempOutput);
        parser.addActionAliasPattern("Google", ".*google.*");
        parser.addActionAliasPattern("Yahoo", ".*yahoo.*");
        parser.addSuccessfulResultPattern("HTTP GET", "OK");

        LoadTestReport report = parser.parse(System.out);

        assertNotNull("Null generated report", report);

        assertNotNull("No testplans associated to the report", report.getTestplans());
        assertEquals("Wrong number of testplan", 1, report.getTestplans().size());
        TestPlan tp = report.getTestplans().get(0);
        assertEquals("Bad testplan name", "TestHttpFromAnt", tp.getName());
        assertNotNull("No probe associated to the testplan", tp.getProbes());
        assertEquals("Wrong number of probes", 2, tp.getProbes().size());

        assertNotNull("No injector associated to the testplan", tp.getInjectors());
        assertEquals("Wrong number of injectors", 1, tp.getInjectors().size());
        Injector i = tp.getInjectors().get(0);
        assertEquals("Bad injector id", "2", i.getId());
        assertEquals("Bad injector name", "IsacRunner", i.getName());
        assertNotNull("No measure associated to the injector", i.getMeasures());
        assertEquals("Wrong number of measures", 2, i.getMeasures().size());

        Measure mGoogle = i.getMeasure("Google");
        assertNotNull("Google measure not found for IsacRunner", mGoogle);
        assertEquals("Bad countError for Google measure", 99, mGoogle.countErrors());
        assertEquals("Bad size for Google measure", 0, mGoogle.getSize());
        assertEquals("Bad avg for Google measure", 0, mGoogle.getAverage());
        assertEquals("Bad median for Google measure", 0, mGoogle.getMedian());
        assertEquals("Bad min for Google measure", 0, mGoogle.getMin());
        assertEquals("Bad max for Google measure", 0, mGoogle.getMax());

        Measure mYahoo = i.getMeasure("Yahoo");
        assertNotNull("Yahoo measure not found for IsacRunner", mYahoo);

        assertNotNull("No aggregated measures attached to the testplan", tp.getAggregatedMeasures());
        assertEquals("Wrong number of aggregated measures", 2, tp.getAggregatedMeasures().size());

        Measure amGoogle = tp.getAggregatedMeasure("Google");
        assertNotNull("Aggregated measure Google not found", amGoogle);

        Measure amYahoo = tp.getAggregatedMeasure("Yahoo");
        assertNotNull("Aggregated measure Yahoo not found", amYahoo);
    }

    // ###########################################################################
    // ## Utility test methods                                                  ##
    // ###########################################################################

    private File getFileResource( String filename )
    {
        File res = FileUtils.toFile( Thread.currentThread().getContextClassLoader().getResource( filename ) );
        assertNotNull( "Resource [" + filename + "] unvailable", res );
        return res;
    }

    private void initContext(ParsingContext ctx) {
        TestDescriptor tp = new TestDescriptor() {
            public String getName() {
                return "test";
            }

            public Date getDate() {
                return null;
            }
        };
        ctx.setTest(tp);
        BladeDescriptor bd = new BladeDescriptor() {
            public String getServerName() {
                return null;
            }

            public String getArgument() {
                return null;
            }

            public String getClassname() {
                return null;
            }

            public String getComment() {
                return null;
            }

            public String getId() {
                return "1";
            }

            public boolean isProbe() {
                return false;
            }

            public boolean isInjector() {
                return false;
            }

            public String[] getEventTypeLabels() {
                return new String[0];
            }
        };
        ctx.setBlade(bd);
        ctx.setEventType("eventType");
    }    

}
