
/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.egui.wizards;

import org.eclipse.birt.report.designer.core.model.SessionHandleAdapter;
import org.eclipse.birt.report.designer.internal.ui.util.ExceptionHandler;
import org.eclipse.birt.report.designer.ui.ReportPlugin;
import org.eclipse.birt.report.model.api.ModuleHandle;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.*;
import org.eclipse.ui.cheatsheets.OpenCheatSheetAction;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;
import org.ow2.clif.analyze.lib.egui.wizards.CLIFReportTemplateChoicePage.Template;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;


/**
 * Its role is to create a new CLIF report in the provided container.
 * If the container resource (a folder or a project) is selected in the
 * workspace when the wizard is opened, it will accept it as the
 * target container. The wizard creates one file with the extension "rptdesign".
 * Then, a next page allows to select CLIF executions. And another one
 * provides templates for the new report.
 * @author Grégory CALONNIER
 * @author Jordan BRUNIER
 */
public class CLIFReportNewWizard extends Wizard implements INewWizard {

	private static final String OPENING_FILE_FOR_EDITING = "Open file for editing...";
	private static final String FILE_EXTENSION = "rptdesign";
	private static final String TEMPLATECHOICEPAGE = "Template";
	
	private IConfigurationElement configElement;
	private ISelection selection;

	// Wizard pages
	private CLIFReportTemplateChoicePage templateChoicePage; 
	private CLIFReportNewWizardPage page;
		

	/**
	 * Constructor for CLIFReportNewWizard.
	 */
	public CLIFReportNewWizard() {
		super();
		setNeedsProgressMonitor(true);
		setWindowTitle("New CLIF Report");		
	}


	
	///////////////////////////
	// abstract class Wizard //
	///////////////////////////

	private boolean isClifAvaible(){
		try {
			Class.forName("org.ow2.clif.analyze.lib.egui.wizards.BirtController");
		} catch (NoClassDefFoundError e) {
			return false;
		} catch (ClassNotFoundException e) {
			return false;
		}
		return true;
	}

	/**
	 * Adding pages to the wizard.
	 */
	public void addPages() {
		page = new CLIFReportNewWizardPage(selection, CLIFReportNewWizardPage.TYPE_RPTDESIGN);
		addPage(page);
		templateChoicePage = new CLIFReportTemplateChoicePage(TEMPLATECHOICEPAGE);
		addPage(templateChoicePage);
	}

	/**
	 * This method is called when 'Finish' button is pressed in the wizard. We
	 * will create an operation and run it using wizard as execution context.
	 * @return boolean
	 */
	public boolean performFinish() {
		final String containerName = page.getContainerName();
		
		String fn = page.getFileName();
		final String fileName;
		if ( !fn.endsWith( "." + FILE_EXTENSION ) ) //$NON-NLS-1$
		{
			fileName = fn + "." + FILE_EXTENSION; //$NON-NLS-1$
		}
		else
		{
			fileName = fn;
		}

		final String cheatSheetId = templateChoicePage.getTemplate()
				.getCheatSheetId();
		final boolean showCheatSheet = templateChoicePage.getShowCheatSheet();

		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor)
					throws InvocationTargetException {
				try {
					doFinish(containerName, fileName, cheatSheetId,
							showCheatSheet, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};

		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException
					.getMessage());
			return false;
		}
		return true;
	}

	/**
	 * Returns whether this wizard could be finished without further user interaction.
	 * The result of this method is typically used by the wizard container to enable or disable the Finish button.
	 * @return true if the wizard could be finished, and false otherwise
	 */
	public boolean canFinish() {
		if(templateChoicePage.isCurrentPage()) {
			return true;
		} else {
			return false;
		}
	}
	
	
	////////////////////////////////
	// interface IWorkbenchWizard //
	////////////////////////////////
	
	/**
	 * We will accept the selection in the workbench to see if we can initialize
	 * from it.
	 * 
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
		if(!isClifAvaible()){
			MessageDialog.openError(getShell(),
					"CLIF Console Plug-in",
					"You need to install Birt to use Clif report");
			dispose();
		}
	}
	
	
	/////////////////////
	// private methods //
	/////////////////////
	
	/**
	 * The worker method. It will find the container, create the file if missing
	 * or just replace its contents, and open the editor on the newly created
	 * file. 
	 * @param containerName
	 * @param fileName
	 * @param cheatSheetId
	 * @param showCheatSheet
	 * @param monitor
	 */
	private void doFinish(String containerName, String fileName,
			String cheatSheetId, boolean showCheatSheet,
			IProgressMonitor monitor) throws CoreException {

		/* Monitor needed by IFile while creation */
		monitor.beginTask("Creating " + fileName, 2);
		
		/* Test if container exist */
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource resource = root.findMember(new Path(containerName));
		if (resource == null || !resource.exists() || !(resource instanceof IContainer)) {
			throwCoreException("Container \"" + containerName
					+ "\" does not exist.");
		}
		
		/* Create file if it does not exist */
		IContainer container = (IContainer) resource;
		final IFile file = container.getFile(new Path(fileName));
		try {
			InputStream stream = openContentStream();
			if (!file.exists()) {
                file.create(stream, true, monitor);
            }
			stream.close();
		} catch (IOException e) {
			MessageDialog.openError(getShell(),
                    "CLIF Console Plug-in",
            		"Cannot create a new file");
		}

		// open the editor
		final String cheatId = cheatSheetId;
		final boolean showCheat = showCheatSheet;

		monitor.worked(1);
		monitor.setTaskName(OPENING_FILE_FOR_EDITING);
		getShell().getDisplay().asyncExec(new Runnable() {

			public void run() {
				IWorkbench workbench = PlatformUI.getWorkbench();
				IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
				IWorkbenchPage page = window.getActivePage();

				// switch perspective to BIRT
				IPerspectiveRegistry reg = workbench.getPerspectiveRegistry();
				page.setPerspective(reg.findPerspectiveWithId("org.eclipse.birt.report.designer.ui.ReportPerspective"));
				

				try {
					IEditorPart editorPart = IDE.openEditor(page, file, true);
					ModuleHandle model = SessionHandleAdapter.getInstance()
							.getReportDesignHandle();
					if (ReportPlugin.getDefault().getEnableCommentPreference()) {
						model.setStringProperty(ModuleHandle.COMMENTS_PROP,
								ReportPlugin.getDefault()
										.getCommentPreference());
						model.save();
						editorPart.doSave(null);
					}

					BasicNewProjectResourceWizard
							.updatePerspective(configElement);
					if (showCheat && !cheatId.equals("")) {
						OpenCheatSheetAction action = null;
						action = new OpenCheatSheetAction(cheatId);
						action.run();
					}
				} catch (Exception e) {
					ExceptionHandler.handle(e);
				}
			}
		});
		monitor.worked(1);
	}

	/**
	 * We will initialize file contents with a sample text.
	 * @return InputStream
	 */
	private InputStream openContentStream() {
		Template selTemplate = templateChoicePage.getTemplate();
		String fullName = selTemplate.getReportFullName();
		InputStream streamFromPage = null;
		URL url = Platform.getBundle("org.eclipse.birt.resources").getResource(fullName);

		if (url != null) {
			try {
				streamFromPage = url.openStream();
			} catch (IOException e1) {
				// ignore.
			}
		} else {
			try {
				streamFromPage = new FileInputStream(fullName);
			} catch (FileNotFoundException e) {
			}
		}

		return streamFromPage;
	}

	/**
	 * 
	 * @param message
	 * @throws org.eclipse.core.runtime.CoreException
	 */
	private void throwCoreException(String message) throws CoreException {
		IStatus status = new Status(IStatus.ERROR,
				"org.ow2.clif.analyze.lib.CLIFReportPlugin",
				IStatus.OK, message, null);
		throw new CoreException(status);
	}
		
}