/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.egui.wizards;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;


/**
 * Provide a wizard page to inform filename of report or report template.
 * @author Grégory CALONNIER
 * @author Jordan BRUNIER
 */
public class CLIFReportNewWizardPage extends WizardPage {
	
	private static final String DESC_RPTDESIGN = "This wizard creates a new file with *.rptdesign extension that can be opened by BIRT editor.";
	private static final String DESC_RPTTEMPLATE = "This wizard creates a new file with *.rpttemplate extension that can be opened by BIRT editor.";
	private static final String EXTENSION_RPTDESIGN = ".rptdesign";
	private static final String EXTENSION_RPTTEMPLATE = ".rpttemplate";
	private static final String FILENAME_RPTDESIGN = "new_clifReport";
	private static final String FILENAME_RPTTEMPLATE = "new_clifReportTemplate";
	public static final String TYPE_RPTDESIGN = "rptdesign";
	public static final String TYPE_RPTTEMPLATE = "rpttemplate";
	
	
	private String descriptionValue;
	private String extensionValue;
	private String filenameValue;	
	
	private Text containerText;
	private Text fileText;
	
	private ISelection selection;
	

	/**
	 * Constructor for CLIFReportNewWizardPage.
	 * 
	 * @param selection
	 * @param fileType its CLIFReportNewWizardPage.TYPE_RPTDESIGN (=report) or CLIFReportNewWizardPage.TYPE_RPTTEMPLATE (=report template)
	 */
	public CLIFReportNewWizardPage(ISelection selection, String fileType) {
		super("wizardPage");
		setTitle("CLIF Report File");
		
		if(fileType.compareTo(TYPE_RPTDESIGN) == 0)
		{
			this.descriptionValue = DESC_RPTDESIGN;
			this.extensionValue = EXTENSION_RPTDESIGN;
			this.filenameValue = FILENAME_RPTDESIGN;
		}
		if(fileType.compareTo(TYPE_RPTTEMPLATE) == 0)
		{
			this.descriptionValue = DESC_RPTTEMPLATE;
			this.extensionValue = EXTENSION_RPTTEMPLATE;
			this.filenameValue = FILENAME_RPTTEMPLATE;
		}
		
		setDescription(this.descriptionValue);
		this.selection = selection;
	}

	
	///////////////////////////
	// interface IDialogPage //
	///////////////////////////
	
	/**
	 * Create graphical interface.
	 * @param parent the container
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 9;
		Label label = new Label(container, SWT.NULL);
		label.setText("&Container:");

		containerText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		containerText.setLayoutData(gd);
		containerText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});

		Button button = new Button(container, SWT.PUSH);
		button.setText("Browse...");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleBrowse();
			}
		});
		label = new Label(container, SWT.NULL);
		label.setText("&File name:");

		fileText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		fileText.setLayoutData(gd);
		fileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});
		initialize();
		dialogChanged();
		setControl(container);
	}

	
	/////////////////////
	// private methods //
	/////////////////////
	
	/**
	 * Tests if the current workbench selection is a suitable container to use.
	 */
	private void initialize() {
		if (selection != null && selection.isEmpty() == false
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				IContainer container;
				if (obj instanceof IContainer)
					container = (IContainer) obj;
				else
					container = ((IResource) obj).getParent();
				containerText.setText(container.getFullPath().toString());
			}
		}
		fileText.setText(this.filenameValue + this.extensionValue);
	}

	/**
	 * Uses the standard container selection dialog to choose the new value for
	 * the container field.
	 */
	private void handleBrowse() {
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(
				getShell(), ResourcesPlugin.getWorkspace().getRoot(), false,
				"Select new file container");
		if (dialog.open() == ContainerSelectionDialog.OK) {
			Object[] result = dialog.getResult();
			if (result.length == 1) {
				containerText.setText(((Path) result[0]).toString());
			}
		}
	}

	/**
	 * Ensures that both text fields are set.
	 */
	private void dialogChanged() {
		IResource container = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(new Path(getContainerName()));
		String fileName = getFileName();
		String containerName = getContainerName();

		if (containerName.length() == 0) {
			updateStatus("File container must be specified");
			return;
		}
		if (container == null
				|| (container.getType() & (IResource.PROJECT | IResource.FOLDER)) == 0) {
			updateStatus("File container must exist");
			return;
		}
		if (!container.isAccessible()) {
			updateStatus("Project must be writable");
			return;
		}
		if (fileName.length() == 0) {
			updateStatus("File name must be specified");
			return;
		}
		if (fileName.replace('\\', '/').indexOf('/', 1) > 0) {
			updateStatus("File name must be valid");
			return;
		}
		int dotLoc = fileName.lastIndexOf('.');
		if (dotLoc != -1) {
			String ext = fileName.substring(dotLoc + 1);
			if (ext.equalsIgnoreCase(this.extensionValue.substring(1)) == false) {
				updateStatus("File extension must be \"" + this.extensionValue + "\"");
				return;
			}
		}else {
            // If no extension, test with the .rptdesign extension
            fileName = fileName + this.extensionValue;
        }
				
		
		// Test if container exist
        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
        IResource resource = root.findMember(new Path(containerName));
        if (resource == null || !resource.exists() || !(resource instanceof IContainer)) {
            updateStatus("Invalid container");
            return;
        }		
        
        // Test if file already exist
        IContainer containerint = (IContainer) resource;
        IFile file = containerint.getFile(new Path(fileName));
        if (file.exists()) {
            updateStatus("File already exist");
            return;				
        }
		updateStatus(null);
	}

	/**
	 * Update error message.
	 * @param message
	 */
	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	
	////////////////////
	// public methods //
	////////////////////
	
	/**
	 * Provide the name of container.
	 * @return String name of container
	 */
	public String getContainerName() {
		return containerText.getText();
	}

	/**
	 * Provide the filename of the report or report template.
	 * @return String filename
	 */
	public String getFileName() {
		return fileText.getText();
	}
}