/*
* CLIF is a Load Injection Framework
* Copyright (C) 2005, 2010 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.console.lib.egui.monitor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.ow2.clif.console.lib.ClifDeployDefinition;
import org.ow2.clif.deploy.DeployDefinition;
import org.ow2.clif.storage.api.AlarmEvent;
import org.ow2.clif.supervisor.lib.AlarmObservation;
import org.ow2.clif.util.ThrowableHelper;


/**
 * Content of a CtabItems in TabItemFolderPrincipal.
 * Displays all alarms generated by blades.
 * 
 * @author Joan Chaumont
 * @author Bruno Dillenseger
 */
public class TableAlarmComposite extends Composite 
    implements SelectionListener, ICheckStateListener, IDoubleClickListener {
    
    private Map<String,DeployDefinition> testPlan;
    private ArrayList<AlarmObservation> alarms;
    
    private CheckboxTableViewer bladesTableviewer;
    private TableViewer alarmsTableviewer;
    
    private CCombo severityCombo;
    
    private TableColumn bladeCol;
    private TableColumn dateCol;
    private TableColumn timeCol;
    private TableColumn severityCol;
    private TableColumn messageCol;
    private TableColumn bladeSelCol;
    
    private static final String BLADE = "Id";
    private static final String CLASS = "Class";
    private static final String SERVER = "Server";
    private static final String DATE = "Date";
    private static final String TIME = "Time";
    private static final String SEVERITY = "Severity";
    private static final String MESSAGE = "Message";
    
    private boolean revert;
    private int comparator;
    private TableColumn bladeServerCol;
    private TableColumn bladeClassCol;
    
    private Map<String,Long> bladeBeginTime;
    private Map<String,Long> bladeSuspendedTime;
    
    private class BladeTableContentProvider implements IStructuredContentProvider {
        
        /**
         * Constructor.
         */
        public BladeTableContentProvider() {
            super();
        }
        
        /**
         * Returns an array of BladeOption objects
         * @param inputElement 
         * @return Object[]
         * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
         */
        public Object[] getElements(Object inputElement) {
            Object elts[] = new Object[((Map)inputElement).size()];
            Object keys[] = ((Map)inputElement).keySet().toArray();
            Object vals[] = ((Map)inputElement).values().toArray();
            
            for (int i = 0; i < elts.length; i++) {
                elts[i] = new Object[] {keys[i],vals[i]};
            }
            
            return elts;
        }
        
        /**
         * @see org.eclipse.jface.viewers.IContentProvider#dispose()
         */
        public void dispose() { }
        
        /**
         * @param viewer 
         * @param oldInput 
         * @param newInput 
         * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
         */
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) { }
        
    }
    
    private class BladeLabelProvider implements ITableLabelProvider, IColorProvider {
        
        private Map<String,Integer> rgbId;
        private Display d;
        
        /**
         * Constructor
         * @param d
         * @param serverColors 
         */
        public BladeLabelProvider(Display d, Map<String,Integer> serverColors){
            rgbId = new HashMap<String,Integer>();

            for (Map.Entry<String,Integer> entry : serverColors.entrySet())
            {
                for (String id : testPlan.keySet())
                {
                    if(testPlan.get(id).getServerName().equals(entry.getKey())){
                        rgbId.put(id, entry.getValue());
                    }
                }
            }
            
            this.d = d;
        }
        
        /**
         * Returns the column text
         * 
         * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
         */
        public String getColumnText(Object element, int columnIndex) {     
            String id = (String)((Object[])element)[0];
            ClifDeployDefinition def = (ClifDeployDefinition)((Object[])element)[1];
            
            switch(columnIndex) {
            case 0 :
                return id;
            case 1 :
                return def.getClassName();
            case 2 :
                return def.getServerName();
            }
            return null;
        }
        
        /**
         * Disposes any created resources
         * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
         */
        public void dispose() { }
        
        /**
         * Returns whether altering this property on this element will affect the label
         *
         * @param element the element
         * @param property the property
         * @return boolean
         * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
         */
        public boolean isLabelProperty(Object element, String property) {
            return false;
        }

        public Color getForeground(Object element) {
            String id = (String)((Object[])element)[0];
            
            return d.getSystemColor(((Integer)rgbId.get(id)).intValue());
        }

        public Color getBackground(Object element) {
            return null;
        }

        public Image getColumnImage(Object element, int columnIndex) {
            return null;
        }

        public void addListener(ILabelProviderListener listener) {}

        public void removeListener(ILabelProviderListener listener) {}
    }

    private class AlarmContentProvider implements IStructuredContentProvider {
        public Object[] getElements(Object inputElement) {
            try {
                ArrayList<Object> bladesId = new ArrayList<Object>();
                Object[] checked = bladesTableviewer.getCheckedElements();
                for (int i = 0; i < checked.length; i++) {
                    bladesId.add(((Object[])checked[i])[0]);
                }
                
                SortedSet<AlarmObservation> alarms = new TreeSet<AlarmObservation>((ArrayList<AlarmObservation>)inputElement);
                Iterator<AlarmObservation> iter = alarms.iterator();
                while(iter.hasNext()) {
                    AlarmObservation alarm = iter.next();
                    if(alarm.getSeverity() < severityCombo.getSelectionIndex()
                            || !bladesId.contains(alarm.getBladeId())) {
                        iter.remove();
                    }
                }
                if(revert) {
                    Object[] tmp = alarms.toArray();
                    Object[] reverted = new Object[tmp.length];
                    for (int i = 0; i < tmp.length; i++) {
                        reverted[i] = tmp[tmp.length-1-i];
                    }
                    return reverted;
                }
                
                return alarms.toArray();
            }catch (SWTException e) {
                e.printStackTrace(System.err);
            }
            return null;
        }

        public void dispose() {}

        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
        
    }
    
    private class AlarmLabelProvider implements ITableLabelProvider, IColorProvider {

        private Map<String,Integer> rgbId;
        private Display d;
        
        /**
         * Constructor
         * @param display
         * @param serverColors
         */
        public AlarmLabelProvider(Display display, Map<String,Integer> serverColors) {
            rgbId = new HashMap<String,Integer>();
            for (Map.Entry<String,Integer> entry : serverColors.entrySet())
            {
            	for (String id : testPlan.keySet())
            	{
                    if(testPlan.get(id).getServerName().equals(entry.getKey())){
                        rgbId.put(id, entry.getValue());
                    }
                }
            }
            
            this.d = display;
        }

        public Image getColumnImage(Object element, int columnIndex) {
            return null;
        }

        public String getColumnText(Object element, int columnIndex) {
            AlarmObservation alarm = ((AlarmObservation)element);
            
            long time = 0;
            if(bladeBeginTime.get(alarm.getBladeId()) != null) {
                time = alarm.getDate()-bladeBeginTime.get(alarm.getBladeId());
            }
            
            switch(columnIndex) {
            case 0 :
                return alarm.getBladeId();
            case 1 :
                return FormatTime.getDate(alarm.getDate());
            case 2 :
                return FormatTime.getTime(time);
            case 3 :
                return severityCombo.getItem(alarm.getSeverity());
            case 4 :
            {
                return ((AlarmObservation)element).getMessage();
            }
            }
            return null;
        }

        public void addListener(ILabelProviderListener listener) {}

        public void dispose() {}

        public boolean isLabelProperty(Object element, String property) {
            return false;
        }

        public void removeListener(ILabelProviderListener listener) {}

        public Color getForeground(Object element) {
            String id = ((AlarmObservation)element).getBladeId();
            
            return d.getSystemColor(((Integer)rgbId.get(id)).intValue());
        }

        public Color getBackground(Object element) {
            return null;
        }
        
    }
    

    public TableAlarmComposite(Composite parent, int style, 
            Map<String,DeployDefinition> selectedTestPlan, Map<String,Integer> serverColors) {
        super(parent, style);
        this.testPlan = selectedTestPlan;
        bladeBeginTime = new HashMap<String,Long>();
        bladeSuspendedTime = new HashMap<String,Long>();
        
        alarms = new ArrayList<AlarmObservation>();
        revert = false;
        comparator = AlarmObservation.COMPARE_TIME;
        
        GridLayout layoutm = new GridLayout(1, true);
        layoutm.verticalSpacing = 0;
        layoutm.marginHeight = 0;
        layoutm.marginWidth = 0;
        this.setLayout(layoutm);
        this.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        /* Main Composite */
        Composite pageItemContent = new Composite(this, SWT.NONE);
        GridLayout layout = new GridLayout(1, true);
        layout.verticalSpacing = 0;
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        pageItemContent.setLayout(layout);
        pageItemContent.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        /* Composite which contains comboTableC and the graph */ 
        final SashForm alarmTableC = new SashForm(pageItemContent, SWT.NONE);
        GridLayout layout2 = new GridLayout(2, false);
        layout2.verticalSpacing = 0;
        layout2.marginHeight = 0;
        layout2.marginWidth = 0;
        alarmTableC.setLayout(layout2);
        alarmTableC.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        // Composite which contains tableComposite and the Combo
        Composite comboTableC = new Composite(alarmTableC, SWT.BORDER); //SWT.BORDER
        GridLayout layout3 = new GridLayout(1, true);
        layout3.verticalSpacing = 0;
        layout3.marginHeight = 0;
        layout3.marginWidth = 0;
        comboTableC.setLayout(layout3);
        comboTableC.setLayoutData(new GridData(GridData.FILL_VERTICAL));
        
        /* Composite which contains table */
        Composite tableComposite = new Composite(comboTableC, SWT.NONE);
        GridLayout flayout = new GridLayout(1, true);
        flayout.marginHeight = 0;
        flayout.marginWidth = 0;
        tableComposite.setLayout(flayout);
        tableComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        /* Add the blades TableViewer */
        Table bladesTable = new Table(tableComposite, SWT.FULL_SELECTION | SWT.CHECK);
        bladesTable.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        /* Selection on this table column select or deselect all blades */
        bladeSelCol = new TableColumn(bladesTable, SWT.CENTER);
        bladeSelCol.setText(BLADE);
        bladeSelCol.setWidth(50);
        bladeSelCol.addSelectionListener(this);
        
        bladeClassCol = new TableColumn(bladesTable, SWT.CENTER);
        bladeClassCol.setText(CLASS);
        bladeClassCol.setWidth(80);
        
        bladeServerCol = new TableColumn(bladesTable, SWT.CENTER);
        bladeServerCol.setText(SERVER);
        bladeServerCol.setWidth(100);
        
        bladesTable.setHeaderVisible(true);
        bladesTable.setLinesVisible(true);
        
        bladesTableviewer = new CheckboxTableViewer(bladesTable);
        
        /* Content and Label provider */ 
        bladesTableviewer.setContentProvider(new BladeTableContentProvider());
        bladesTableviewer.setLabelProvider(new BladeLabelProvider(parent.getDisplay(), serverColors));
        bladesTableviewer.setInput(testPlan);
        
        bladesTableviewer.setAllChecked(true);
        bladesTableviewer.addCheckStateListener(this);
        
        /* Add the TableViewer */
        alarmsTableviewer = new TableViewer(alarmTableC,SWT.FULL_SELECTION | SWT.MULTI);
        Table alarmsTable = alarmsTableviewer.getTable();

        /* Blade column add listener for sort operation */
        bladeCol = new TableColumn(alarmsTable, SWT.CENTER);
        bladeCol.setText(BLADE);
        bladeCol.addSelectionListener(this);
        bladeCol.setWidth(60);
        
        /* Date column add listener for sort operation */
        dateCol = new TableColumn(alarmsTable, SWT.CENTER);
        dateCol.setText(DATE);
        dateCol.addSelectionListener(this);
        dateCol.setWidth(120);
        
        /* Time column add listener for sort operation */
        timeCol = new TableColumn(alarmsTable, SWT.CENTER);
        timeCol.setText(TIME);
        timeCol.addSelectionListener(this);
        timeCol.setWidth(120);
        
        /* Severity column add listener for sort operation */
        severityCol = new TableColumn(alarmsTable, SWT.CENTER);
        severityCol.setText(SEVERITY);
        severityCol.addSelectionListener(this);
        severityCol.setWidth(120);
        
        /* Message column add listener for sort operation */
        messageCol = new TableColumn(alarmsTable, SWT.LEFT);
        messageCol.setText(MESSAGE);
        messageCol.addSelectionListener(this);
        messageCol.setWidth(200);

        alarmsTable.setHeaderVisible(true);
        alarmsTable.setLinesVisible(true);
        
        /* Content and Label provider */ 
        alarmsTableviewer.setContentProvider(new AlarmContentProvider());
        alarmsTableviewer.setLabelProvider(new AlarmLabelProvider(parent.getDisplay(), serverColors));
        
        alarmsTableviewer.addDoubleClickListener(this);
        
        /* Combo severity : info, warning, debug, fatal */
        severityCombo = new CCombo(comboTableC, SWT.DROP_DOWN | SWT.READ_ONLY | SWT.BORDER);
        severityCombo.setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
        GridData gct = new GridData(GridData.FILL_HORIZONTAL);
        severityCombo.setLayoutData(gct);
        severityCombo.add("Info", AlarmEvent.INFO);
        severityCombo.add("Warning", AlarmEvent.WARNING);
        severityCombo.add("Error", AlarmEvent.ERROR);
        severityCombo.add("Fatal", AlarmEvent.FATAL);
        
        severityCombo.select(AlarmEvent.INFO);
        severityCombo.addSelectionListener(this);
        
        // adjusts weight of "comboTableC" and "pointsTable"
        int [] weights = {2,5};
        alarmTableC.setWeights(weights);
    }


    public void widgetSelected(SelectionEvent e) {
        Object source = e.getSource();

        if (source == severityCombo) {
            alarmsTableviewer.refresh();
        }
        
        else if (source == bladeSelCol) {
            bladesTableviewer.setAllChecked(!(bladesTableviewer.getCheckedElements().length
                    == bladesTableviewer.getTable().getItemCount()));
            alarmsTableviewer.refresh();
        }
        
        else if (source == bladeCol || source == dateCol || source == timeCol
                || source == severityCol || source == messageCol) {
            int oldComp = comparator;
            comparator = AlarmObservation.COMPARE_TIME;
            
            if (source == bladeCol) {
                comparator = AlarmObservation.COMPARE_ID;   
            }
            else if (source == severityCol) {
                comparator = AlarmObservation.COMPARE_SEVERITY;
            }
            else if (source == messageCol) {
                comparator = AlarmObservation.COMPARE_MESSAGE;
            }
            for (AlarmObservation al : alarms)
            {
                al.compareBy(comparator);
            }
            
            revert = (oldComp == comparator)?!revert:false;
            
            alarmsTableviewer.setInput(alarms);
        }
    }

    public void widgetDefaultSelected(SelectionEvent e) {}


    public void doubleClick(DoubleClickEvent event)
    {
        Table t = alarmsTableviewer.getTable();
        if(t.getSelectionCount() == 1)
        {
            final AlarmObservation alarm = (AlarmObservation)alarmsTableviewer.getElementAt(t.getSelectionIndex());
            try
            {
                alarmsTableviewer.getTable().getDisplay().asyncExec(
                	new Runnable() {
                		public void run() {
                        	if (!alarmsTableviewer.getTable().isDisposed())
                        	{
                        		String message = "Alarm from " + alarm.getBladeId() + ": " + alarm.getMessage();
                        		Object arg = alarm.getAlarm().getFieldValue(AlarmEvent.ARGUMENT_FIELD);
                        		if (arg instanceof Throwable)
                        		{
                        			message += "\n" + ThrowableHelper.toString((Throwable)arg);
                        		}
	                			MessageDialog.openInformation(
	                				alarmsTableviewer.getTable().getShell(),
	                				"Alarm from " + alarm.getBladeId(),
		                			message);
                        	}
                		}
                    });
            }
            catch (SWTException e)
            {
                e.printStackTrace(System.err);
            }
        }
    }

    public void checkStateChanged(CheckStateChangedEvent event) {
        alarmsTableviewer.refresh();
    }
    
    /**
     * Set begin time for a new started blade  
     * @param id
     */
    public void setBeginTime(String id) {
        if(bladeSuspendedTime.get(id) == null) {
            bladeBeginTime.put(id, new Long(System.currentTimeMillis()));
        }
        else {
            bladeBeginTime.put(id, new Long(System.currentTimeMillis() - bladeSuspendedTime.get(id)));
        }
    }
    
    /**
     * Suspend time for blade id 
     * @param id
     */
    public void setSuspendedTime(String id) {
        bladeSuspendedTime.put(id, new Long(System.currentTimeMillis()));
    }
    
    /**
     * Add an alarm
     * @param alarm
     */
    public void addAlarm(AlarmObservation alarm) {
        alarm.compareBy(comparator);
        alarms.add(alarm);
        alarmsTableviewer.setInput(alarms);
    }
    
    /**
     * Remove all alarms in tableviewer.
     */
    public void clearAll() {
        alarms.clear();
        alarmsTableviewer.setInput(alarms);
    }


    /**
     * Remove selected alarms
     */
    public void clear() {
        TableItem[] items = alarmsTableviewer.getTable().getSelection();
        for (TableItem i : items) {
            alarms.remove(i.getData());
        }
        alarmsTableviewer.setInput(alarms);
    }
}
